// VRML97/X3D file "" converted to C++ with white_dune

#include <stddef.h> // for NULL

class X3dSceneGraph;

class X3dNode {
public:
    X3dNode() {
        m_parent = NULL;
        m_protoRoot = NULL;
        m_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
    }
    X3dNode *m_parent;
    X3dNode *m_protoRoot;
    void *m_data;
    virtual int getType() const = 0;
};

typedef void (*X3dCallback)(X3dNode *node, void *dataptr);
typedef bool (*X3dProcessEventCallback)(X3dNode *node, const char *event, void *dataptr);


class X3dColorRGBA : public X3dNode {
public:
    float* color;
    int color_length;
    void* extra_data;

    int getType() const { return 40; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColorRGBA() {
        color = NULL;
        color_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColorRGBA::renderCallback = NULL;
X3dCallback X3dColorRGBA::treeRenderCallback = NULL;
X3dCallback X3dColorRGBA::doWithDataCallback = NULL;
X3dCallback X3dColorRGBA::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColorRGBA::processEventCallback = NULL;

int X3dColorRGBAType = 40;



class X3dVolumeEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    X3dNode* coord;
    float* direction;
    int direction_length;
    int* coordIndex;
    int coordIndex_length;
    bool internal;
    void* extra_data;

    int getType() const { return 325; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dVolumeEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coord = NULL;
        direction = NULL;
        direction_length = 0;
        coordIndex = NULL;
        coordIndex_length = 0;
        internal = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (coord)
                coord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (coord)
                coord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dVolumeEmitter::renderCallback = NULL;
X3dCallback X3dVolumeEmitter::treeRenderCallback = NULL;
X3dCallback X3dVolumeEmitter::doWithDataCallback = NULL;
X3dCallback X3dVolumeEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dVolumeEmitter::processEventCallback = NULL;

int X3dVolumeEmitterType = 325;



class X3dHAnimJoint : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    X3dNode** displacers;
    int displacers_length;
    float* limitOrientation;
    int limitOrientation_length;
    float* llimit;
    int llimit_length;
    const char* name;
    int* skinCoordIndex;
    int skinCoordIndex_length;
    float* skinCoordWeight;
    int skinCoordWeight_length;
    float* stiffness;
    int stiffness_length;
    float* ulimit;
    int ulimit_length;
    void* extra_data;

    int getType() const { return 107; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimJoint() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        displacers = NULL;
        displacers_length = 0;
        limitOrientation = NULL;
        limitOrientation_length = 0;
        llimit = NULL;
        llimit_length = 0;
        name = NULL;
        skinCoordIndex = NULL;
        skinCoordIndex_length = 0;
        skinCoordWeight = NULL;
        skinCoordWeight_length = 0;
        stiffness = NULL;
        stiffness_length = 0;
        ulimit = NULL;
        ulimit_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimJoint::renderCallback = NULL;
X3dCallback X3dHAnimJoint::treeRenderCallback = NULL;
X3dCallback X3dHAnimJoint::doWithDataCallback = NULL;
X3dCallback X3dHAnimJoint::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimJoint::processEventCallback = NULL;

int X3dHAnimJointType = 107;



class X3dWindPhysicsModel : public X3dNode {
public:
    float* direction;
    int direction_length;
    bool enabled;
    float gustiness;
    float speed;
    float turbulence;
    void* extra_data;

    int getType() const { return 330; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dWindPhysicsModel() {
        direction = NULL;
        direction_length = 0;
        enabled = false;
        gustiness = 0.0f;
        speed = 0.0f;
        turbulence = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dWindPhysicsModel::renderCallback = NULL;
X3dCallback X3dWindPhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dWindPhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dWindPhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dWindPhysicsModel::processEventCallback = NULL;

int X3dWindPhysicsModelType = 330;



class X3dTransform : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    void* extra_data;
    int num_route_source;
    X3dNode **route_sources;

    int getType() const { return 291; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTransform() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTransform::renderCallback = NULL;
X3dCallback X3dTransform::treeRenderCallback = NULL;
X3dCallback X3dTransform::doWithDataCallback = NULL;
X3dCallback X3dTransform::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTransform::processEventCallback = NULL;

int X3dTransformType = 291;



class X3dCylinderSensor : public X3dNode {
public:
    bool isActive;
    bool isOver;
    float* rotation_changed;
    int rotation_changed_length;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    bool autoOffset;
    float* axisRotation;
    int axisRotation_length;
    const char* description;
    float diskAngle;
    bool enabled;
    float maxAngle;
    float minAngle;
    float offset;
    void* extra_data;

    int getType() const { return 65; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCylinderSensor() {
        autoOffset = false;
        axisRotation = NULL;
        axisRotation_length = 0;
        description = NULL;
        diskAngle = 0.0f;
        enabled = false;
        maxAngle = 0.0f;
        minAngle = 0.0f;
        offset = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCylinderSensor::renderCallback = NULL;
X3dCallback X3dCylinderSensor::treeRenderCallback = NULL;
X3dCallback X3dCylinderSensor::doWithDataCallback = NULL;
X3dCallback X3dCylinderSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCylinderSensor::processEventCallback = NULL;

int X3dCylinderSensorType = 65;



class X3dCommonSurfaceShader : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 42; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCommonSurfaceShader() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCommonSurfaceShader::renderCallback = NULL;
X3dCallback X3dCommonSurfaceShader::treeRenderCallback = NULL;
X3dCallback X3dCommonSurfaceShader::doWithDataCallback = NULL;
X3dCallback X3dCommonSurfaceShader::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCommonSurfaceShader::processEventCallback = NULL;

int X3dCommonSurfaceShaderType = 42;



class X3dPlaneSensor : public X3dNode {
public:
    bool isOver;
    bool isActive;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    float* translation_changed;
    int translation_changed_length;
    bool autoOffset;
    float* axisRotation;
    int axisRotation_length;
    const char* description;
    bool enabled;
    float* maxPosition;
    int maxPosition_length;
    float* minPosition;
    int minPosition_length;
    float* offset;
    int offset_length;
    void* extra_data;

    int getType() const { return 197; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPlaneSensor() {
        autoOffset = false;
        axisRotation = NULL;
        axisRotation_length = 0;
        description = NULL;
        enabled = false;
        maxPosition = NULL;
        maxPosition_length = 0;
        minPosition = NULL;
        minPosition_length = 0;
        offset = NULL;
        offset_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPlaneSensor::renderCallback = NULL;
X3dCallback X3dPlaneSensor::treeRenderCallback = NULL;
X3dCallback X3dPlaneSensor::doWithDataCallback = NULL;
X3dCallback X3dPlaneSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPlaneSensor::processEventCallback = NULL;

int X3dPlaneSensorType = 197;



class X3dPositionInterpolator2D : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 213; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPositionInterpolator2D() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPositionInterpolator2D::renderCallback = NULL;
X3dCallback X3dPositionInterpolator2D::treeRenderCallback = NULL;
X3dCallback X3dPositionInterpolator2D::doWithDataCallback = NULL;
X3dCallback X3dPositionInterpolator2D::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPositionInterpolator2D::processEventCallback = NULL;

int X3dPositionInterpolator2DType = 213;



class X3dHAnimSite : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    const char* name;
    void* extra_data;

    int getType() const { return 110; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimSite() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        name = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimSite::renderCallback = NULL;
X3dCallback X3dHAnimSite::treeRenderCallback = NULL;
X3dCallback X3dHAnimSite::doWithDataCallback = NULL;
X3dCallback X3dHAnimSite::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimSite::processEventCallback = NULL;

int X3dHAnimSiteType = 110;



class X3dImageTexture : public X3dNode {
public:
    const char** url;
    int url_length;
    bool repeatS;
    bool repeatT;
    X3dNode* textureProperties;
    X3dNode** effects;
    int effects_length;
    const char* alphaChannel;
    void* extra_data;

    int getType() const { return 113; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dImageTexture() {
        url = NULL;
        url_length = 0;
        repeatS = false;
        repeatT = false;
        textureProperties = NULL;
        effects = NULL;
        effects_length = 0;
        alphaChannel = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dImageTexture::renderCallback = NULL;
X3dCallback X3dImageTexture::treeRenderCallback = NULL;
X3dCallback X3dImageTexture::doWithDataCallback = NULL;
X3dCallback X3dImageTexture::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dImageTexture::processEventCallback = NULL;

int X3dImageTextureType = 113;



class X3dScreenFontStyle : public X3dNode {
public:
    const char** family;
    int family_length;
    bool horizontal;
    const char** justify;
    int justify_length;
    const char* language;
    bool leftToRight;
    float pointSize;
    float spacing;
    const char* style;
    bool topToBottom;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScreenFontStyle() {
        family = NULL;
        family_length = 0;
        horizontal = false;
        justify = NULL;
        justify_length = 0;
        language = NULL;
        leftToRight = false;
        pointSize = 0.0f;
        spacing = 0.0f;
        style = NULL;
        topToBottom = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScreenFontStyle::renderCallback = NULL;
X3dCallback X3dScreenFontStyle::treeRenderCallback = NULL;
X3dCallback X3dScreenFontStyle::doWithDataCallback = NULL;
X3dCallback X3dScreenFontStyle::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScreenFontStyle::processEventCallback = NULL;

int X3dScreenFontStyleType = 234;



class X3dNavigationInfo : public X3dNode {
public:
    bool set_bind;
    double bindTime;
    bool isBound;
    bool transitionComplete;
    float* avatarSize;
    int avatarSize_length;
    bool headlight;
    float speed;
    double transitionTime;
    const char** transitionType;
    int transitionType_length;
    const char** type;
    int type_length;
    float visibilityLimit;
    const char* blendingSort;
    void* extra_data;

    int getType() const { return 167; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNavigationInfo() {
        avatarSize = NULL;
        avatarSize_length = 0;
        headlight = false;
        speed = 0.0f;
        transitionTime = 0;
        transitionType = NULL;
        transitionType_length = 0;
        type = NULL;
        type_length = 0;
        visibilityLimit = 0.0f;
        blendingSort = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNavigationInfo::renderCallback = NULL;
X3dCallback X3dNavigationInfo::treeRenderCallback = NULL;
X3dCallback X3dNavigationInfo::doWithDataCallback = NULL;
X3dCallback X3dNavigationInfo::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNavigationInfo::processEventCallback = NULL;

int X3dNavigationInfoType = 167;



class X3dRemoteSelectionGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 226; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dRemoteSelectionGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dRemoteSelectionGroup::renderCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::treeRenderCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::doWithDataCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dRemoteSelectionGroup::processEventCallback = NULL;

int X3dRemoteSelectionGroupType = 226;



class X3dCoordinateInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 57; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCoordinateInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCoordinateInterpolator::renderCallback = NULL;
X3dCallback X3dCoordinateInterpolator::treeRenderCallback = NULL;
X3dCallback X3dCoordinateInterpolator::doWithDataCallback = NULL;
X3dCallback X3dCoordinateInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCoordinateInterpolator::processEventCallback = NULL;

int X3dCoordinateInterpolatorType = 57;



class X3dDirectionalLight : public X3dNode {
public:
    float ambientIntensity;
    float* color;
    int color_length;
    float* direction;
    int direction_length;
    bool global;
    float intensity;
    bool on;
    float* projectionRectangle;
    int projectionRectangle_length;
    float* projectionLocation;
    int projectionLocation_length;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 67; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dDirectionalLight() {
        ambientIntensity = 0.0f;
        color = NULL;
        color_length = 0;
        direction = NULL;
        direction_length = 0;
        global = false;
        intensity = 0.0f;
        on = false;
        projectionRectangle = NULL;
        projectionRectangle_length = 0;
        projectionLocation = NULL;
        projectionLocation_length = 0;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dDirectionalLight::renderCallback = NULL;
X3dCallback X3dDirectionalLight::treeRenderCallback = NULL;
X3dCallback X3dDirectionalLight::doWithDataCallback = NULL;
X3dCallback X3dDirectionalLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dDirectionalLight::processEventCallback = NULL;

int X3dDirectionalLightType = 67;



class X3dCollision : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    double collideTime;
    bool isActive;
    X3dNode** children;
    int children_length;
    bool enabled;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* proxy;
    void* extra_data;

    int getType() const { return 31; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCollision() {
        children = NULL;
        children_length = 0;
        enabled = false;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        proxy = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (proxy)
                proxy->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (proxy)
                proxy->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCollision::renderCallback = NULL;
X3dCallback X3dCollision::treeRenderCallback = NULL;
X3dCallback X3dCollision::doWithDataCallback = NULL;
X3dCallback X3dCollision::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCollision::processEventCallback = NULL;

int X3dCollisionType = 31;



class X3dNurbsGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float tessellationScale;
    void* extra_data;

    int getType() const { return 173; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNurbsGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        tessellationScale = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNurbsGroup::renderCallback = NULL;
X3dCallback X3dNurbsGroup::treeRenderCallback = NULL;
X3dCallback X3dNurbsGroup::doWithDataCallback = NULL;
X3dCallback X3dNurbsGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNurbsGroup::processEventCallback = NULL;

int X3dNurbsGroupType = 173;



class X3dPointEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float* direction;
    int direction_length;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 198; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        direction = NULL;
        direction_length = 0;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointEmitter::renderCallback = NULL;
X3dCallback X3dPointEmitter::treeRenderCallback = NULL;
X3dCallback X3dPointEmitter::doWithDataCallback = NULL;
X3dCallback X3dPointEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointEmitter::processEventCallback = NULL;

int X3dPointEmitterType = 198;



class X3dProximitySensor : public X3dNode {
public:
    double enterTime;
    double exitTime;
    float* centerOfRotation_changed;
    int centerOfRotation_changed_length;
    bool isActive;
    float* position_changed;
    int position_changed_length;
    float* orientation_changed;
    int orientation_changed_length;
    float* center;
    int center_length;
    bool enabled;
    float* size;
    int size_length;
    void* extra_data;

    int getType() const { return 218; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dProximitySensor() {
        center = NULL;
        center_length = 0;
        enabled = false;
        size = NULL;
        size_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dProximitySensor::renderCallback = NULL;
X3dCallback X3dProximitySensor::treeRenderCallback = NULL;
X3dCallback X3dProximitySensor::doWithDataCallback = NULL;
X3dCallback X3dProximitySensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dProximitySensor::processEventCallback = NULL;

int X3dProximitySensorType = 218;



class X3dHAnimHumanoid : public X3dNode {
public:
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    const char** info;
    int info_length;
    float* jointBindingPositions;
    int jointBindingPositions_length;
    float* jointBindingRotations;
    int jointBindingRotations_length;
    float* jointBindingScales;
    int jointBindingScales_length;
    X3dNode** joints;
    int joints_length;
    int loa;
    X3dNode** motions;
    int motions_length;
    bool* motionsEnabled;
    int motionsEnabled_length;
    const char* name;
    X3dNode** segments;
    int segments_length;
    X3dNode** sites;
    int sites_length;
    X3dNode** skeleton;
    int skeleton_length;
    X3dNode** skin;
    int skin_length;
    X3dNode* skinCoord;
    X3dNode* skinNormal;
    const char* version;
    X3dNode** viewpoints;
    int viewpoints_length;
    void* extra_data;

    int getType() const { return 106; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimHumanoid() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        info = NULL;
        info_length = 0;
        jointBindingPositions = NULL;
        jointBindingPositions_length = 0;
        jointBindingRotations = NULL;
        jointBindingRotations_length = 0;
        jointBindingScales = NULL;
        jointBindingScales_length = 0;
        joints = NULL;
        joints_length = 0;
        loa = 0;
        motions = NULL;
        motions_length = 0;
        motionsEnabled = NULL;
        motionsEnabled_length = 0;
        name = NULL;
        segments = NULL;
        segments_length = 0;
        sites = NULL;
        sites_length = 0;
        skeleton = NULL;
        skeleton_length = 0;
        skin = NULL;
        skin_length = 0;
        skinCoord = NULL;
        skinNormal = NULL;
        version = NULL;
        viewpoints = NULL;
        viewpoints_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (joints)
                for (i = 0; i < joints_length; i++)
                    if (joints[i])
                        joints[i]->treeRender(dataptr);
            if (motions)
                for (i = 0; i < motions_length; i++)
                    if (motions[i])
                        motions[i]->treeRender(dataptr);
            if (segments)
                for (i = 0; i < segments_length; i++)
                    if (segments[i])
                        segments[i]->treeRender(dataptr);
            if (sites)
                for (i = 0; i < sites_length; i++)
                    if (sites[i])
                        sites[i]->treeRender(dataptr);
            if (skeleton)
                for (i = 0; i < skeleton_length; i++)
                    if (skeleton[i])
                        skeleton[i]->treeRender(dataptr);
            if (skin)
                for (i = 0; i < skin_length; i++)
                    if (skin[i])
                        skin[i]->treeRender(dataptr);
            if (skinCoord)
                skinCoord->treeRender(dataptr);
            if (skinNormal)
                skinNormal->treeRender(dataptr);
            if (viewpoints)
                for (i = 0; i < viewpoints_length; i++)
                    if (viewpoints[i])
                        viewpoints[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (joints)
                for (i = 0; i < joints_length; i++)
                    if (joints[i])
                        joints[i]->treeDoWithData(dataptr);
            if (motions)
                for (i = 0; i < motions_length; i++)
                    if (motions[i])
                        motions[i]->treeDoWithData(dataptr);
            if (segments)
                for (i = 0; i < segments_length; i++)
                    if (segments[i])
                        segments[i]->treeDoWithData(dataptr);
            if (sites)
                for (i = 0; i < sites_length; i++)
                    if (sites[i])
                        sites[i]->treeDoWithData(dataptr);
            if (skeleton)
                for (i = 0; i < skeleton_length; i++)
                    if (skeleton[i])
                        skeleton[i]->treeDoWithData(dataptr);
            if (skin)
                for (i = 0; i < skin_length; i++)
                    if (skin[i])
                        skin[i]->treeDoWithData(dataptr);
            if (skinCoord)
                skinCoord->treeDoWithData(dataptr);
            if (skinNormal)
                skinNormal->treeDoWithData(dataptr);
            if (viewpoints)
                for (i = 0; i < viewpoints_length; i++)
                    if (viewpoints[i])
                        viewpoints[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimHumanoid::renderCallback = NULL;
X3dCallback X3dHAnimHumanoid::treeRenderCallback = NULL;
X3dCallback X3dHAnimHumanoid::doWithDataCallback = NULL;
X3dCallback X3dHAnimHumanoid::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimHumanoid::processEventCallback = NULL;

int X3dHAnimHumanoidType = 106;



class X3dIndexedFaceSet : public X3dNode {
public:
    int* set_colorIndex;
    int set_colorIndex_length;
    int* set_coordIndex;
    int set_coordIndex_length;
    int* set_normalIndex;
    int set_normalIndex_length;
    int* set_texCoordIndex;
    int set_texCoordIndex_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* normal;
    X3dNode* texCoord;
    bool ccw;
    int* colorIndex;
    int colorIndex_length;
    bool colorPerVertex;
    bool convex;
    int* coordIndex;
    int coordIndex_length;
    float creaseAngle;
    int* normalIndex;
    int normalIndex_length;
    bool normalPerVertex;
    bool solid;
    int* texCoordIndex;
    int texCoordIndex_length;
    float* radianceTransfer;
    int radianceTransfer_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 116; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback createNormalsCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dIndexedFaceSet() {
        color = NULL;
        coord = NULL;
        normal = NULL;
        texCoord = NULL;
        ccw = false;
        colorIndex = NULL;
        colorIndex_length = 0;
        colorPerVertex = false;
        convex = false;
        coordIndex = NULL;
        coordIndex_length = 0;
        creaseAngle = 0.0f;
        normalIndex = NULL;
        normalIndex_length = 0;
        normalPerVertex = false;
        solid = false;
        texCoordIndex = NULL;
        texCoordIndex_length = 0;
        radianceTransfer = NULL;
        radianceTransfer_length = 0;
        attrib = NULL;
        attrib_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (normal)
                normal->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void createNormals(void *dataptr) {
        if (createNormalsCallback)
            createNormalsCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (normal)
                normal->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dIndexedFaceSet::renderCallback = NULL;
X3dCallback X3dIndexedFaceSet::treeRenderCallback = NULL;
X3dCallback X3dIndexedFaceSet::createNormalsCallback = NULL;
X3dCallback X3dIndexedFaceSet::doWithDataCallback = NULL;
X3dCallback X3dIndexedFaceSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dIndexedFaceSet::processEventCallback = NULL;

int X3dIndexedFaceSetType = 116;



class X3dSound : public X3dNode {
public:
    float* direction;
    int direction_length;
    float intensity;
    float* location;
    int location_length;
    float maxBack;
    float maxFront;
    float minBack;
    float minFront;
    float priority;
    X3dNode* source;
    bool spatialize;
    void* extra_data;

    int getType() const { return 251; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSound() {
        direction = NULL;
        direction_length = 0;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        maxBack = 0.0f;
        maxFront = 0.0f;
        minBack = 0.0f;
        minFront = 0.0f;
        priority = 0.0f;
        source = NULL;
        spatialize = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (source)
                source->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (source)
                source->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSound::renderCallback = NULL;
X3dCallback X3dSound::treeRenderCallback = NULL;
X3dCallback X3dSound::doWithDataCallback = NULL;
X3dCallback X3dSound::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSound::processEventCallback = NULL;

int X3dSoundType = 251;



class X3dTriangleSet : public X3dNode {
public:
    X3dNode* color;
    X3dNode* coord;
    X3dNode* normal;
    X3dNode* texCoord;
    bool ccw;
    bool colorPerVertex;
    bool normalPerVertex;
    bool solid;
    float* radianceTransfer;
    int radianceTransfer_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* fogCoord;
    void* extra_data;

    int getType() const { return 295; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTriangleSet() {
        color = NULL;
        coord = NULL;
        normal = NULL;
        texCoord = NULL;
        ccw = false;
        colorPerVertex = false;
        normalPerVertex = false;
        solid = false;
        radianceTransfer = NULL;
        radianceTransfer_length = 0;
        attrib = NULL;
        attrib_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (normal)
                normal->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (normal)
                normal->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTriangleSet::renderCallback = NULL;
X3dCallback X3dTriangleSet::treeRenderCallback = NULL;
X3dCallback X3dTriangleSet::doWithDataCallback = NULL;
X3dCallback X3dTriangleSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTriangleSet::processEventCallback = NULL;

int X3dTriangleSetType = 295;



class X3dFontStyle : public X3dNode {
public:
    const char** family;
    int family_length;
    bool horizontal;
    const char** justify;
    int justify_length;
    const char* language;
    bool leftToRight;
    float size;
    float spacing;
    const char* style;
    bool topToBottom;
    void* extra_data;

    int getType() const { return 89; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dFontStyle() {
        family = NULL;
        family_length = 0;
        horizontal = false;
        justify = NULL;
        justify_length = 0;
        language = NULL;
        leftToRight = false;
        size = 0.0f;
        spacing = 0.0f;
        style = NULL;
        topToBottom = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dFontStyle::renderCallback = NULL;
X3dCallback X3dFontStyle::treeRenderCallback = NULL;
X3dCallback X3dFontStyle::doWithDataCallback = NULL;
X3dCallback X3dFontStyle::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dFontStyle::processEventCallback = NULL;

int X3dFontStyleType = 89;



class X3dInline : public X3dNode {
public:
    bool load;
    const char** url;
    int url_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 122; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dInline() {
        load = false;
        url = NULL;
        url_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dInline::renderCallback = NULL;
X3dCallback X3dInline::treeRenderCallback = NULL;
X3dCallback X3dInline::doWithDataCallback = NULL;
X3dCallback X3dInline::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dInline::processEventCallback = NULL;

int X3dInlineType = 122;



class X3dMaterial : public X3dNode {
public:
    float ambientIntensity;
    float* diffuseColor;
    int diffuseColor_length;
    float* emissiveColor;
    int emissiveColor_length;
    float shininess;
    float* specularColor;
    int specularColor_length;
    float transparency;
    float mirror;
    float* reflSpecular;
    int reflSpecular_length;
    float* reflDiffuse;
    int reflDiffuse_length;
    float* transSpecular;
    int transSpecular_length;
    float* transDiffuse;
    int transDiffuse_length;
    float reflSpecularExp;
    float transSpecularExp;
    void* extra_data;

    int getType() const { return 148; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dMaterial() {
        ambientIntensity = 0.0f;
        diffuseColor = NULL;
        diffuseColor_length = 0;
        emissiveColor = NULL;
        emissiveColor_length = 0;
        shininess = 0.0f;
        specularColor = NULL;
        specularColor_length = 0;
        transparency = 0.0f;
        mirror = 0.0f;
        reflSpecular = NULL;
        reflSpecular_length = 0;
        reflDiffuse = NULL;
        reflDiffuse_length = 0;
        transSpecular = NULL;
        transSpecular_length = 0;
        transDiffuse = NULL;
        transDiffuse_length = 0;
        reflSpecularExp = 0.0f;
        transSpecularExp = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dMaterial::renderCallback = NULL;
X3dCallback X3dMaterial::treeRenderCallback = NULL;
X3dCallback X3dMaterial::doWithDataCallback = NULL;
X3dCallback X3dMaterial::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dMaterial::processEventCallback = NULL;

int X3dMaterialType = 148;



class X3dConeEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float angle;
    float* direction;
    int direction_length;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 48; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dConeEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        angle = 0.0f;
        direction = NULL;
        direction_length = 0;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dConeEmitter::renderCallback = NULL;
X3dCallback X3dConeEmitter::treeRenderCallback = NULL;
X3dCallback X3dConeEmitter::doWithDataCallback = NULL;
X3dCallback X3dConeEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dConeEmitter::processEventCallback = NULL;

int X3dConeEmitterType = 48;



class X3dOrientationInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 187; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dOrientationInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dOrientationInterpolator::renderCallback = NULL;
X3dCallback X3dOrientationInterpolator::treeRenderCallback = NULL;
X3dCallback X3dOrientationInterpolator::doWithDataCallback = NULL;
X3dCallback X3dOrientationInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dOrientationInterpolator::processEventCallback = NULL;

int X3dOrientationInterpolatorType = 187;



class X3dTimeSensor : public X3dNode {
public:
    double cycleTime;
    double elapsedTime;
    float fraction_changed;
    bool isActive;
    bool isPaused;
    double time;
    double cycleInterval;
    bool enabled;
    bool loop;
    double pauseTime;
    double resumeTime;
    double startTime;
    double stopTime;
    void* extra_data;

    int getType() const { return 286; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTimeSensor() {
        cycleInterval = 0;
        enabled = false;
        loop = false;
        pauseTime = 0;
        resumeTime = 0;
        startTime = 0;
        stopTime = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTimeSensor::renderCallback = NULL;
X3dCallback X3dTimeSensor::treeRenderCallback = NULL;
X3dCallback X3dTimeSensor::doWithDataCallback = NULL;
X3dCallback X3dTimeSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTimeSensor::processEventCallback = NULL;

int X3dTimeSensorType = 286;



class X3dIndexedLineSet : public X3dNode {
public:
    int* set_colorIndex;
    int set_colorIndex_length;
    int* set_coordIndex;
    int set_coordIndex_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    int* colorIndex;
    int colorIndex_length;
    bool colorPerVertex;
    int* coordIndex;
    int coordIndex_length;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 117; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dIndexedLineSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        colorIndex = NULL;
        colorIndex_length = 0;
        colorPerVertex = false;
        coordIndex = NULL;
        coordIndex_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dIndexedLineSet::renderCallback = NULL;
X3dCallback X3dIndexedLineSet::treeRenderCallback = NULL;
X3dCallback X3dIndexedLineSet::doWithDataCallback = NULL;
X3dCallback X3dIndexedLineSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dIndexedLineSet::processEventCallback = NULL;

int X3dIndexedLineSetType = 117;



class X3dMultiPart : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 163; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dMultiPart() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dMultiPart::renderCallback = NULL;
X3dCallback X3dMultiPart::treeRenderCallback = NULL;
X3dCallback X3dMultiPart::doWithDataCallback = NULL;
X3dCallback X3dMultiPart::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dMultiPart::processEventCallback = NULL;

int X3dMultiPartType = 163;



class X3dViewport : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* clipBoundary;
    int clipBoundary_length;
    void* extra_data;

    int getType() const { return 320; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dViewport() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        clipBoundary = NULL;
        clipBoundary_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dViewport::renderCallback = NULL;
X3dCallback X3dViewport::treeRenderCallback = NULL;
X3dCallback X3dViewport::doWithDataCallback = NULL;
X3dCallback X3dViewport::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dViewport::processEventCallback = NULL;

int X3dViewportType = 320;



class X3dAnchor : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    const char* description;
    const char** parameter;
    int parameter_length;
    const char** url;
    int url_length;
    void* extra_data;

    int getType() const { return 0; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAnchor() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        description = NULL;
        parameter = NULL;
        parameter_length = 0;
        url = NULL;
        url_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAnchor::renderCallback = NULL;
X3dCallback X3dAnchor::treeRenderCallback = NULL;
X3dCallback X3dAnchor::doWithDataCallback = NULL;
X3dCallback X3dAnchor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAnchor::processEventCallback = NULL;

int X3dAnchorType = 0;



class X3dColor : public X3dNode {
public:
    float* color;
    int color_length;
    void* extra_data;

    int getType() const { return 35; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColor() {
        color = NULL;
        color_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColor::renderCallback = NULL;
X3dCallback X3dColor::treeRenderCallback = NULL;
X3dCallback X3dColor::doWithDataCallback = NULL;
X3dCallback X3dColor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColor::processEventCallback = NULL;

int X3dColorType = 35;



class X3dShape : public X3dNode {
public:
    X3dNode* appearance;
    X3dNode* geometry;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* octreeTriangles;
    const char* shading;
    void* extra_data;

    int getType() const { return 242; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dShape() {
        appearance = NULL;
        geometry = NULL;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        octreeTriangles = NULL;
        shading = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (appearance)
                appearance->treeRender(dataptr);
            if (geometry)
                geometry->treeRender(dataptr);
            if (octreeTriangles)
                octreeTriangles->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (appearance)
                appearance->treeDoWithData(dataptr);
            if (geometry)
                geometry->treeDoWithData(dataptr);
            if (octreeTriangles)
                octreeTriangles->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dShape::renderCallback = NULL;
X3dCallback X3dShape::treeRenderCallback = NULL;
X3dCallback X3dShape::doWithDataCallback = NULL;
X3dCallback X3dShape::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dShape::processEventCallback = NULL;

int X3dShapeType = 242;



class X3dNormalInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 169; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNormalInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNormalInterpolator::renderCallback = NULL;
X3dCallback X3dNormalInterpolator::treeRenderCallback = NULL;
X3dCallback X3dNormalInterpolator::doWithDataCallback = NULL;
X3dCallback X3dNormalInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNormalInterpolator::processEventCallback = NULL;

int X3dNormalInterpolatorType = 169;



class X3dScalarInterpolator : public X3dNode {
public:
    float set_fraction;
    float value_changed;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 232; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScalarInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScalarInterpolator::renderCallback = NULL;
X3dCallback X3dScalarInterpolator::treeRenderCallback = NULL;
X3dCallback X3dScalarInterpolator::doWithDataCallback = NULL;
X3dCallback X3dScalarInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScalarInterpolator::processEventCallback = NULL;

int X3dScalarInterpolatorType = 232;



class X3dHAnimSegment : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* centerOfMass;
    int centerOfMass_length;
    X3dNode* coord;
    X3dNode** displacers;
    int displacers_length;
    float mass;
    float* momentsOfInertia;
    int momentsOfInertia_length;
    const char* name;
    void* extra_data;

    int getType() const { return 109; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimSegment() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        centerOfMass = NULL;
        centerOfMass_length = 0;
        coord = NULL;
        displacers = NULL;
        displacers_length = 0;
        mass = 0.0f;
        momentsOfInertia = NULL;
        momentsOfInertia_length = 0;
        name = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimSegment::renderCallback = NULL;
X3dCallback X3dHAnimSegment::treeRenderCallback = NULL;
X3dCallback X3dHAnimSegment::doWithDataCallback = NULL;
X3dCallback X3dHAnimSegment::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimSegment::processEventCallback = NULL;

int X3dHAnimSegmentType = 109;



class X3dTextureCoordinate : public X3dNode {
public:
    float* point;
    int point_length;
    void* extra_data;

    int getType() const { return 278; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTextureCoordinate() {
        point = NULL;
        point_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTextureCoordinate::renderCallback = NULL;
X3dCallback X3dTextureCoordinate::treeRenderCallback = NULL;
X3dCallback X3dTextureCoordinate::doWithDataCallback = NULL;
X3dCallback X3dTextureCoordinate::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTextureCoordinate::processEventCallback = NULL;

int X3dTextureCoordinateType = 278;



class X3dNormal : public X3dNode {
public:
    float* vector;
    int vector_length;
    void* extra_data;

    int getType() const { return 168; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNormal() {
        vector = NULL;
        vector_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNormal::renderCallback = NULL;
X3dCallback X3dNormal::treeRenderCallback = NULL;
X3dCallback X3dNormal::doWithDataCallback = NULL;
X3dCallback X3dNormal::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNormal::processEventCallback = NULL;

int X3dNormalType = 168;



class X3dBillboard : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* axisOfRotation;
    int axisOfRotation_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 9; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBillboard() {
        axisOfRotation = NULL;
        axisOfRotation_length = 0;
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBillboard::renderCallback = NULL;
X3dCallback X3dBillboard::treeRenderCallback = NULL;
X3dCallback X3dBillboard::doWithDataCallback = NULL;
X3dCallback X3dBillboard::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBillboard::processEventCallback = NULL;

int X3dBillboardType = 9;



class X3dSpotLight : public X3dNode {
public:
    float ambientIntensity;
    float* attenuation;
    int attenuation_length;
    float beamWidth;
    float* color;
    int color_length;
    float cutOffAngle;
    float* direction;
    int direction_length;
    bool global;
    float intensity;
    float* location;
    int location_length;
    bool on;
    float radius;
    float projectionAngle;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 259; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSpotLight() {
        ambientIntensity = 0.0f;
        attenuation = NULL;
        attenuation_length = 0;
        beamWidth = 0.0f;
        color = NULL;
        color_length = 0;
        cutOffAngle = 0.0f;
        direction = NULL;
        direction_length = 0;
        global = false;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        on = false;
        radius = 0.0f;
        projectionAngle = 0.0f;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSpotLight::renderCallback = NULL;
X3dCallback X3dSpotLight::treeRenderCallback = NULL;
X3dCallback X3dSpotLight::doWithDataCallback = NULL;
X3dCallback X3dSpotLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSpotLight::processEventCallback = NULL;

int X3dSpotLightType = 259;



class X3dColorInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 38; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColorInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColorInterpolator::renderCallback = NULL;
X3dCallback X3dColorInterpolator::treeRenderCallback = NULL;
X3dCallback X3dColorInterpolator::doWithDataCallback = NULL;
X3dCallback X3dColorInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColorInterpolator::processEventCallback = NULL;

int X3dColorInterpolatorType = 38;



class X3dExplosionEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 81; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dExplosionEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dExplosionEmitter::renderCallback = NULL;
X3dCallback X3dExplosionEmitter::treeRenderCallback = NULL;
X3dCallback X3dExplosionEmitter::doWithDataCallback = NULL;
X3dCallback X3dExplosionEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dExplosionEmitter::processEventCallback = NULL;

int X3dExplosionEmitterType = 81;



class X3dLineSet : public X3dNode {
public:
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* fogCoord;
    int* vertexCount;
    int vertexCount_length;
    void* extra_data;
    int glName_number;

    int getType() const { return 144; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dLineSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        fogCoord = NULL;
        vertexCount = NULL;
        vertexCount_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dLineSet::renderCallback = NULL;
X3dCallback X3dLineSet::treeRenderCallback = NULL;
X3dCallback X3dLineSet::doWithDataCallback = NULL;
X3dCallback X3dLineSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dLineSet::processEventCallback = NULL;

int X3dLineSetType = 144;



class X3dPixelTexture : public X3dNode {
public:
    int* image;
    int image_length;
    bool repeatS;
    bool repeatT;
    X3dNode* textureProperties;
    X3dNode** effects;
    int effects_length;
    const char* alphaChannel;
    void* extra_data;

    int getType() const { return 194; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPixelTexture() {
        image = NULL;
        image_length = 0;
        repeatS = false;
        repeatT = false;
        textureProperties = NULL;
        effects = NULL;
        effects_length = 0;
        alphaChannel = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPixelTexture::renderCallback = NULL;
X3dCallback X3dPixelTexture::treeRenderCallback = NULL;
X3dCallback X3dPixelTexture::doWithDataCallback = NULL;
X3dCallback X3dPixelTexture::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPixelTexture::processEventCallback = NULL;

int X3dPixelTextureType = 194;



class X3dSwitch : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode** children;
    int children_length;
    int whichChoice;
    void* extra_data;

    int getType() const { return 271; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSwitch() {
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        children = NULL;
        children_length = 0;
        whichChoice = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSwitch::renderCallback = NULL;
X3dCallback X3dSwitch::treeRenderCallback = NULL;
X3dCallback X3dSwitch::doWithDataCallback = NULL;
X3dCallback X3dSwitch::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSwitch::processEventCallback = NULL;

int X3dSwitchType = 271;



class X3dBlock : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 10; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBlock() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBlock::renderCallback = NULL;
X3dCallback X3dBlock::treeRenderCallback = NULL;
X3dCallback X3dBlock::doWithDataCallback = NULL;
X3dCallback X3dBlock::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBlock::processEventCallback = NULL;

int X3dBlockType = 10;



class X3dViewpoint : public X3dNode {
public:
    bool set_bind;
    double bindTime;
    bool isBound;
    float* centerOfRotation;
    int centerOfRotation_length;
    const char* description;
    float fieldOfView;
    bool jump;
    float* orientation;
    int orientation_length;
    bool retainUserOffsets;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 318; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dViewpoint() {
        centerOfRotation = NULL;
        centerOfRotation_length = 0;
        description = NULL;
        fieldOfView = 0.0f;
        jump = false;
        orientation = NULL;
        orientation_length = 0;
        retainUserOffsets = false;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dViewpoint::renderCallback = NULL;
X3dCallback X3dViewpoint::treeRenderCallback = NULL;
X3dCallback X3dViewpoint::doWithDataCallback = NULL;
X3dCallback X3dViewpoint::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dViewpoint::processEventCallback = NULL;

int X3dViewpointType = 318;



class X3dWorldInfo : public X3dNode {
public:
    const char** info;
    int info_length;
    const char* title;
    void* extra_data;

    int getType() const { return 332; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dWorldInfo() {
        info = NULL;
        info_length = 0;
        title = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dWorldInfo::renderCallback = NULL;
X3dCallback X3dWorldInfo::treeRenderCallback = NULL;
X3dCallback X3dWorldInfo::doWithDataCallback = NULL;
X3dCallback X3dWorldInfo::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dWorldInfo::processEventCallback = NULL;

int X3dWorldInfoType = 332;



class X3dAudioClip : public X3dNode {
public:
    double duration_changed;
    double elapsedTime;
    bool isActive;
    bool isPaused;
    const char* description;
    bool loop;
    double pauseTime;
    float pitch;
    double resumeTime;
    double startTime;
    double stopTime;
    const char** url;
    int url_length;
    void* extra_data;

    int getType() const { return 5; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAudioClip() {
        description = NULL;
        loop = false;
        pauseTime = 0;
        pitch = 0.0f;
        resumeTime = 0;
        startTime = 0;
        stopTime = 0;
        url = NULL;
        url_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAudioClip::renderCallback = NULL;
X3dCallback X3dAudioClip::treeRenderCallback = NULL;
X3dCallback X3dAudioClip::doWithDataCallback = NULL;
X3dCallback X3dAudioClip::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAudioClip::processEventCallback = NULL;

int X3dAudioClipType = 5;



class X3dPointSet : public X3dNode {
public:
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 202; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointSet::renderCallback = NULL;
X3dCallback X3dPointSet::treeRenderCallback = NULL;
X3dCallback X3dPointSet::doWithDataCallback = NULL;
X3dCallback X3dPointSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointSet::processEventCallback = NULL;

int X3dPointSetType = 202;



class X3dBoundedPhysicsModel : public X3dNode {
public:
    bool enabled;
    X3dNode* geometry;
    void* extra_data;

    int getType() const { return 18; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBoundedPhysicsModel() {
        enabled = false;
        geometry = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (geometry)
                geometry->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (geometry)
                geometry->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBoundedPhysicsModel::renderCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBoundedPhysicsModel::processEventCallback = NULL;

int X3dBoundedPhysicsModelType = 18;



class X3dGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 104; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dGroup::renderCallback = NULL;
X3dCallback X3dGroup::treeRenderCallback = NULL;
X3dCallback X3dGroup::doWithDataCallback = NULL;
X3dCallback X3dGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dGroup::processEventCallback = NULL;

int X3dGroupType = 104;



class X3dPolylineEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    X3dNode* coord;
    float* direction;
    int direction_length;
    int* coordIndex;
    int coordIndex_length;
    void* extra_data;

    int getType() const { return 204; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPolylineEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coord = NULL;
        direction = NULL;
        direction_length = 0;
        coordIndex = NULL;
        coordIndex_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (coord)
                coord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (coord)
                coord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPolylineEmitter::renderCallback = NULL;
X3dCallback X3dPolylineEmitter::treeRenderCallback = NULL;
X3dCallback X3dPolylineEmitter::doWithDataCallback = NULL;
X3dCallback X3dPolylineEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPolylineEmitter::processEventCallback = NULL;

int X3dPolylineEmitterType = 204;



class X3dPositionInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 212; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPositionInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPositionInterpolator::renderCallback = NULL;
X3dCallback X3dPositionInterpolator::treeRenderCallback = NULL;
X3dCallback X3dPositionInterpolator::doWithDataCallback = NULL;
X3dCallback X3dPositionInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPositionInterpolator::processEventCallback = NULL;

int X3dPositionInterpolatorType = 212;



class X3dCoordinate : public X3dNode {
public:
    float* point;
    int point_length;
    void* extra_data;

    int getType() const { return 52; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCoordinate() {
        point = NULL;
        point_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCoordinate::renderCallback = NULL;
X3dCallback X3dCoordinate::treeRenderCallback = NULL;
X3dCallback X3dCoordinate::doWithDataCallback = NULL;
X3dCallback X3dCoordinate::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCoordinate::processEventCallback = NULL;

int X3dCoordinateType = 52;



class X3dForcePhysicsModel : public X3dNode {
public:
    bool enabled;
    float* force;
    int force_length;
    void* extra_data;

    int getType() const { return 90; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dForcePhysicsModel() {
        enabled = false;
        force = NULL;
        force_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dForcePhysicsModel::renderCallback = NULL;
X3dCallback X3dForcePhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dForcePhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dForcePhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dForcePhysicsModel::processEventCallback = NULL;

int X3dForcePhysicsModelType = 90;



class X3dSphereSensor : public X3dNode {
public:
    bool isActive;
    bool isOver;
    float* rotation_changed;
    int rotation_changed_length;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    bool autoOffset;
    const char* description;
    bool enabled;
    float* offset;
    int offset_length;
    void* extra_data;

    int getType() const { return 255; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSphereSensor() {
        autoOffset = false;
        description = NULL;
        enabled = false;
        offset = NULL;
        offset_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSphereSensor::renderCallback = NULL;
X3dCallback X3dSphereSensor::treeRenderCallback = NULL;
X3dCallback X3dSphereSensor::doWithDataCallback = NULL;
X3dCallback X3dSphereSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSphereSensor::processEventCallback = NULL;

int X3dSphereSensorType = 255;



class X3dBackground : public X3dNode {
public:
    bool set_bind;
    bool isBound;
    float* groundAngle;
    int groundAngle_length;
    float* groundColor;
    int groundColor_length;
    const char** backUrl;
    int backUrl_length;
    const char** rightUrl;
    int rightUrl_length;
    const char** frontUrl;
    int frontUrl_length;
    const char** leftUrl;
    int leftUrl_length;
    const char** topUrl;
    int topUrl_length;
    const char** bottomUrl;
    int bottomUrl_length;
    float* skyAngle;
    int skyAngle_length;
    float transparency;
    float* skyColor;
    int skyColor_length;
    void* extra_data;

    int getType() const { return 6; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBackground() {
        groundAngle = NULL;
        groundAngle_length = 0;
        groundColor = NULL;
        groundColor_length = 0;
        backUrl = NULL;
        backUrl_length = 0;
        rightUrl = NULL;
        rightUrl_length = 0;
        frontUrl = NULL;
        frontUrl_length = 0;
        leftUrl = NULL;
        leftUrl_length = 0;
        topUrl = NULL;
        topUrl_length = 0;
        bottomUrl = NULL;
        bottomUrl_length = 0;
        skyAngle = NULL;
        skyAngle_length = 0;
        transparency = 0.0f;
        skyColor = NULL;
        skyColor_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBackground::renderCallback = NULL;
X3dCallback X3dBackground::treeRenderCallback = NULL;
X3dCallback X3dBackground::doWithDataCallback = NULL;
X3dCallback X3dBackground::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBackground::processEventCallback = NULL;

int X3dBackgroundType = 6;



class X3dSurfaceEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    int* coordIndex;
    int coordIndex_length;
    X3dNode* surface;
    void* extra_data;

    int getType() const { return 269; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSurfaceEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coordIndex = NULL;
        coordIndex_length = 0;
        surface = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (surface)
                surface->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (surface)
                surface->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSurfaceEmitter::renderCallback = NULL;
X3dCallback X3dSurfaceEmitter::treeRenderCallback = NULL;
X3dCallback X3dSurfaceEmitter::doWithDataCallback = NULL;
X3dCallback X3dSurfaceEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSurfaceEmitter::processEventCallback = NULL;

int X3dSurfaceEmitterType = 269;



class X3dTouchSensor : public X3dNode {
public:
    float* hitNormal_changed;
    int hitNormal_changed_length;
    float* hitPoint_changed;
    int hitPoint_changed_length;
    float* hitTexCoord_changed;
    int hitTexCoord_changed_length;
    bool isActive;
    bool isOver;
    double touchTime;
    const char* description;
    bool enabled;
    void* extra_data;

    int getType() const { return 290; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTouchSensor() {
        description = NULL;
        enabled = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTouchSensor::renderCallback = NULL;
X3dCallback X3dTouchSensor::treeRenderCallback = NULL;
X3dCallback X3dTouchSensor::doWithDataCallback = NULL;
X3dCallback X3dTouchSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTouchSensor::processEventCallback = NULL;

int X3dTouchSensorType = 290;



class X3dPointLight : public X3dNode {
public:
    float ambientIntensity;
    float* attenuation;
    int attenuation_length;
    float* color;
    int color_length;
    bool global;
    float intensity;
    float* location;
    int location_length;
    bool on;
    float radius;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 199; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointLight() {
        ambientIntensity = 0.0f;
        attenuation = NULL;
        attenuation_length = 0;
        color = NULL;
        color_length = 0;
        global = false;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        on = false;
        radius = 0.0f;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointLight::renderCallback = NULL;
X3dCallback X3dPointLight::treeRenderCallback = NULL;
X3dCallback X3dPointLight::doWithDataCallback = NULL;
X3dCallback X3dPointLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointLight::processEventCallback = NULL;

int X3dPointLightType = 199;



class X3dText : public X3dNode {
public:
    float* lineBounds;
    int lineBounds_length;
    float* origin;
    int origin_length;
    float* textBounds;
    int textBounds_length;
    X3dNode* fontStyle;
    float* length;
    int length_length;
    float maxExtent;
    const char** string;
    int string_length;
    bool solid;
    X3dNode* texCoord;
    void* extra_data;

    int getType() const { return 275; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dText() {
        fontStyle = NULL;
        length = NULL;
        length_length = 0;
        maxExtent = 0.0f;
        string = NULL;
        string_length = 0;
        solid = false;
        texCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (fontStyle)
                fontStyle->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (fontStyle)
                fontStyle->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dText::renderCallback = NULL;
X3dCallback X3dText::treeRenderCallback = NULL;
X3dCallback X3dText::doWithDataCallback = NULL;
X3dCallback X3dText::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dText::processEventCallback = NULL;

int X3dTextType = 275;



class X3dAppearance : public X3dNode {
public:
    X3dNode* material;
    X3dNode* texture;
    X3dNode* textureTransform;
    X3dNode* fillProperties;
    X3dNode* lineProperties;
    X3dNode* pointProperties;
    X3dNode** shaders;
    int shaders_length;
    X3dNode** receiveShadows;
    int receiveShadows_length;
    bool shadowCaster;
    X3dNode* normalMap;
    X3dNode* heightMap;
    float heightMapScale;
    X3dNode** effects;
    int effects_length;
    void* extra_data;

    int getType() const { return 1; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAppearance() {
        material = NULL;
        texture = NULL;
        textureTransform = NULL;
        fillProperties = NULL;
        lineProperties = NULL;
        pointProperties = NULL;
        shaders = NULL;
        shaders_length = 0;
        receiveShadows = NULL;
        receiveShadows_length = 0;
        shadowCaster = false;
        normalMap = NULL;
        heightMap = NULL;
        heightMapScale = 0.0f;
        effects = NULL;
        effects_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (material)
                material->treeRender(dataptr);
            if (texture)
                texture->treeRender(dataptr);
            if (textureTransform)
                textureTransform->treeRender(dataptr);
            if (fillProperties)
                fillProperties->treeRender(dataptr);
            if (lineProperties)
                lineProperties->treeRender(dataptr);
            if (pointProperties)
                pointProperties->treeRender(dataptr);
            if (shaders)
                for (i = 0; i < shaders_length; i++)
                    if (shaders[i])
                        shaders[i]->treeRender(dataptr);
            if (receiveShadows)
                for (i = 0; i < receiveShadows_length; i++)
                    if (receiveShadows[i])
                        receiveShadows[i]->treeRender(dataptr);
            if (normalMap)
                normalMap->treeRender(dataptr);
            if (heightMap)
                heightMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (material)
                material->treeDoWithData(dataptr);
            if (texture)
                texture->treeDoWithData(dataptr);
            if (textureTransform)
                textureTransform->treeDoWithData(dataptr);
            if (fillProperties)
                fillProperties->treeDoWithData(dataptr);
            if (lineProperties)
                lineProperties->treeDoWithData(dataptr);
            if (pointProperties)
                pointProperties->treeDoWithData(dataptr);
            if (shaders)
                for (i = 0; i < shaders_length; i++)
                    if (shaders[i])
                        shaders[i]->treeDoWithData(dataptr);
            if (receiveShadows)
                for (i = 0; i < receiveShadows_length; i++)
                    if (receiveShadows[i])
                        receiveShadows[i]->treeDoWithData(dataptr);
            if (normalMap)
                normalMap->treeDoWithData(dataptr);
            if (heightMap)
                heightMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAppearance::renderCallback = NULL;
X3dCallback X3dAppearance::treeRenderCallback = NULL;
X3dCallback X3dAppearance::doWithDataCallback = NULL;
X3dCallback X3dAppearance::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAppearance::processEventCallback = NULL;

int X3dAppearanceType = 1;



class X3dParticleSystem : public X3dNode {
public:
    X3dNode* appearance;
    bool createParticles;
    X3dNode* geometry;
    bool enabled;
    float lifetimeVariation;
    int maxParticles;
    float particleLifetime;
    float* particleSize;
    int particleSize_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* colorRamp;
    float* colorKey;
    int colorKey_length;
    X3dNode* emitter;
    const char* geometryType;
    X3dNode** physics;
    int physics_length;
    X3dNode* texCoordRamp;
    float* texCoordKey;
    int texCoordKey_length;
    void* extra_data;

    int getType() const { return 192; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dParticleSystem() {
        appearance = NULL;
        createParticles = false;
        geometry = NULL;
        enabled = false;
        lifetimeVariation = 0.0f;
        maxParticles = 0;
        particleLifetime = 0.0f;
        particleSize = NULL;
        particleSize_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        colorRamp = NULL;
        colorKey = NULL;
        colorKey_length = 0;
        emitter = NULL;
        geometryType = NULL;
        physics = NULL;
        physics_length = 0;
        texCoordRamp = NULL;
        texCoordKey = NULL;
        texCoordKey_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (appearance)
                appearance->treeRender(dataptr);
            if (geometry)
                geometry->treeRender(dataptr);
            if (colorRamp)
                colorRamp->treeRender(dataptr);
            if (emitter)
                emitter->treeRender(dataptr);
            if (physics)
                for (i = 0; i < physics_length; i++)
                    if (physics[i])
                        physics[i]->treeRender(dataptr);
            if (texCoordRamp)
                texCoordRamp->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (appearance)
                appearance->treeDoWithData(dataptr);
            if (geometry)
                geometry->treeDoWithData(dataptr);
            if (colorRamp)
                colorRamp->treeDoWithData(dataptr);
            if (emitter)
                emitter->treeDoWithData(dataptr);
            if (physics)
                for (i = 0; i < physics_length; i++)
                    if (physics[i])
                        physics[i]->treeDoWithData(dataptr);
            if (texCoordRamp)
                texCoordRamp->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dParticleSystem::renderCallback = NULL;
X3dCallback X3dParticleSystem::treeRenderCallback = NULL;
X3dCallback X3dParticleSystem::doWithDataCallback = NULL;
X3dCallback X3dParticleSystem::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dParticleSystem::processEventCallback = NULL;

int X3dParticleSystemType = 192;



class X3dTextureTransform : public X3dNode {
public:
    float* center;
    int center_length;
    float rotation;
    float* scale;
    int scale_length;
    float* translation;
    int translation_length;
    void* extra_data;

    int getType() const { return 283; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTextureTransform() {
        center = NULL;
        center_length = 0;
        rotation = 0.0f;
        scale = NULL;
        scale_length = 0;
        translation = NULL;
        translation_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTextureTransform::renderCallback = NULL;
X3dCallback X3dTextureTransform::treeRenderCallback = NULL;
X3dCallback X3dTextureTransform::doWithDataCallback = NULL;
X3dCallback X3dTextureTransform::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTextureTransform::processEventCallback = NULL;

int X3dTextureTransformType = 283;



class X3dScript_BOX1_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_HIT_CHECK::processEventCallback = NULL;

int X3dScriptType = 236;



class X3dScript_BOX1_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX1_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_STEER : public X3dNode {
public:
    float* wall_xy_in;
    int wall_xy_in_length;
    float xvalue;
    float yvalue;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_STEER() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_STEER::renderCallback = NULL;
X3dCallback X3dScript_STEER::treeRenderCallback = NULL;
X3dCallback X3dScript_STEER::doWithDataCallback = NULL;
X3dCallback X3dScript_STEER::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_STEER::processEventCallback = NULL;



class X3dScript_ROT_TRANS1 : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* rotation_in;
    int rotation_in_length;
    const char** debug;
    int debug_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* rotation;
    int rotation_length;
    float* quaternion1;
    int quaternion1_length;
    float* quaternion2;
    int quaternion2_length;
    float* quaternion3;
    int quaternion3_length;
    float* quaternion4;
    int quaternion4_length;
    float leng;
    float winkelkorrektur;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROT_TRANS1() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rotation = NULL;
        rotation_length = 0;
        quaternion1 = NULL;
        quaternion1_length = 0;
        quaternion2 = NULL;
        quaternion2_length = 0;
        quaternion3 = NULL;
        quaternion3_length = 0;
        quaternion4 = NULL;
        quaternion4_length = 0;
        leng = 0.0f;
        winkelkorrektur = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROT_TRANS1::renderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::treeRenderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::doWithDataCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROT_TRANS1::processEventCallback = NULL;



class X3dScript_ROT_TRANS2 : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* rotation_in;
    int rotation_in_length;
    const char** debug;
    int debug_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* rotation;
    int rotation_length;
    float* quaternion1;
    int quaternion1_length;
    float* quaternion2;
    int quaternion2_length;
    float* quaternion3;
    int quaternion3_length;
    float* quaternion4;
    int quaternion4_length;
    float leng;
    float winkelkorrektur;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROT_TRANS2() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rotation = NULL;
        rotation_length = 0;
        quaternion1 = NULL;
        quaternion1_length = 0;
        quaternion2 = NULL;
        quaternion2_length = 0;
        quaternion3 = NULL;
        quaternion3_length = 0;
        quaternion4 = NULL;
        quaternion4_length = 0;
        leng = 0.0f;
        winkelkorrektur = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROT_TRANS2::renderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::treeRenderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::doWithDataCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROT_TRANS2::processEventCallback = NULL;



class X3dScript_WALL_FIRE : public X3dNode {
public:
    bool isactive_in;
    bool fire;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    bool old_isactive;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_WALL_FIRE() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        old_isactive = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_WALL_FIRE::renderCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::treeRenderCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::doWithDataCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_WALL_FIRE::processEventCallback = NULL;



class X3dScript_ROLL_TURN : public X3dNode {
public:
    float xvalue_in;
    int rock_and_roll_flag_in;
    float* rotation_out;
    int rotation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int rock_and_roll_flag;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROLL_TURN() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rock_and_roll_flag = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROLL_TURN::renderCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::treeRenderCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::doWithDataCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROLL_TURN::processEventCallback = NULL;



class X3dScript_UP_DOWN_TURN : public X3dNode {
public:
    float yvalue_in;
    float* rotation_out;
    int rotation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_UP_DOWN_TURN() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_UP_DOWN_TURN::renderCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::treeRenderCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::doWithDataCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_UP_DOWN_TURN::processEventCallback = NULL;



class X3dScript_FLY_FORWARD : public X3dNode {
public:
    double time_in;
    float* translation_out;
    int translation_out_length;
    double time_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_FLY_FORWARD() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_FLY_FORWARD::renderCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::treeRenderCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::doWithDataCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_FLY_FORWARD::processEventCallback = NULL;



class X3dScript_FLY_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    double time_in;
    double enter_time;
    double exit_time;
    bool fire_in;
    float* translation_out;
    int translation_out_length;
    float* top_translation_out;
    int top_translation_out_length;
    int bullet_choice_out;
    float* bullet_translation_out;
    int bullet_translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* top_translation;
    int top_translation_length;
    float* translation_old;
    int translation_old_length;
    float* bullet_translation_start;
    int bullet_translation_start_length;
    float* bullet_direction;
    int bullet_direction_length;
    float bulletway_length;
    double old_time;
    float factor;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_FLY_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        top_translation = NULL;
        top_translation_length = 0;
        translation_old = NULL;
        translation_old_length = 0;
        bullet_translation_start = NULL;
        bullet_translation_start_length = 0;
        bullet_direction = NULL;
        bullet_direction_length = 0;
        bulletway_length = 0.0f;
        old_time = 0;
        factor = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_FLY_ON::renderCallback = NULL;
X3dCallback X3dScript_FLY_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_FLY_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_FLY_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_FLY_ON::processEventCallback = NULL;



class X3dScript_THING_SCRIPT : public X3dNode {
public:
    bool isover_in;
    bool fire_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_THING_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_THING_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_THING_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX2_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX2_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX2_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX3_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX3_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX3_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX4_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX4_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX4_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX5_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX5_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX5_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_LOAD_NEW_WORLD : public X3dNode {
public:
    double starttime_in;
    int box1_in;
    int box2_in;
    int box3_in;
    int box4_in;
    int box5_in;
    double starttime_out;
    int plane_choice;
    int choice_out;
    int victory_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int* box_in;
    int box_in_length;
    const char** your_url;
    int your_url_length;
    const char** param;
    int param_length;
    int deleted;
    int waitticks;
    bool victory;
    void* extra_data;

    int getType() const { return 236; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_LOAD_NEW_WORLD() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        box_in = NULL;
        box_in_length = 0;
        your_url = NULL;
        your_url_length = 0;
        param = NULL;
        param_length = 0;
        deleted = 0;
        waitticks = 0;
        victory = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_LOAD_NEW_WORLD::renderCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::treeRenderCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::doWithDataCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_LOAD_NEW_WORLD::processEventCallback = NULL;




bool X3dScript_BOX1_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_HIT_CHECK *self = (X3dScript_BOX1_HIT_CHECK *)node;
   static bool BOX1_HIT_CHECK_init = false;
   if (!BOX1_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX1_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX1_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_MOVE_ON *self = (X3dScript_BOX1_MOVE_ON *)node;
   static bool BOX1_MOVE_ON_init = false;
   if (!BOX1_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX1_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX1_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_HIDE_SCRIPT *self = (X3dScript_BOX1_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_STEERSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_STEER *self = (X3dScript_STEER *)node;
   self->xvalue = -self->wall_xy_in[0] / 1.5;
   self->yvalue = self->wall_xy_in[1] / 1.5;
    return true;
}

bool X3dScript_ROT_TRANS1SendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;
   static bool ROT_TRANS1_init = false;
   if (!ROT_TRANS1_init) {
       self->translation_out[0] = 0;
       self->translation_out[1] = 0;
       self->translation_out[2] = 0;
       ROT_TRANS1_init = true;
   }         

// account quaternion multiplication a b
   extern void quaternionMult(float *c, float *a, float *b);
// make rotation to quaternion with norm 1 
   extern void norm_quaternion_from_rotation(float *quad, float *rot); 

   self->rotation[0] = self->rotation_in[0];
   self->rotation[1] = self->rotation_in[1];
   self->rotation[2] = self->rotation_in[2];
   self->rotation[3] = self->rotation_in[3];
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);
   self->quaternion2[0] = self->translation_in[0];
   self->quaternion2[1] = self->translation_in[1];
   self->quaternion2[2] = self->translation_in[2];
   self->quaternion2[3] = 0;
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);
   self->quaternion4[0] = -self->quaternion1[0];
   self->quaternion4[1] = -self->quaternion1[1];
   self->quaternion4[2] = -self->quaternion1[2];
   self->quaternion4[3] = self->quaternion1[3];
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);
   self->translation_out[0] = self->quaternion2[0];
   self->translation_out[1] = self->quaternion2[1];
   self->translation_out[2] = self->quaternion2[2];
    return true;
}

bool X3dScript_ROT_TRANS2SendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;
   static bool ROT_TRANS1_init = false;
   if (!ROT_TRANS1_init) {
       self->translation_out[0] = 0;
       self->translation_out[1] = 0;
       self->translation_out[2] = 0;
       ROT_TRANS1_init = true;
   }         

// account quaternion multiplication a b
   extern void quaternionMult(float *c, float *a, float *b);
// make rotation to quaternion with norm 1 
   extern void norm_quaternion_from_rotation(float *quad, float *rot); 

   self->rotation[0] = self->rotation_in[0];
   self->rotation[1] = self->rotation_in[1];
   self->rotation[2] = self->rotation_in[2];
   self->rotation[3] = self->rotation_in[3];
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);
   self->quaternion2[0] = self->translation_in[0];
   self->quaternion2[1] = self->translation_in[1];
   self->quaternion2[2] = self->translation_in[2];
   self->quaternion2[3] = 0;
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);
   self->quaternion4[0] = -self->quaternion1[0];
   self->quaternion4[1] = -self->quaternion1[1];
   self->quaternion4[2] = -self->quaternion1[2];
   self->quaternion4[3] = self->quaternion1[3];
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);
   self->translation_out[0] = self->quaternion2[0];
   self->translation_out[1] = self->quaternion2[1];
   self->translation_out[2] = self->quaternion2[2];
    return true;
}

bool X3dScript_WALL_FIRESendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_WALL_FIRE *self = (X3dScript_WALL_FIRE *)node;
      static bool WALL_FIRE_init = false;
      if (!WALL_FIRE_init) {
           self->fire = false; 
           WALL_FIRE_init = true;
      }         
      if ((!self->isactive_in) && self->old_isactive)
          self->fire = true;
      else
          self->fire = false;
      self->old_isactive = self->isactive_in;           
    return true;
}

bool X3dScript_ROLL_TURNSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROLL_TURN *self = (X3dScript_ROLL_TURN *)node;
   static bool ROLL_TURN_init = false;
   if (!ROLL_TURN_init) {
       self->rotation_out[0]=1;
       self->rotation_out[1]=0;
       self->rotation_out[2]=0;
       self->rotation_out[3]=0;
       ROLL_TURN_init = true;
   }         
   self->rock_and_roll_flag = self->rock_and_roll_flag_in;
   if (self->rock_and_roll_flag==1) {
       self->rotation_out[1]=0;
       self->rotation_out[2]=1;
   } else {
       self->rotation_out[1]=1;
       self->rotation_out[2]=0;
   }
   self->rotation_out[0]=0;
   self->rotation_out[3]=self->xvalue_in * 2.0;
    return true;
}

bool X3dScript_UP_DOWN_TURNSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_UP_DOWN_TURN *self = (X3dScript_UP_DOWN_TURN *)node;
   static bool UP_DOWN_TURN_init = false;
   if (!UP_DOWN_TURN_init) {
       self->rotation_out[0]=0;
       self->rotation_out[1]=0;
       self->rotation_out[2]=0;
       self->rotation_out[3]=0;
       UP_DOWN_TURN_init = true;
   }         
   self->rotation_out[0]=1;
   self->rotation_out[1]=0;
   self->rotation_out[2]=0;
   self->rotation_out[3]=self->yvalue_in * 2.0;
    return true;
}

bool X3dScript_FLY_FORWARDSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_FLY_FORWARD *self = (X3dScript_FLY_FORWARD *)node;
   static bool FLY_FORWARD_init = false;
   if (!FLY_FORWARD_init) {
       self->translation_out[0]=0;
       self->translation_out[1]=0;
       self->translation_out[2]=0;
       FLY_FORWARD_init = true;
   }         
   self->translation_out[0]=0;
   self->translation_out[1]=0;
   self->translation_out[2]=0.15;
   self->time_out = self->time_in;
    return true;
}

bool X3dScript_FLY_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_FLY_ON *self = (X3dScript_FLY_ON *)node;
      static bool FLY_ON_init = false;
      if (!FLY_ON_init) {
           self->bullet_choice_out = -1; 
           FLY_ON_init = true;
      }         
/*
   function exit_time(val)
      {
      brake=TRUE;
      }
   // restart movement
   function enter_time(val)
      {
      brake=FALSE;
      }
*/
      self->factor = 0.06;
      self->old_time = self->time_in;
      if (self->fire_in)
         {
         self->bullet_translation_start = self->top_translation_out;
         // switch on bullet
         self->bullet_choice_out = 0;
         self->bulletway_length = 0;
         }
      self->top_translation[0] = self->translation_old[0] -
                                 self->translation_in[0] * self->factor;
      self->top_translation[1] = self->translation_old[1] -
                                 self->translation_in[1] * self->factor;
      self->top_translation[2] = self->translation_old[2] -
                                 self->translation_in[2] * self->factor;
      if (self->top_translation[2] < -35)
          self->top_translation[2] = 7;
      if (self->top_translation[2] > 7)
          self->top_translation[2] = -35;
      if (self->top_translation[0] < -7)
          self->top_translation[0] = 7;
      if (self->top_translation[0] > 7)
          self->top_translation[0] = -7;
      if (self->top_translation[1] < -7)
          self->top_translation[1] = 7;
      if (self->top_translation[1] > 7)
          self->top_translation[1] = -7;
      self->translation_old = self->top_translation;
      self->translation_out[0] = self->translation_old[0] -
                                 self->translation_in[0] * self->factor;
      self->translation_out[1] = self->translation_old[1] -
                                 self->translation_in[1] * self->factor;
      self->translation_out[2] = self->translation_old[2] - 
                                 self->translation_in[2] * self->factor;
      self->translation_old[0] = self->translation_out[0];
      self->translation_old[1] = self->translation_out[1];
      self->translation_old[2] = self->translation_out[2];
      if (self->bullet_choice_out == 0) 
         {
         if (self->bulletway_length == 0)
            {
            self->bullet_direction = self->translation_in;
            }
         if ((self->bullet_translation_out[2] > 10) ||
             (self->bullet_translation_out[2] < -35))
            {
            self->bulletway_length = 0;
            self->bullet_choice_out = -1;
            self->bullet_translation_out[2] = self->translation_in[2];
            }
         else if (self->bulletway_length < 45)
            {
            self->bullet_translation_out[0] = self->bullet_translation_start[0] -
                                              self->bulletway_length *
                                              self->bullet_direction[0];
            self->bullet_translation_out[1] = self->bullet_translation_start[1] -
                                              self->bulletway_length *
                                              self->bullet_direction[1];
            self->bullet_translation_out[2] = self->bullet_translation_start[2] -
                                              self->bulletway_length *
                                              self->bullet_direction[2];
            self->bulletway_length += 10.0 * self->factor;
            }
         else
            {
            self->bulletway_length = 0;
            self->bullet_choice_out = -1;
            self->bullet_translation_out[2] = self->translation_in[2];
            }
         }
      self->top_translation_out = self->top_translation;   
    return true;
}

bool X3dScript_THING_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_THING_SCRIPT *self = (X3dScript_THING_SCRIPT *)node;
      if (self->isover_in)
         {
         self->hide_out[0]=0.1;
         self->hide_out[1]=0.1;
         self->hide_out[2]=0.1;
         }
      else
         {
         self->hide_out[0]=1.0;
         self->hide_out[1]=1.0;
         self->hide_out[2]=1.0;
         }
      if (self->fire_in)
         {
         self->hide_out[0]=1;
         self->hide_out[1]=1;
         self->hide_out[2]=1;
         }                  
    return true;
}

bool X3dScript_BOX2_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_HIT_CHECK *self = (X3dScript_BOX2_HIT_CHECK *)node;
   static bool BOX2_HIT_CHECK_init = false;
   if (!BOX2_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX2_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX2_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_MOVE_ON *self = (X3dScript_BOX2_MOVE_ON *)node;
   static bool BOX2_MOVE_ON_init = false;
   if (!BOX2_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX2_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX2_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_HIDE_SCRIPT *self = (X3dScript_BOX2_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX3_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_HIT_CHECK *self = (X3dScript_BOX3_HIT_CHECK *)node;
   static bool BOX3_HIT_CHECK_init = false;
   if (!BOX3_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 9;
       BOX3_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX3_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_MOVE_ON *self = (X3dScript_BOX3_MOVE_ON *)node;
   static bool BOX3_MOVE_ON_init = false;
   if (!BOX3_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX3_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX3_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_HIDE_SCRIPT *self = (X3dScript_BOX3_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX4_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_HIT_CHECK *self = (X3dScript_BOX4_HIT_CHECK *)node;
   static bool BOX4_HIT_CHECK_init = false;
   if (!BOX4_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX4_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX4_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_MOVE_ON *self = (X3dScript_BOX4_MOVE_ON *)node;
   static bool BOX4_MOVE_ON_init = false;
   if (!BOX4_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX4_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX4_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_HIDE_SCRIPT *self = (X3dScript_BOX4_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX5_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_HIT_CHECK *self = (X3dScript_BOX5_HIT_CHECK *)node;
   static bool BOX5_HIT_CHECK_init = false;
   if (!BOX5_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX5_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX5_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_MOVE_ON *self = (X3dScript_BOX5_MOVE_ON *)node;
   static bool BOX5_MOVE_ON_init = false;
   if (!BOX5_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX5_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX5_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_HIDE_SCRIPT *self = (X3dScript_BOX5_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_LOAD_NEW_WORLDSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_LOAD_NEW_WORLD *self = (X3dScript_LOAD_NEW_WORLD *)node;
      static bool LOAD_NEW_WORLD_init = false;
      if (!LOAD_NEW_WORLD_init) {
          self->box_in[0] = 0;
          self->box_in[1] = 0;
          self->box_in[2] = 0;
          self->box_in[3] = 0;
          self->box_in[4] = 0;
          self->starttime_out = 0;
          self->plane_choice = 0;
          self->deleted = self->waitticks;
          LOAD_NEW_WORLD_init = true;
      }         
      if (!self->victory)      
         if (self->deleted > 0)
            {
            if (self->deleted == self->waitticks)
               {
//               starttime_out=val;
               self->choice_out = 0;
               }
            self->deleted--;
            }         
         else
            self->choice_out = -1;

      if ((self->box1_in == 1) && (self->box_in[0] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[0] = self->box1_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box2_in == 1) && (self->box_in[1] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[1] = self->box2_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box3_in == 1) && (self->box_in[2] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[2] = self->box3_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box4_in == 1) && (self->box_in[3] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[3] = self->box4_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box5_in == 1) && (self->box_in[4] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[4] = self->box5_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box_in[0] + 
           self->box_in[1] + 
           self->box_in[2] + 
           self->box_in[3] +
           self->box_in[4]) == 5)
          {
          self->victory = true;
          self->victory_out = 1;
          }
    return true;
}

class X3dSceneGraph {
public:
    X3dGroup root;
    X3dSwitch Switch2;
    X3dGroup Group1;
    X3dTransform BOX1m_HIDE;
    X3dSwitch BOX1m_SWITCH;
    X3dTransform BOX1m_MOVE;
    X3dTouchSensor BOX1m_SENSOR;
    X3dGroup Inline1;
    X3dGroup Group2;
    X3dShape Shape1;
    X3dAppearance Appearance1;
    X3dMaterial Material1;
    X3dIndexedFaceSet IndexedFaceSet1;
    X3dColor Color1;
    X3dCoordinate Coordinate1;
    X3dNormal Normal19;
    X3dColor Color2;
    X3dCoordinate Coordinate2;
    X3dShape Shape2;
    X3dAppearance Appearance2;
    X3dMaterial Material2;
    X3dImageTexture ImageTexture1;
    X3dIndexedFaceSet IndexedFaceSet2;
    X3dCoordinate Coordinate3;
    X3dNormal Normal20;
    X3dTextureCoordinate TextureCoordinate1;
    X3dCoordinate Coordinate4;
    X3dTextureCoordinate TextureCoordinate2;
    X3dWorldInfo WorldInfo1;
    X3dScript_BOX1_HIT_CHECK BOX1m_HITm_CHECK;
    X3dScript_BOX1_MOVE_ON BOX1m_MOVEm_ON;
    X3dScript_BOX1_HIDE_SCRIPT BOX1m_HIDEm_SCRIPT;
    X3dScript_STEER STEER;
    X3dPlaneSensor WALLm_SENSOR;
    X3dTouchSensor WALLm_SENSOR2;
    X3dScript_ROT_TRANS1 ROTm_TRANS1;
    X3dScript_ROT_TRANS2 ROTm_TRANS2;
    X3dBackground Background1;
    X3dScript_WALL_FIRE WALLm_FIRE;
    X3dTransform THING;
    X3dTransform ROLLm_AXIS;
    X3dTransform UPm_DOWNm_AXIS;
    X3dTouchSensor THINGm_SENSOR;
    X3dTransform HIDEm_THING;
    X3dGroup Inline6;
    X3dTransform Transform1;
    X3dGroup Inline2;
    X3dGroup Group3;
    X3dTransform Transform2;
    X3dShape Shape3;
    X3dAppearance Appearance3;
    X3dMaterial A0m_1;
    X3dIndexedFaceSet IndexedFaceSet3;
    X3dColor Color3;
    X3dCoordinate Coordinate5;
    X3dNormal Normal1;
    X3dColor Color4;
    X3dCoordinate Coordinate6;
    X3dNormal Normal21;
    X3dTransform Transform3;
    X3dShape Shape4;
    X3dAppearance Appearance4;
    X3dMaterial A0m_2;
    X3dIndexedFaceSet IndexedFaceSet4;
    X3dColor Color5;
    X3dCoordinate Coordinate7;
    X3dNormal Normal2;
    X3dColor Color6;
    X3dCoordinate Coordinate8;
    X3dNormal Normal22;
    X3dTransform Transform4;
    X3dShape Shape5;
    X3dAppearance Appearance5;
    X3dMaterial Material3;
    X3dIndexedFaceSet IndexedFaceSet5;
    X3dCoordinate Coordinate9;
    X3dNormal Normal3;
    X3dCoordinate Coordinate10;
    X3dNormal Normal23;
    X3dTransform Transform5;
    X3dShape Shape6;
    X3dAppearance Appearance6;
    X3dMaterial Material4;
    X3dIndexedFaceSet IndexedFaceSet6;
    X3dCoordinate Coordinate11;
    X3dNormal Normal4;
    X3dCoordinate Coordinate12;
    X3dNormal Normal24;
    X3dTransform Transform6;
    X3dTransform Transform7;
    X3dTransform Transform8;
    X3dShape Shape7;
    X3dAppearance Appearance7;
    X3dMaterial Material5;
    X3dIndexedFaceSet IndexedFaceSet7;
    X3dCoordinate Coordinate13;
    X3dNormal Normal5;
    X3dTextureCoordinate TextureCoordinate3;
    X3dCoordinate Coordinate14;
    X3dNormal Normal25;
    X3dTextureCoordinate TextureCoordinate4;
    X3dTransform Transform9;
    X3dShape Shape8;
    X3dAppearance Appearance8;
    X3dMaterial Material6;
    X3dIndexedFaceSet IndexedFaceSet8;
    X3dCoordinate Coordinate15;
    X3dNormal Normal6;
    X3dTextureCoordinate TextureCoordinate5;
    X3dCoordinate Coordinate16;
    X3dNormal Normal26;
    X3dTextureCoordinate TextureCoordinate6;
    X3dTransform Transform10;
    X3dTransform Transform11;
    X3dShape Shape9;
    X3dAppearance Appearance9;
    X3dMaterial Material7;
    X3dIndexedFaceSet IndexedFaceSet9;
    X3dCoordinate Coordinate17;
    X3dNormal Normal7;
    X3dTextureCoordinate TextureCoordinate7;
    X3dCoordinate Coordinate18;
    X3dNormal Normal27;
    X3dTextureCoordinate TextureCoordinate8;
    X3dTransform Transform12;
    X3dShape Shape10;
    X3dAppearance Appearance10;
    X3dMaterial Material8;
    X3dIndexedFaceSet IndexedFaceSet10;
    X3dCoordinate Coordinate19;
    X3dNormal Normal8;
    X3dTextureCoordinate TextureCoordinate9;
    X3dCoordinate Coordinate20;
    X3dNormal Normal28;
    X3dTextureCoordinate TextureCoordinate10;
    X3dGroup Inline5;
    X3dGroup Inline3;
    X3dGroup Group4;
    X3dTransform Transform13;
    X3dCoordinateInterpolator CORINTm_5;
    X3dTimeSensor TIMERm_4;
    X3dGroup Group5;
    X3dBackground Background2;
    X3dShape Shape11;
    X3dAppearance Appearance11;
    X3dMaterial Material9;
    X3dIndexedFaceSet IndexedFaceSet11;
    X3dCoordinate COORD1m_5;
    X3dNormal Normal29;
    X3dCoordinate COORD1m_4;
    X3dGroup Inline4;
    X3dGroup Group6;
    X3dTransform Transform14;
    X3dCoordinateInterpolator CORINTm_6;
    X3dTimeSensor TIMERm_5;
    X3dGroup Group7;
    X3dBackground Background3;
    X3dShape Shape12;
    X3dAppearance Appearance12;
    X3dMaterial Material10;
    X3dIndexedFaceSet IndexedFaceSet12;
    X3dCoordinate COORD1m_7;
    X3dNormal Normal30;
    X3dCoordinate COORD1m_6;
    X3dWorldInfo WorldInfo2;
    X3dSwitch PLANEm_SWITCH;
    X3dGroup Inline7;
    X3dTransform Transform15;
    X3dShape Shape13;
    X3dAppearance Appearance13;
    X3dMaterial Material11;
    X3dIndexedFaceSet IndexedFaceSet13;
    X3dColor Color7;
    X3dCoordinate Coordinate21;
    X3dNormal Normal31;
    X3dColor Color8;
    X3dCoordinate Coordinate22;
    X3dTransform Transform16;
    X3dSwitch BULLETm_SWITCH;
    X3dTransform BULLETm_MOVE;
    X3dTouchSensor DUMMYm_BULLET;
    X3dGroup Inline9;
    X3dTransform Transform17;
    X3dGroup Inline8;
    X3dShape Shape14;
    X3dAppearance Appearance14;
    X3dMaterial Material12;
    X3dImageTexture ImageTexture2;
    X3dIndexedFaceSet IndexedFaceSet14;
    X3dCoordinate Coordinate23;
    X3dNormal Normal9;
    X3dTextureCoordinate TextureCoordinate11;
    X3dCoordinate Coordinate24;
    X3dNormal Normal32;
    X3dTextureCoordinate TextureCoordinate12;
    X3dWorldInfo WorldInfo3;
    X3dWorldInfo WorldInfo4;
    X3dTimeSensor TIMER;
    X3dSound SOUND;
    X3dAudioClip SOUNDCLIP;
    X3dNavigationInfo NavigationInfo1;
    X3dScript_ROLL_TURN ROLLm_TURN;
    X3dScript_UP_DOWN_TURN UPm_DOWNm_TURN;
    X3dScript_FLY_FORWARD FLYm_FORWARD;
    X3dScript_FLY_ON FLYm_ON;
    X3dScript_THING_SCRIPT THINGm_SCRIPT;
    X3dViewpoint Viewpoint1;
    X3dViewpoint Viewpoint2;
    X3dWorldInfo WorldInfo5;
    X3dTransform Transform18;
    X3dShape Shape15;
    X3dAppearance Appearance15;
    X3dMaterial Material13;
    X3dText Text1;
    X3dGroup Group8;
    X3dSwitch Switch1;
    X3dTransform Transform19;
    X3dShape Shape16;
    X3dAppearance Appearance16;
    X3dMaterial Material14;
    X3dText Text1m_5;
    X3dTouchSensor DUMMYm_TEXT;
    X3dShape Shape17;
    X3dAppearance Appearance17;
    X3dMaterial Material15;
    X3dIndexedFaceSet IndexedFaceSet15;
    X3dCoordinate COORDm_4;
    X3dNormal Normal10;
    X3dCoordinate COORD;
    X3dNormal Normal33;
    X3dTransform BOX2m_HIDE;
    X3dSwitch BOX2m_SWITCH;
    X3dTransform BOX2m_MOVE;
    X3dTouchSensor BOX2m_SENSOR;
    X3dGroup Inline10;
    X3dGroup Group9;
    X3dShape Shape18;
    X3dAppearance Appearance18;
    X3dMaterial Material16;
    X3dIndexedFaceSet IndexedFaceSet16;
    X3dColor Color9;
    X3dCoordinate Coordinate25;
    X3dNormal Normal34;
    X3dColor Color10;
    X3dCoordinate Coordinate26;
    X3dShape Shape19;
    X3dAppearance Appearance19;
    X3dMaterial Material17;
    X3dImageTexture ImageTexture3;
    X3dIndexedFaceSet IndexedFaceSet17;
    X3dCoordinate Coordinate27;
    X3dNormal Normal35;
    X3dTextureCoordinate TextureCoordinate13;
    X3dCoordinate Coordinate28;
    X3dTextureCoordinate TextureCoordinate14;
    X3dWorldInfo WorldInfo6;
    X3dScript_BOX2_HIT_CHECK BOX2m_HITm_CHECK;
    X3dScript_BOX2_MOVE_ON BOX2m_MOVEm_ON;
    X3dScript_BOX2_HIDE_SCRIPT BOX2m_HIDEm_SCRIPT;
    X3dTransform BOX3m_HIDE;
    X3dSwitch BOX3m_SWITCH;
    X3dTransform BOX3m_MOVE;
    X3dTouchSensor BOX3m_SENSOR;
    X3dGroup Inline11;
    X3dGroup Group10;
    X3dShape Shape20;
    X3dAppearance Appearance20;
    X3dMaterial Material18;
    X3dIndexedFaceSet IndexedFaceSet18;
    X3dColor Color11;
    X3dCoordinate Coordinate29;
    X3dNormal Normal36;
    X3dColor Color12;
    X3dCoordinate Coordinate30;
    X3dShape Shape21;
    X3dAppearance Appearance21;
    X3dMaterial Material19;
    X3dImageTexture ImageTexture4;
    X3dIndexedFaceSet IndexedFaceSet19;
    X3dCoordinate Coordinate31;
    X3dNormal Normal37;
    X3dTextureCoordinate TextureCoordinate15;
    X3dCoordinate Coordinate32;
    X3dTextureCoordinate TextureCoordinate16;
    X3dWorldInfo WorldInfo7;
    X3dScript_BOX3_HIT_CHECK BOX3m_HITm_CHECK;
    X3dScript_BOX3_MOVE_ON BOX3m_MOVEm_ON;
    X3dScript_BOX3_HIDE_SCRIPT BOX3m_HIDEm_SCRIPT;
    X3dTransform BOX4m_HIDE;
    X3dSwitch BOX4m_SWITCH;
    X3dTransform BOX4m_MOVE;
    X3dTouchSensor BOX4m_SENSOR;
    X3dGroup Inline12;
    X3dGroup Group11;
    X3dShape Shape22;
    X3dAppearance Appearance22;
    X3dMaterial Material20;
    X3dIndexedFaceSet IndexedFaceSet20;
    X3dColor Color13;
    X3dCoordinate Coordinate33;
    X3dNormal Normal38;
    X3dColor Color14;
    X3dCoordinate Coordinate34;
    X3dShape Shape23;
    X3dAppearance Appearance23;
    X3dMaterial Material21;
    X3dImageTexture ImageTexture5;
    X3dIndexedFaceSet IndexedFaceSet21;
    X3dCoordinate Coordinate35;
    X3dNormal Normal39;
    X3dTextureCoordinate TextureCoordinate17;
    X3dCoordinate Coordinate36;
    X3dTextureCoordinate TextureCoordinate18;
    X3dWorldInfo WorldInfo8;
    X3dScript_BOX4_HIT_CHECK BOX4m_HITm_CHECK;
    X3dScript_BOX4_MOVE_ON BOX4m_MOVEm_ON;
    X3dScript_BOX4_HIDE_SCRIPT BOX4m_HIDEm_SCRIPT;
    X3dTransform BOX5m_HIDE;
    X3dSwitch BOX5m_SWITCH;
    X3dTransform BOX5m_MOVE;
    X3dTouchSensor BOX5m_SENSOR;
    X3dGroup Inline13;
    X3dGroup Group12;
    X3dShape Shape24;
    X3dAppearance Appearance24;
    X3dMaterial Material22;
    X3dIndexedFaceSet IndexedFaceSet22;
    X3dColor Color15;
    X3dCoordinate Coordinate37;
    X3dNormal Normal40;
    X3dColor Color16;
    X3dCoordinate Coordinate38;
    X3dShape Shape25;
    X3dAppearance Appearance25;
    X3dMaterial Material23;
    X3dImageTexture ImageTexture6;
    X3dIndexedFaceSet IndexedFaceSet23;
    X3dCoordinate Coordinate39;
    X3dNormal Normal41;
    X3dTextureCoordinate TextureCoordinate19;
    X3dCoordinate Coordinate40;
    X3dTextureCoordinate TextureCoordinate20;
    X3dWorldInfo WorldInfo9;
    X3dScript_BOX5_HIT_CHECK BOX5m_HITm_CHECK;
    X3dScript_BOX5_MOVE_ON BOX5m_MOVEm_ON;
    X3dScript_BOX5_HIDE_SCRIPT BOX5m_HIDEm_SCRIPT;
    X3dTransform Transform20;
    X3dGroup Inline19;
    X3dBackground Background4;
    X3dAnchor Anchor1;
    X3dBillboard Billboard1;
    X3dShape Shape26;
    X3dAppearance Appearance26;
    X3dMaterial Material24;
    X3dIndexedFaceSet IndexedFaceSet24;
    X3dCoordinate COORDm_6;
    X3dNormal Normal42;
    X3dCoordinate COORDm_5;
    X3dAnchor Anchor2;
    X3dTransform SCHRIFTm_6;
    X3dTransform Transform21;
    X3dGroup Inline18;
    X3dTransform Transform22;
    X3dGroup Inline14;
    X3dGroup Group13;
    X3dTransform Transform23;
    X3dShape Shape27;
    X3dAppearance Appearance27;
    X3dMaterial A0m_3;
    X3dIndexedFaceSet IndexedFaceSet25;
    X3dColor Color17;
    X3dCoordinate Coordinate41;
    X3dNormal Normal11;
    X3dColor Color18;
    X3dCoordinate Coordinate42;
    X3dNormal Normal43;
    X3dTransform Transform24;
    X3dShape Shape28;
    X3dAppearance Appearance28;
    X3dMaterial A0m_4;
    X3dIndexedFaceSet IndexedFaceSet26;
    X3dColor Color19;
    X3dCoordinate Coordinate43;
    X3dNormal Normal12;
    X3dColor Color20;
    X3dCoordinate Coordinate44;
    X3dNormal Normal44;
    X3dTransform Transform25;
    X3dShape Shape29;
    X3dAppearance Appearance29;
    X3dMaterial Material25;
    X3dIndexedFaceSet IndexedFaceSet27;
    X3dCoordinate Coordinate45;
    X3dNormal Normal13;
    X3dCoordinate Coordinate46;
    X3dNormal Normal45;
    X3dTransform Transform26;
    X3dShape Shape30;
    X3dAppearance Appearance30;
    X3dMaterial Material26;
    X3dIndexedFaceSet IndexedFaceSet28;
    X3dCoordinate Coordinate47;
    X3dNormal Normal14;
    X3dCoordinate Coordinate48;
    X3dNormal Normal46;
    X3dTransform Transform27;
    X3dTransform Transform28;
    X3dTransform Transform29;
    X3dShape Shape31;
    X3dAppearance Appearance31;
    X3dMaterial Material27;
    X3dIndexedFaceSet IndexedFaceSet29;
    X3dCoordinate Coordinate49;
    X3dNormal Normal15;
    X3dTextureCoordinate TextureCoordinate21;
    X3dCoordinate Coordinate50;
    X3dNormal Normal47;
    X3dTextureCoordinate TextureCoordinate22;
    X3dTransform Transform30;
    X3dShape Shape32;
    X3dAppearance Appearance32;
    X3dMaterial Material28;
    X3dIndexedFaceSet IndexedFaceSet30;
    X3dCoordinate Coordinate51;
    X3dNormal Normal16;
    X3dTextureCoordinate TextureCoordinate23;
    X3dCoordinate Coordinate52;
    X3dNormal Normal48;
    X3dTextureCoordinate TextureCoordinate24;
    X3dTransform Transform31;
    X3dTransform Transform32;
    X3dShape Shape33;
    X3dAppearance Appearance33;
    X3dMaterial Material29;
    X3dIndexedFaceSet IndexedFaceSet31;
    X3dCoordinate Coordinate53;
    X3dNormal Normal17;
    X3dTextureCoordinate TextureCoordinate25;
    X3dCoordinate Coordinate54;
    X3dNormal Normal49;
    X3dTextureCoordinate TextureCoordinate26;
    X3dTransform Transform33;
    X3dShape Shape34;
    X3dAppearance Appearance34;
    X3dMaterial Material30;
    X3dIndexedFaceSet IndexedFaceSet32;
    X3dCoordinate Coordinate55;
    X3dNormal Normal18;
    X3dTextureCoordinate TextureCoordinate27;
    X3dCoordinate Coordinate56;
    X3dNormal Normal50;
    X3dTextureCoordinate TextureCoordinate28;
    X3dGroup Inline17;
    X3dGroup Inline15;
    X3dGroup Group14;
    X3dTransform Transform34;
    X3dCoordinateInterpolator CORINTm_7;
    X3dTimeSensor TIMERm_6;
    X3dGroup Group15;
    X3dBackground Background5;
    X3dShape Shape35;
    X3dAppearance Appearance35;
    X3dMaterial Material31;
    X3dIndexedFaceSet IndexedFaceSet33;
    X3dCoordinate COORD1m_9;
    X3dNormal Normal51;
    X3dCoordinate COORD1m_8;
    X3dGroup Inline16;
    X3dGroup Group16;
    X3dTransform Transform35;
    X3dCoordinateInterpolator CORINTm_8;
    X3dTimeSensor TIMERm_7;
    X3dGroup Group17;
    X3dBackground Background6;
    X3dShape Shape36;
    X3dAppearance Appearance36;
    X3dMaterial Material32;
    X3dIndexedFaceSet IndexedFaceSet34;
    X3dCoordinate COORD1m_11;
    X3dNormal Normal52;
    X3dCoordinate COORD1m_10;
    X3dWorldInfo WorldInfo10;
    X3dTransform Transform36;
    X3dShape Shape37;
    X3dText Text2;
    X3dOrientationInterpolator ROTATORm_6;
    X3dTimeSensor TIMERm_8;
    X3dScript_LOAD_NEW_WORLD LOADm_NEWm_WORLD;
public:
    X3dSceneGraph();
    void render(void *data) { root.treeRender(data); }
    void doWithData(void *data) { root.treeDoWithData(data); }
    X3dNode *getNodeFromGlName(int glName);
};

X3dSceneGraph::X3dSceneGraph() {
    root.m_parent = (X3dNode *)NULL;
    Switch2.m_parent = (X3dNode *)&root;
    Group1.m_parent = (X3dNode *)&Switch2;
    BOX1m_HIDE.route_sources =  NULL;
    BOX1m_HIDE.num_route_source = 0;
    BOX1m_HIDE.m_parent = (X3dNode *)&Group1;
    BOX1m_SWITCH.m_parent = (X3dNode *)&BOX1m_HIDE;
    BOX1m_MOVE.route_sources =  NULL;
    BOX1m_MOVE.num_route_source = 0;
    BOX1m_MOVE.m_parent = (X3dNode *)&BOX1m_SWITCH;
    BOX1m_SENSOR.m_parent = (X3dNode *)&BOX1m_MOVE;
    BOX1m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX1m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX1m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX1m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX1m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX1m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX1m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX1m_SENSOR.isActive = false;
    BOX1m_SENSOR.isOver = false;
    BOX1m_SENSOR.touchTime = 0;
    Inline1.m_parent = (X3dNode *)&BOX1m_MOVE;
    Group2.m_parent = (X3dNode *)&Inline1;
    Shape1.m_parent = (X3dNode *)NULL;
    Appearance1.m_parent = (X3dNode *)NULL;
    Material1.m_parent = (X3dNode *)NULL;
    Material1.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material1.diffuseColor = m_diffuseColor;
    Material1.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material1.emissiveColor = m_emissiveColor;
    Material1.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material1.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material1.specularColor = m_specularColor;
    Material1.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material1.transparency = 0;
    Appearance1.material = &Material1;
    Appearance1.texture = NULL;
    Appearance1.textureTransform = NULL;
    IndexedFaceSet1.setGlName(0);
    IndexedFaceSet1.m_parent = (X3dNode *)NULL;
    Color1.m_parent = (X3dNode *)&IndexedFaceSet1;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color1.color = m_color;
    Color1.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate1.m_parent = (X3dNode *)&IndexedFaceSet1;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate1.point = m_point;
    Coordinate1.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal19.m_parent = (X3dNode *)&IndexedFaceSet1;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal19.vector = m_vector;
    Normal19.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet1.color = &Color1;
    IndexedFaceSet1.coord = &Coordinate1;
    IndexedFaceSet1.normal = &Normal19;
    IndexedFaceSet1.texCoord = NULL;
    IndexedFaceSet1.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 31360649, -1, 0, 0, 0, -1, 0, 6, 0, -1, 0, 0, 0, -1, 8, 11, 7, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet1.colorIndex = m_colorIndex;
    IndexedFaceSet1.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet1.colorPerVertex = false;
    IndexedFaceSet1.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet1.coordIndex = m_coordIndex;
    IndexedFaceSet1.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet1.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet1.normalIndex = m_normalIndex;
    IndexedFaceSet1.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet1.normalPerVertex = true;
    IndexedFaceSet1.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet1.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet1.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet1.set_colorIndex = NULL;
    IndexedFaceSet1.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet1.set_coordIndex = NULL;
    IndexedFaceSet1.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet1.set_normalIndex = NULL;
    IndexedFaceSet1.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet1.set_texCoordIndex = NULL;
    IndexedFaceSet1.set_texCoordIndex_length = 0;
    }
    Shape1.appearance = &Appearance1;
    Shape1.geometry = &IndexedFaceSet1;
    Shape2.m_parent = (X3dNode *)NULL;
    Appearance2.m_parent = (X3dNode *)NULL;
    Material2.m_parent = (X3dNode *)NULL;
    Material2.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material2.diffuseColor = m_diffuseColor;
    Material2.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material2.emissiveColor = m_emissiveColor;
    Material2.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material2.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material2.specularColor = m_specularColor;
    Material2.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material2.transparency = 0;
    ImageTexture1.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture1.url = m_url;
    ImageTexture1.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture1.repeatS = true;
    ImageTexture1.repeatT = true;
    Appearance2.material = &Material2;
    Appearance2.texture = &ImageTexture1;
    Appearance2.textureTransform = NULL;
    IndexedFaceSet2.setGlName(1);
    IndexedFaceSet2.m_parent = (X3dNode *)NULL;
    Coordinate3.m_parent = (X3dNode *)&IndexedFaceSet2;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate3.point = m_point;
    Coordinate3.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal20.m_parent = (X3dNode *)&IndexedFaceSet2;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal20.vector = m_vector;
    Normal20.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate1.m_parent = (X3dNode *)&IndexedFaceSet2;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate1.point = m_point;
    TextureCoordinate1.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet2.color = NULL;
    IndexedFaceSet2.coord = &Coordinate3;
    IndexedFaceSet2.normal = &Normal20;
    IndexedFaceSet2.texCoord = &TextureCoordinate1;
    IndexedFaceSet2.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet2.colorIndex = m_colorIndex;
    IndexedFaceSet2.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet2.colorPerVertex = false;
    IndexedFaceSet2.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet2.coordIndex = m_coordIndex;
    IndexedFaceSet2.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet2.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet2.normalIndex = m_normalIndex;
    IndexedFaceSet2.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet2.normalPerVertex = true;
    IndexedFaceSet2.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet2.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet2.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet2.set_colorIndex = NULL;
    IndexedFaceSet2.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet2.set_coordIndex = NULL;
    IndexedFaceSet2.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet2.set_normalIndex = NULL;
    IndexedFaceSet2.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet2.set_texCoordIndex = NULL;
    IndexedFaceSet2.set_texCoordIndex_length = 0;
    }
    Shape2.appearance = &Appearance2;
    Shape2.geometry = &IndexedFaceSet2;
    {
    static X3dNode* m_children[] = { NULL, &Shape1, NULL, &Shape2 };
    Group2.children = m_children;
    Group2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group2.bboxCenter = m_bboxCenter;
    Group2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group2.bboxSize = m_bboxSize;
    Group2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group2.addChildren = NULL;
    Group2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group2.removeChildren = NULL;
    Group2.removeChildren_length = 0;
    }
    WorldInfo1.m_parent = (X3dNode *)&Inline1;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo1.info = m_info;
    WorldInfo1.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo1.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group2, NULL, &WorldInfo1 };
    Inline1.children = m_children;
    Inline1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline1.bboxCenter = m_bboxCenter;
    Inline1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline1.bboxSize = m_bboxSize;
    Inline1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline1.addChildren = NULL;
    Inline1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline1.removeChildren = NULL;
    Inline1.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX1m_MOVE.rotation = m_rotation;
    BOX1m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX1m_MOVE.translation = m_translation;
    BOX1m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX1m_MOVE.bboxCenter = m_bboxCenter;
    BOX1m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX1m_MOVE.bboxSize = m_bboxSize;
    BOX1m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX1m_MOVE.center = m_center;
    BOX1m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX1m_MOVE.scale = m_scale;
    BOX1m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX1m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX1m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_SENSOR, &Inline1 };
    BOX1m_MOVE.children = m_children;
    BOX1m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX1m_MOVE.addChildren = NULL;
    BOX1m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX1m_MOVE.removeChildren = NULL;
    BOX1m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX1m_MOVE };
    BOX1m_SWITCH.children = m_children;
    BOX1m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX1m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX1m_HIDE.rotation = m_rotation;
    BOX1m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX1m_HIDE.translation = m_translation;
    BOX1m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX1m_HIDE.bboxCenter = m_bboxCenter;
    BOX1m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX1m_HIDE.bboxSize = m_bboxSize;
    BOX1m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX1m_HIDE.center = m_center;
    BOX1m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX1m_HIDE.scale = m_scale;
    BOX1m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX1m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX1m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_SWITCH };
    BOX1m_HIDE.children = m_children;
    BOX1m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX1m_HIDE.addChildren = NULL;
    BOX1m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX1m_HIDE.removeChildren = NULL;
    BOX1m_HIDE.removeChildren_length = 0;
    }
    BOX1m_HITm_CHECK.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function initialize()  \
         {  \
         choice_out=0;  \
         deleted_out=0;  \
         }  \
      function translation_in(val)    \
         {  \
         if (!deleted)  \
            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)  \
                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)  \
                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)  \
               )  \
               {  \
               choice_out=1;  \
               deleted_out=1;  \
               deleted=1;  \
               }  \
            else  \
               choice_out=0;  \
         else  \
            choice_out=1;  \
         }  \
           \
      function box_position(val)  \
         {  \
         box[0]=val[0];  \
         box[1]=val[1];  \
         box[2]=val[2];  \
         }    \
      ", "c++:  \
   X3dScript_BOX1_HIT_CHECK *self = (X3dScript_BOX1_HIT_CHECK *)node;  \
   static bool BOX1_HIT_CHECK_init = false;  \
   if (!BOX1_HIT_CHECK_init) {  \
       self->choice_out = 0;  \
       self->deleted_out = 0;  \
       BOX1_HIT_CHECK_init = true;  \
   } else if (!self->deleted)   \
      {  \
      self->box[0] = self->box_position[0];  \
      self->box[1] = self->box_position[1];  \
      self->box[2] = self->box_position[2];  \
      if (  \
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) &&   \
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) &&   \
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) &&   \
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) &&   \
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) &&   \
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)  \
         )  \
         {  \
         self->choice_out = 1;  \
         self->deleted_out = 1;  \
         self->deleted = 1;  \
         }  \
      else  \
         self->choice_out = 0;  \
   }  \
   else  \
      self->choice_out = 1;  \
" };
    BOX1m_HITm_CHECK.url = m_url;
    BOX1m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_HITm_CHECK.directOutput = false;
    BOX1m_HITm_CHECK.mustEvaluate = false;
    BOX1m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX1m_HITm_CHECK.box = m_box;
    BOX1m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX1m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX1m_HITm_CHECK.translation_in = m_translation_in;
    BOX1m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX1m_HITm_CHECK.box_position = m_box_position;
    BOX1m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX1m_HITm_CHECK.deleted_out = 0;
    BOX1m_HITm_CHECK.choice_out = 0;
    BOX1m_MOVEm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function nextsteps()  \
         {  \
         step_translation[0]=(Math.random()-0.5)*0.04;  \
         step_translation[1]=(Math.random()-0.5)*0.04;  \
         step_translation[2]=(Math.random()-0.5)*0.04;  \
         step=Math.random()*200+1;  \
         }  \
      function initialize()  \
         {  \
         translation_out[0]=Math.random()*8-4;  \
         translation_out[1]=Math.random()*6-3;  \
         translation_out[2]=Math.random()*-30-1;  \
         nextsteps();  \
         }  \
      function translation_in(val)    \
         {  \
         step--;  \
         if (step<=0)  \
            nextsteps();  \
         if (   (translation_out[0]+step_translation[0]<4)   \
             && (translation_out[0]+step_translation[0]>-4))   \
            translation_out[0]+=step_translation[0];  \
         if (   (translation_out[1]+step_translation[1]<3)   \
             && (translation_out[1]+step_translation[1]>-3))   \
             translation_out[1]+=step_translation[1];  \
         if (   (translation_out[2]+step_translation[2]<-1)   \
             && (translation_out[2]+step_translation[2]>-30))   \
            translation_out[2]+=step_translation[2];  \
         }  \
      ", "c++:  \
   X3dScript_BOX1_MOVE_ON *self = (X3dScript_BOX1_MOVE_ON *)node;  \
   static bool BOX1_MOVE_ON_init = false;  \
   if (!BOX1_MOVE_ON_init) {  \
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;  \
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;  \
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;  \
       BOX1_MOVE_ON_init = true;  \
   }           \
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   if ((self->translation_out[0] + self->step_translation[0] < 4) &&   \
       (self->translation_out[0] + self->step_translation[0] > -4))   \
       self->translation_out[0] += self->step_translation[0];  \
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&  \
       (self->translation_out[1] + self->step_translation[1] > -3))   \
       self->translation_out[1] += self->step_translation[1];  \
   if ((self->translation_out[2] + self->step_translation[2] < -1) &&   \
       (self->translation_out[2] + self->step_translation[2] > -30))   \
       self->translation_out[2] += self->step_translation[2];  \
" };
    BOX1m_MOVEm_ON.url = m_url;
    BOX1m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_MOVEm_ON.directOutput = false;
    BOX1m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.step_translation = m_step_translation;
    BOX1m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX1m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.translation_in = m_translation_in;
    BOX1m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.translation_out = m_translation_out;
    BOX1m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX1m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=100;  \
         }  \
      else  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=0;  \
         }  \
      }      \
   ", "c++:  \
   X3dScript_BOX1_HIDE_SCRIPT *self = (X3dScript_BOX1_HIDE_SCRIPT *)node;  \
   if (self->isover_in) {               \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 100;  \
   } else {  \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 0;  \
   }  \
" };
    BOX1m_HIDEm_SCRIPT.url = m_url;
    BOX1m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_HIDEm_SCRIPT.directOutput = false;
    BOX1m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX1m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX1m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX1m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    STEER.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()  \
      {  \
      }  \
/*  \
   function steer()  \
      {  \
      xvalue=wall_xy[0];  \
      yvalue=wall_xy[1];  \
      }  \
*/  \
   function wall_xy_in(val)  \
      {  \
      xvalue=val[0]/1.5;  \
      yvalue=val[1]/1.5;  \
      }  \
   ", "c++:  \
   X3dScript_STEER *self = (X3dScript_STEER *)node;  \
   self->xvalue = -self->wall_xy_in[0] / 1.5;  \
   self->yvalue = self->wall_xy_in[1] / 1.5;  \
" };
    STEER.url = m_url;
    STEER.url_length = sizeof(m_url) / sizeof(const char*);
    }
    STEER.directOutput = false;
    STEER.mustEvaluate = false;
    {
    static float m_wall_xy_in[] = { 0, 0, 0 };
    STEER.wall_xy_in = m_wall_xy_in;
    STEER.wall_xy_in_length = sizeof(m_wall_xy_in) / sizeof(float);
    }
    STEER.xvalue = 0;
    STEER.yvalue = 0;
    WALLm_SENSOR.m_parent = (X3dNode *)&Group1;
    WALLm_SENSOR.autoOffset = true;
    WALLm_SENSOR.enabled = true;
    {
    static float m_maxPosition[2] = { 1e4, 1e4 };
    WALLm_SENSOR.maxPosition = m_maxPosition;
    WALLm_SENSOR.maxPosition_length = sizeof(m_maxPosition) / sizeof(float);
    }
    {
    static float m_minPosition[2] = { -1e4, -1e4 };
    WALLm_SENSOR.minPosition = m_minPosition;
    WALLm_SENSOR.minPosition_length = sizeof(m_minPosition) / sizeof(float);
    }
    {
    static float m_offset[] = { 0, 0, 0 };
    WALLm_SENSOR.offset = m_offset;
    WALLm_SENSOR.offset_length = sizeof(m_offset) / sizeof(float);
    }
    WALLm_SENSOR.isActive = false;
    {
    static float m_trackPoint_changed[] = { 0, 0, 0 };
    WALLm_SENSOR.trackPoint_changed = m_trackPoint_changed;
    WALLm_SENSOR.trackPoint_changed_length = sizeof(m_trackPoint_changed) / sizeof(float);
    }
    {
    static float m_translation_changed[] = { 0, 0, 0 };
    WALLm_SENSOR.translation_changed = m_translation_changed;
    WALLm_SENSOR.translation_changed_length = sizeof(m_translation_changed) / sizeof(float);
    }
    WALLm_SENSOR2.m_parent = (X3dNode *)&Group1;
    WALLm_SENSOR2.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    WALLm_SENSOR2.hitNormal_changed = m_hitNormal_changed;
    WALLm_SENSOR2.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    WALLm_SENSOR2.hitPoint_changed = m_hitPoint_changed;
    WALLm_SENSOR2.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    WALLm_SENSOR2.hitTexCoord_changed = m_hitTexCoord_changed;
    WALLm_SENSOR2.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    WALLm_SENSOR2.isActive = false;
    WALLm_SENSOR2.isOver = false;
    WALLm_SENSOR2.touchTime = 0;
    ROTm_TRANS1.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()  \
      {  \
      translation_out[0]=0;  \
      translation_out[1]=0;  \
      translation_out[2]=0;  \
      }  \
// account vektorproduct c = a x b  \
   function vectorproduct(c,a,b)  \
      {  \
      c[0]=a[1]*b[2]-a[2]*b[1];  \
      c[1]=a[2]*b[0]-a[0]*b[2];  \
      c[2]=a[0]*b[1]-a[1]*b[0];  \
      }  \
// account scalarproduct a * b  \
   function scalarproduct(a,b)  \
      {  \
      return(a[0]*b[0]+a[1]*b[1]+a[2]*b[2]);  \
      }  \
// account quaternion multiplication a b  \
   function quaternionmult(c,a,b)  \
      {  \
      vectorproduct(c,a,b);  \
      c[0]=c[0]+a[3]*b[0]+b[3]*a[0];  \
      c[1]=c[1]+a[3]*b[1]+b[3]*a[1];  \
      c[2]=c[2]+a[3]*b[2]+b[3]*a[2];  \
      c[3]=a[3]*b[3]-scalarproduct(a,b);  \
      }               \
// make rotation to quaternion with norm 1   \
   function norm_quaternion_from_rotation(quad,rot)    \
      {  \
      quad[3]=Math.cos(rot[3]/2.0);  \
      winkelkorrektur=Math.sin(rot[3]/2.0);  \
      quad[0]=rot[0]*winkelkorrektur;  \
      quad[1]=rot[1]*winkelkorrektur;  \
      quad[2]=rot[2]*winkelkorrektur;  \
      leng=Math.sqrt(quad[0]*quad[0]+quad[1]*quad[1]+quad[2]*quad[2]+  \
                     quad[3]*quad[3]);  \
      if (leng!=0)  \
         {  \
         quad[0]=quad[0]/leng;  \
         quad[1]=quad[1]/leng;  \
         quad[2]=quad[2]/leng;  \
         quad[3]=quad[3]/leng;  \
         }  \
      else  \
         {  \
         quad[0]=0;  \
         quad[1]=0;  \
         quad[2]=0;  \
         quad[3]=0;  \
         }  \
      }  \
   function rotation_in(val)    \
      {  \
      rotation[0]=val[0];  \
      rotation[1]=val[1];  \
      rotation[2]=val[2];  \
      rotation[3]=val[3];  \
//            debug = new MFString(rotation);  \
      }  \
   function translation_in(val)    \
      {  \
      norm_quaternion_from_rotation(quaternion1,rotation);  \
      quaternion2[0]=val[0];  \
      quaternion2[1]=val[1];  \
      quaternion2[2]=val[2];  \
      quaternion2[3]=0;  \
      quaternionmult(quaternion3,quaternion1,quaternion2);  \
      quaternion4[0]=-quaternion1[0];  \
      quaternion4[1]=-quaternion1[1];  \
      quaternion4[2]=-quaternion1[2];  \
      quaternion4[3]= quaternion1[3];  \
      quaternionmult(quaternion2,quaternion3,quaternion4);  \
      translation_out[0]=quaternion2[0];  \
      translation_out[1]=quaternion2[1];  \
      translation_out[2]=quaternion2[2];  \
      }  \
   ", "c++:  \
   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;  \
   static bool ROT_TRANS1_init = false;  \
   if (!ROT_TRANS1_init) {  \
       self->translation_out[0] = 0;  \
       self->translation_out[1] = 0;  \
       self->translation_out[2] = 0;  \
       ROT_TRANS1_init = true;  \
   }           \
  \
// account quaternion multiplication a b  \
   extern void quaternionMult(float *c, float *a, float *b);  \
// make rotation to quaternion with norm 1   \
   extern void norm_quaternion_from_rotation(float *quad, float *rot);   \
  \
   self->rotation[0] = self->rotation_in[0];  \
   self->rotation[1] = self->rotation_in[1];  \
   self->rotation[2] = self->rotation_in[2];  \
   self->rotation[3] = self->rotation_in[3];  \
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);  \
   self->quaternion2[0] = self->translation_in[0];  \
   self->quaternion2[1] = self->translation_in[1];  \
   self->quaternion2[2] = self->translation_in[2];  \
   self->quaternion2[3] = 0;  \
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);  \
   self->quaternion4[0] = -self->quaternion1[0];  \
   self->quaternion4[1] = -self->quaternion1[1];  \
   self->quaternion4[2] = -self->quaternion1[2];  \
   self->quaternion4[3] = self->quaternion1[3];  \
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);  \
   self->translation_out[0] = self->quaternion2[0];  \
   self->translation_out[1] = self->quaternion2[1];  \
   self->translation_out[2] = self->quaternion2[2];  \
" };
    ROTm_TRANS1.url = m_url;
    ROTm_TRANS1.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROTm_TRANS1.directOutput = false;
    ROTm_TRANS1.mustEvaluate = false;
    {
    static float m_rotation[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.rotation = m_rotation;
    ROTm_TRANS1.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_quaternion1[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion1 = m_quaternion1;
    ROTm_TRANS1.quaternion1_length = sizeof(m_quaternion1) / sizeof(float);
    }
    {
    static float m_quaternion2[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion2 = m_quaternion2;
    ROTm_TRANS1.quaternion2_length = sizeof(m_quaternion2) / sizeof(float);
    }
    {
    static float m_quaternion3[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion3 = m_quaternion3;
    ROTm_TRANS1.quaternion3_length = sizeof(m_quaternion3) / sizeof(float);
    }
    {
    static float m_quaternion4[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion4 = m_quaternion4;
    ROTm_TRANS1.quaternion4_length = sizeof(m_quaternion4) / sizeof(float);
    }
    ROTm_TRANS1.leng = 1;
    ROTm_TRANS1.winkelkorrektur = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    ROTm_TRANS1.translation_in = m_translation_in;
    ROTm_TRANS1.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_rotation_in[] = { 0, 0, 1, 0 };
    ROTm_TRANS1.rotation_in = m_rotation_in;
    ROTm_TRANS1.rotation_in_length = sizeof(m_rotation_in) / sizeof(float);
    }
    {
    static const char* m_debug[1];
    ROTm_TRANS1.debug = NULL;
    ROTm_TRANS1.debug_length = 0;
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    ROTm_TRANS1.translation_out = m_translation_out;
    ROTm_TRANS1.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    ROTm_TRANS2.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()  \
      {  \
      translation_out[0]=0;  \
      translation_out[1]=0;  \
      translation_out[2]=0;  \
      }  \
// account vektorproduct c = a x b  \
   function vectorproduct(c,a,b)  \
      {  \
      c[0]=a[1]*b[2]-a[2]*b[1];  \
      c[1]=a[2]*b[0]-a[0]*b[2];  \
      c[2]=a[0]*b[1]-a[1]*b[0];  \
      }  \
// account scalarproduct a * b  \
   function scalarproduct(a,b)  \
      {  \
      return(a[0]*b[0]+a[1]*b[1]+a[2]*b[2]);  \
      }  \
// account quaternion multiplication a b  \
   function quaternionmult(c,a,b)  \
      {  \
      vectorproduct(c,a,b);  \
      c[0]=c[0]+a[3]*b[0]+b[3]*a[0];  \
      c[1]=c[1]+a[3]*b[1]+b[3]*a[1];  \
      c[2]=c[2]+a[3]*b[2]+b[3]*a[2];  \
      c[3]=a[3]*b[3]-scalarproduct(a,b);  \
      }               \
// make rotation to quaternion with norm 1   \
   function norm_quaternion_from_rotation(quad,rot)    \
      {  \
      quad[3]=Math.cos(rot[3]/2.0);  \
      winkelkorrektur=Math.sin(rot[3]/2.0);  \
      quad[0]=rot[0]*winkelkorrektur;  \
      quad[1]=rot[1]*winkelkorrektur;  \
      quad[2]=rot[2]*winkelkorrektur;  \
      leng=Math.sqrt(quad[0]*quad[0]+quad[1]*quad[1]+quad[2]*quad[2]+  \
                     quad[3]*quad[3]);  \
      if (leng!=0)  \
         {  \
         quad[0]=quad[0]/leng;  \
         quad[1]=quad[1]/leng;  \
         quad[2]=quad[2]/leng;  \
         quad[3]=quad[3]/leng;  \
         }  \
      else  \
         {  \
         quad[0]=0;  \
         quad[1]=0;  \
         quad[2]=0;  \
         quad[3]=0;  \
         }  \
      }  \
   function rotation_in(val)    \
      {  \
      rotation[0]=val[0];  \
      rotation[1]=val[1];  \
      rotation[2]=val[2];  \
      rotation[3]=val[3];  \
//            debug = new MFString(rotation);  \
      }  \
   function translation_in(val)    \
      {  \
      norm_quaternion_from_rotation(quaternion1,rotation);  \
      quaternion2[0]=val[0];  \
      quaternion2[1]=val[1];  \
      quaternion2[2]=val[2];  \
      quaternion2[3]=0;  \
      quaternionmult(quaternion3,quaternion1,quaternion2);  \
      quaternion4[0]=-quaternion1[0];  \
      quaternion4[1]=-quaternion1[1];  \
      quaternion4[2]=-quaternion1[2];  \
      quaternion4[3]= quaternion1[3];  \
      quaternionmult(quaternion2,quaternion3,quaternion4);  \
      translation_out[0]=quaternion2[0];  \
      translation_out[1]=quaternion2[1];  \
      translation_out[2]=quaternion2[2];  \
      }  \
   ", "c++:  \
   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;  \
   static bool ROT_TRANS1_init = false;  \
   if (!ROT_TRANS1_init) {  \
       self->translation_out[0] = 0;  \
       self->translation_out[1] = 0;  \
       self->translation_out[2] = 0;  \
       ROT_TRANS1_init = true;  \
   }           \
  \
// account quaternion multiplication a b  \
   extern void quaternionMult(float *c, float *a, float *b);  \
// make rotation to quaternion with norm 1   \
   extern void norm_quaternion_from_rotation(float *quad, float *rot);   \
  \
   self->rotation[0] = self->rotation_in[0];  \
   self->rotation[1] = self->rotation_in[1];  \
   self->rotation[2] = self->rotation_in[2];  \
   self->rotation[3] = self->rotation_in[3];  \
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);  \
   self->quaternion2[0] = self->translation_in[0];  \
   self->quaternion2[1] = self->translation_in[1];  \
   self->quaternion2[2] = self->translation_in[2];  \
   self->quaternion2[3] = 0;  \
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);  \
   self->quaternion4[0] = -self->quaternion1[0];  \
   self->quaternion4[1] = -self->quaternion1[1];  \
   self->quaternion4[2] = -self->quaternion1[2];  \
   self->quaternion4[3] = self->quaternion1[3];  \
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);  \
   self->translation_out[0] = self->quaternion2[0];  \
   self->translation_out[1] = self->quaternion2[1];  \
   self->translation_out[2] = self->quaternion2[2];  \
" };
    ROTm_TRANS2.url = m_url;
    ROTm_TRANS2.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROTm_TRANS2.directOutput = false;
    ROTm_TRANS2.mustEvaluate = false;
    {
    static float m_rotation[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.rotation = m_rotation;
    ROTm_TRANS2.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_quaternion1[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion1 = m_quaternion1;
    ROTm_TRANS2.quaternion1_length = sizeof(m_quaternion1) / sizeof(float);
    }
    {
    static float m_quaternion2[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion2 = m_quaternion2;
    ROTm_TRANS2.quaternion2_length = sizeof(m_quaternion2) / sizeof(float);
    }
    {
    static float m_quaternion3[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion3 = m_quaternion3;
    ROTm_TRANS2.quaternion3_length = sizeof(m_quaternion3) / sizeof(float);
    }
    {
    static float m_quaternion4[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion4 = m_quaternion4;
    ROTm_TRANS2.quaternion4_length = sizeof(m_quaternion4) / sizeof(float);
    }
    ROTm_TRANS2.leng = 1;
    ROTm_TRANS2.winkelkorrektur = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    ROTm_TRANS2.translation_in = m_translation_in;
    ROTm_TRANS2.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_rotation_in[] = { 0, 0, 1, 0 };
    ROTm_TRANS2.rotation_in = m_rotation_in;
    ROTm_TRANS2.rotation_in_length = sizeof(m_rotation_in) / sizeof(float);
    }
    {
    static const char* m_debug[1];
    ROTm_TRANS2.debug = NULL;
    ROTm_TRANS2.debug_length = 0;
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    ROTm_TRANS2.translation_out = m_translation_out;
    ROTm_TRANS2.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    Background1.m_parent = (X3dNode *)&Group1;
    {
    static float m_groundAngle[1];
    Background1.groundAngle = NULL;
    Background1.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background1.groundColor = m_groundColor;
    Background1.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background1.backUrl = NULL;
    Background1.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background1.rightUrl = NULL;
    Background1.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background1.frontUrl = NULL;
    Background1.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background1.leftUrl = NULL;
    Background1.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background1.topUrl = NULL;
    Background1.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background1.bottomUrl = NULL;
    Background1.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background1.skyAngle = NULL;
    Background1.skyAngle_length = 0;
    }
    Background1.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background1.skyColor = m_skyColor;
    Background1.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background1.set_bind = false;
    Background1.isBound = false;
    WALLm_FIRE.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
  \
 // field SFBool old_isactive //  \
  \
 // eventOut SFBool fire //  \
  \
      function initialize()              \
         {              \
         fire=FALSE;  \
         }              \
// release button == fire       \
      function isactive_in(val)              \
         {              \
         if ((!val) && old_isactive)   \
            fire=TRUE;  \
         else   \
            fire=FALSE;             \
         old_isactive=val;             \
         }              \
      ", "c++:  \
      X3dScript_WALL_FIRE *self = (X3dScript_WALL_FIRE *)node;  \
      static bool WALL_FIRE_init = false;  \
      if (!WALL_FIRE_init) {  \
           self->fire = false;   \
           WALL_FIRE_init = true;  \
      }           \
      if ((!self->isactive_in) && self->old_isactive)  \
          self->fire = true;  \
      else  \
          self->fire = false;  \
      self->old_isactive = self->isactive_in;             \
" };
    WALLm_FIRE.url = m_url;
    WALLm_FIRE.url_length = sizeof(m_url) / sizeof(const char*);
    }
    WALLm_FIRE.directOutput = false;
    WALLm_FIRE.mustEvaluate = false;
    WALLm_FIRE.old_isactive = false;
    WALLm_FIRE.isactive_in = false;
    WALLm_FIRE.fire = false;
    THING.route_sources =  NULL;
    THING.num_route_source = 0;
    THING.m_parent = (X3dNode *)&Group1;
    ROLLm_AXIS.route_sources =  NULL;
    ROLLm_AXIS.num_route_source = 0;
    ROLLm_AXIS.m_parent = (X3dNode *)&THING;
    UPm_DOWNm_AXIS.route_sources =  NULL;
    UPm_DOWNm_AXIS.num_route_source = 0;
    UPm_DOWNm_AXIS.m_parent = (X3dNode *)&ROLLm_AXIS;
    THINGm_SENSOR.m_parent = (X3dNode *)&UPm_DOWNm_AXIS;
    THINGm_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    THINGm_SENSOR.hitNormal_changed = m_hitNormal_changed;
    THINGm_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    THINGm_SENSOR.hitPoint_changed = m_hitPoint_changed;
    THINGm_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    THINGm_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    THINGm_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    THINGm_SENSOR.isActive = false;
    THINGm_SENSOR.isOver = false;
    THINGm_SENSOR.touchTime = 0;
    HIDEm_THING.route_sources =  NULL;
    HIDEm_THING.num_route_source = 0;
    HIDEm_THING.m_parent = (X3dNode *)&UPm_DOWNm_AXIS;
    Inline6.m_parent = (X3dNode *)&HIDEm_THING;
    Transform1.route_sources =  NULL;
    Transform1.num_route_source = 0;
    Transform1.m_parent = (X3dNode *)&Inline6;
    Inline2.m_parent = (X3dNode *)NULL;
    Group3.m_parent = (X3dNode *)&Inline2;
    Transform2.route_sources =  NULL;
    Transform2.num_route_source = 0;
    Transform2.m_parent = (X3dNode *)NULL;
    Shape3.m_parent = (X3dNode *)NULL;
    Appearance3.m_parent = (X3dNode *)NULL;
    A0m_1.m_parent = (X3dNode *)NULL;
    A0m_1.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_1.diffuseColor = m_diffuseColor;
    A0m_1.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_1.emissiveColor = m_emissiveColor;
    A0m_1.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_1.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_1.specularColor = m_specularColor;
    A0m_1.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_1.transparency = 0;
    Appearance3.material = &A0m_1;
    Appearance3.texture = NULL;
    Appearance3.textureTransform = NULL;
    IndexedFaceSet3.setGlName(2);
    IndexedFaceSet3.m_parent = (X3dNode *)NULL;
    Color3.m_parent = (X3dNode *)&IndexedFaceSet3;
    {
    static float m_color[] = { 1, 0.793614, 0.087319, 1, 0.638174, 0.077397, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.754465, 0.084820, 1, 0.754465, 0.084820, 1, 0.775771, 0.086180, 1, 0.756482, 0.084949, 1, 0.756482, 0.084949, 1, 0.777047, 0.086262 };
    Color3.color = m_color;
    Color3.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate5.m_parent = (X3dNode *)&IndexedFaceSet3;
    {
    static float m_point[] = { 0.150260, 0.330511, 1.553120, 0.256526, 0.628258, 1.205410, 0.043506, 0.580147, 1.404830, 0.041913, 0.281121, 1.582420, -0.098818, 0.305979, 1.539120, -0.172704, 0.627974, 1.205410, -0.237476, 0.530979, 1.149410, -0.066433, 0.330228, 1.553120, 0.043506, 0.580146, 1.404830, 0.041913, 0.257624, 1.511120, 0.041913, 0.257624, 1.511120, 0.150260, 0.282014, 1.525120, 0.321306, 0.531263, 1.149410, 0.041913, 0.281121, 1.582420, -0.066433, 0.281730, 1.525120, 0.182646, 0.306263, 1.539120, 0.090713, 0.547383, 1.392800, 0.248053, 0.604517, 1.233140, 0.248850, 0.594803, 1.240080, 0.054951, 0.553382, 1.420730, 0.026772, 0.542105, 1.427400, -0.006386, 0.549079, 1.391450, -0.160300, 0.593221, 1.246000, -0.165083, 0.594759, 1.239830, 0.231697, 0.398694, 1.194280, 0.202474, 0.389779, 1.198610, 0.041913, 0.328005, 1.091790, -0.121436, 0.391855, 1.193190, 0.041913, 0.328657, 1.087900, -0.146267, 0.396108, 1.200800 };
    Coordinate5.point = m_point;
    Coordinate5.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal1.m_parent = (X3dNode *)&IndexedFaceSet3;
    {
    static float m_vector[] = { 0.240520, -0.178855, 0.954024, 0.241302, -0.186810, 0.952300, 0.240520, -0.178855, 0.954024, 0.240520, -0.178855, 0.954024, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.293132, 0.577824, 0.761704, 0.293132, 0.577824, 0.761704, -0.000257, 0.510630, 0.859801, -0.540597, 0.550255, 0.636376, -0.540597, 0.550255, 0.636376, -0.627971, 0.485889, 0.607918, -0.741730, 0.393026, 0.543477, -0.741730, 0.393026, 0.543477, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, 0.273372, -0.961689, -0.020544, 0.273372, -0.961689, -0.020544, 0.274033, -0.961490, -0.021035, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, -0.237959, -0.155430, 0.958758, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.001166, 0.510636, 0.859796, -0.278860, 0.577795, 0.767066, -0.278860, 0.577795, 0.767066, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.871407, -0.488674, 0.042975, -0.871407, -0.488674, 0.042975, -0.867993, -0.495110, 0.038144, -0.168427, -0.936187, 0.308521, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.947812, -0.246434, 0.202292, 0.949402, -0.233910, 0.209575, 0.949402, -0.233910, 0.209575, 0.947812, -0.246434, 0.202292, 0.539886, 0.550710, 0.636586, 0.539886, 0.550710, 0.636586, 0.765835, 0.369611, 0.526198, 0.765835, 0.369611, 0.526198, 0.765820, 0.369627, 0.526210, 0.741377, 0.393368, 0.543712, 0.741377, 0.393368, 0.543712, 0.502593, 0.598896, 0.623477, 0.502593, 0.598896, 0.623477, -0.489526, 0.601019, 0.631775, -0.489526, 0.601019, 0.631775, 0.578916, -0.803148, -0.140751, 0.578916, -0.803148, -0.140751, 0.344423, -0.915028, -0.209994, 0.344423, -0.915028, -0.209994, 0.439693, -0.885759, -0.148667, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, -0.438743, -0.886217, -0.148742, -0.829634, -0.558259, 0.007340, -0.242173, -0.932158, -0.269134 };
    Normal1.vector = m_vector;
    Normal1.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet3.color = &Color3;
    IndexedFaceSet3.coord = &Coordinate5;
    IndexedFaceSet3.normal = &Normal1;
    IndexedFaceSet3.texCoord = NULL;
    IndexedFaceSet3.ccw = true;
    {
    static int m_colorIndex[] = { 1, 0, 11, 0, -1, 16, 0, 19, 0, 0, -1, 1, 1, 0, -1, 1, 0, 13, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 1, 0, 1, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 7, 0, 0, 9, 2, -1, 5, 0, 6, -1, 0, 0, 8, -1, 0, 3, 8, -1, 10, 0, 4, -1, 15, 0, 12, -1, 0, 14, 17, -1, 0, 18, 0, -1, 0, 23, 0, -1, 22, 0, 1, 25, -1, 1, 26, 24, -1, 1, 27, 29, -1, 0, 0, 21, -1, 28, 1, 0, 20, -1 };
    IndexedFaceSet3.colorIndex = m_colorIndex;
    IndexedFaceSet3.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet3.colorPerVertex = true;
    IndexedFaceSet3.convex = true;
    {
    static int m_coordIndex[] = { 3, 0, 19, 2, -1, 22, 5, 23, 4, 7, -1, 3, 10, 11, -1, 13, 8, 20, 7, -1, 13, 7, 4, -1, 13, 4, 14, -1, 6, 14, 4, -1, 13, 14, 9, -1, 3, 15, 0, -1, 3, 11, 15, -1, 12, 15, 11, -1, 17, 0, 15, 18, 1, -1, 16, 0, 17, -1, 15, 12, 18, -1, 12, 1, 18, -1, 19, 0, 16, -1, 21, 7, 20, -1, 7, 21, 22, -1, 4, 23, 6, -1, 11, 24, 12, -1, 24, 11, 10, 25, -1, 10, 26, 25, -1, 9, 27, 28, -1, 14, 6, 29, -1, 27, 9, 14, 29, -1 };
    IndexedFaceSet3.coordIndex = m_coordIndex;
    IndexedFaceSet3.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet3.creaseAngle = 2.942970;
    {
    static int m_normalIndex[] = { 0, 4, 9, 11, -1, 12, 14, 15, 17, 22, -1, 1, 27, 30, -1, 35, 39, 40, 23, -1, 36, 24, 18, -1, 37, 19, 42, -1, 47, 43, 20, -1, 38, 44, 50, -1, 2, 53, 5, -1, 3, 31, 54, -1, 58, 55, 32, -1, 62, 6, 56, 64, 67, -1, 69, 7, 63, -1, 57, 59, 65, -1, 60, 68, 66, -1, 10, 8, 70, -1, 71, 25, 41, -1, 26, 72, 13, -1, 21, 16, 48, -1, 33, 73, 61, -1, 74, 34, 28, 75, -1, 29, 77, 76, -1, 51, 78, 80, -1, 45, 49, 81, -1, 79, 52, 46, 82, -1 };
    IndexedFaceSet3.normalIndex = m_normalIndex;
    IndexedFaceSet3.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet3.normalPerVertex = true;
    IndexedFaceSet3.solid = true;
    {
    static int m_texCoordIndex[1];
    IndexedFaceSet3.texCoordIndex = NULL;
    IndexedFaceSet3.texCoordIndex_length = 0;
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet3.set_colorIndex = NULL;
    IndexedFaceSet3.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet3.set_coordIndex = NULL;
    IndexedFaceSet3.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet3.set_normalIndex = NULL;
    IndexedFaceSet3.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet3.set_texCoordIndex = NULL;
    IndexedFaceSet3.set_texCoordIndex_length = 0;
    }
    Shape3.appearance = &Appearance3;
    Shape3.geometry = &IndexedFaceSet3;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform2.rotation = m_rotation;
    Transform2.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -2.980230e-8, 5.005030e-3, -3.011850e-2 };
    Transform2.translation = m_translation;
    Transform2.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform2.bboxCenter = m_bboxCenter;
    Transform2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform2.bboxSize = m_bboxSize;
    Transform2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform2.center = m_center;
    Transform2.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform2.scale = m_scale;
    Transform2.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform2.scaleOrientation = m_scaleOrientation;
    Transform2.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape3 };
    Transform2.children = m_children;
    Transform2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform2.addChildren = NULL;
    Transform2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform2.removeChildren = NULL;
    Transform2.removeChildren_length = 0;
    }
    Transform3.route_sources =  NULL;
    Transform3.num_route_source = 0;
    Transform3.m_parent = (X3dNode *)NULL;
    Shape4.m_parent = (X3dNode *)NULL;
    Appearance4.m_parent = (X3dNode *)NULL;
    A0m_2.m_parent = (X3dNode *)NULL;
    A0m_2.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_2.diffuseColor = m_diffuseColor;
    A0m_2.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_2.emissiveColor = m_emissiveColor;
    A0m_2.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_2.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_2.specularColor = m_specularColor;
    A0m_2.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_2.transparency = 0;
    Appearance4.material = &A0m_2;
    Appearance4.texture = NULL;
    Appearance4.textureTransform = NULL;
    IndexedFaceSet4.setGlName(3);
    IndexedFaceSet4.m_parent = (X3dNode *)NULL;
    Color5.m_parent = (X3dNode *)&IndexedFaceSet4;
    {
    static float m_color[] = { 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794 };
    Color5.color = m_color;
    Color5.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate7.m_parent = (X3dNode *)&IndexedFaceSet4;
    {
    static float m_point[] = { 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180, 0.041613, 0.179757, -1.951840, 0.275296, 0.046906, -1.173840, 0.607257, 0.161629, -1.835830, 0.041613, 0.179757, -1.951840, -0.524042, 0.158129, -1.836490, -0.192141, 0.045295, -1.174140, 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180 };
    Coordinate7.point = m_point;
    Coordinate7.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal2.m_parent = (X3dNode *)&IndexedFaceSet4;
    {
    static float m_vector[] = { -0.958763, 0.012792, 0.283919, -0.181027, 0.455152, 0.871817, -0.110099, 0.301903, 0.946960, -0.110099, 0.301903, 0.946960, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.698330, -0.710853, 0.083806, -0.181027, 0.455152, 0.871817, -0.698330, -0.710853, 0.083806, 0.875466, 0.215616, 0.432514, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, 0.003126, -0.985533, -0.169456, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, 0.003126, -0.985533, -0.169456, -0.896416, 0.017432, 0.442870, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112 };
    Normal2.vector = m_vector;
    Normal2.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet4.color = &Color5;
    IndexedFaceSet4.coord = &Coordinate7;
    IndexedFaceSet4.normal = &Normal2;
    IndexedFaceSet4.texCoord = NULL;
    IndexedFaceSet4.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 1, 1, 0, -1, 0, 1, 1, -1, 2, 3, 2, -1, 5, 4, 4, -1 };
    IndexedFaceSet4.colorIndex = m_colorIndex;
    IndexedFaceSet4.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet4.colorPerVertex = true;
    IndexedFaceSet4.convex = true;
    {
    static int m_coordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet4.coordIndex = m_coordIndex;
    IndexedFaceSet4.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet4.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 2, 4, 7, -1, 2, 7, 0, -1, 3, 10, 12, -1, 3, 12, 5, -1, 11, 13, 8, -1, 11, 8, 1, -1, 17, 19, 22, -1, 17, 22, 15, -1, 25, 27, 20, -1, 25, 20, 18, -1, 26, 28, 23, -1, 26, 23, 16, -1, 14, 9, 6, -1, 29, 24, 21, -1, 30, 31, 32, -1, 33, 34, 35, -1 };
    IndexedFaceSet4.normalIndex = m_normalIndex;
    IndexedFaceSet4.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet4.normalPerVertex = true;
    IndexedFaceSet4.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet4.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet4.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet4.set_colorIndex = NULL;
    IndexedFaceSet4.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet4.set_coordIndex = NULL;
    IndexedFaceSet4.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet4.set_normalIndex = NULL;
    IndexedFaceSet4.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet4.set_texCoordIndex = NULL;
    IndexedFaceSet4.set_texCoordIndex_length = 0;
    }
    Shape4.appearance = &Appearance4;
    Shape4.geometry = &IndexedFaceSet4;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform3.rotation = m_rotation;
    Transform3.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0.179843, -9.580490e-2 };
    Transform3.translation = m_translation;
    Transform3.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform3.bboxCenter = m_bboxCenter;
    Transform3.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform3.bboxSize = m_bboxSize;
    Transform3.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform3.center = m_center;
    Transform3.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform3.scale = m_scale;
    Transform3.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform3.scaleOrientation = m_scaleOrientation;
    Transform3.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape4 };
    Transform3.children = m_children;
    Transform3.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform3.addChildren = NULL;
    Transform3.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform3.removeChildren = NULL;
    Transform3.removeChildren_length = 0;
    }
    Transform4.route_sources =  NULL;
    Transform4.num_route_source = 0;
    Transform4.m_parent = (X3dNode *)NULL;
    Shape5.m_parent = (X3dNode *)NULL;
    Appearance5.m_parent = (X3dNode *)NULL;
    Material3.m_parent = (X3dNode *)NULL;
    Material3.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material3.diffuseColor = m_diffuseColor;
    Material3.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material3.emissiveColor = m_emissiveColor;
    Material3.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material3.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material3.specularColor = m_specularColor;
    Material3.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material3.transparency = 6.382980e-2;
    Appearance5.material = &Material3;
    Appearance5.texture = NULL;
    Appearance5.textureTransform = NULL;
    IndexedFaceSet5.setGlName(4);
    IndexedFaceSet5.m_parent = (X3dNode *)NULL;
    Coordinate9.m_parent = (X3dNode *)&IndexedFaceSet5;
    {
    static float m_point[] = { 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.357975, 0.274623, 0.391110, 0.435755, 0.282992, -1.405890, 0.732023, 0.277202, -1.011950, 0.803608, 0.283498, -0.488598, 0.807795, 0.291706, -0.169135, 0.642283, 0.308596, 0.135106, -0.576839, 0.295092, 0.138182, -0.741419, 0.274624, -0.165225, -0.736057, 0.266519, -0.484712, -0.662406, 0.261798, -1.008430, -0.364349, 0.274033, -1.403870, 0.034173, 1.521740, 0.072271, 0.032379, 1.258290, 0.402776, 0.324052, 1.234020, 0.434518, 0.382959, 1.442650, 0.114314, 0.462972, 1.076210, 0.512842, 0.634192, 1.091710, 0.196483, 0.036047, 1.641430, -0.319814, 0.468123, 1.540110, -0.266019, 0.780864, 1.161780, -0.151701, 0.753362, 0.779925, 0.302416, 0.875071, 0.752977, 0.006324, 0.646152, 0.462049, 0.391531, 0.036992, 1.662050, -0.688097, 0.522545, 1.549060, -0.628109, 0.873437, 1.148710, -0.497152, 0.978519, 0.692729, -0.320894, 0.036779, 1.562450, -1.012960, 0.537744, 1.451000, -0.953720, 0.879328, 1.083370, -0.818803, 0.983199, 0.632646, -0.644566, 0.672590, 0.889258, -1.304530, 0.877021, 0.531470, -1.119920, 0.439314, 0.644779, -1.532240, 0.477913, 0.461302, -1.461330, 0.028006, 0.456359, -1.522590, 0.025463, 0.201297, -1.389500, -0.259292, 1.233740, 0.434518, -0.316266, 1.434790, 0.116078, -0.398215, 1.075930, 0.512842, -0.569430, 1.091430, 0.196483, -0.398142, 1.530440, -0.263833, -0.718144, 1.145240, -0.147918, -0.688602, 0.779642, 0.302416, -0.820289, 0.734310, 0.010603, -0.581395, 0.461765, 0.391532, -0.450917, 1.538220, -0.625652, -0.809539, 1.130170, -0.492904, -0.923472, 0.671821, -0.316093, -0.466519, 1.439820, -0.951186, -0.815262, 1.064720, -0.814525, -0.927883, 0.611639, -0.639742, -0.391565, 1.162260, -1.429880, -0.821517, 0.512768, -1.115630, -0.422165, 0.451259, -1.459060, 0.032379, 1.315590, 1.240890, 0.228842, 1.352260, 0.975027, 0.032379, 1.384890, 0.907439, 0.314142, 1.285900, 1.111910, 0.314142, 1.207010, 1.274610, 0.228842, 1.140650, 1.411480, 0.032379, 1.107740, 1.479070, 0.032379, 1.284680, 0.661614, 0.313972, 1.236840, 0.760583, 0.438872, 1.139660, 0.961005, 0.438872, 0.959292, 1.192490, 0.313972, 0.916734, 1.377380, 0.032379, 0.878853, 1.498640, 0.457846, 0.598657, 1.016000, 0.325246, 0.449644, 1.073590, -0.164085, 1.351980, 0.975027, -0.249383, 1.285620, 1.111910, -0.249383, 1.206730, 1.274610, -0.164085, 1.140370, 1.411480, -0.249218, 1.236560, 0.760583, -0.374117, 1.139380, 0.961005, -0.374117, 0.959003, 1.192490, -0.249218, 0.916450, 1.377380, -0.412717, 0.598373, 1.016000, -0.277239, 0.449360, 1.073590, 0.030998, 0.455945, -1.522540, 0.200163, 0.782040, -2.021660, 0.022803, 0.816558, -2.021660, -0.153194, 0.791256, -2.021660, 0.030997, 0.455945, -1.522540, -0.638224, 0.898115, -1.302900, -0.394524, 0.650450, -1.531170, 0.012304, 1.278610, -1.485710, 0.022803, 0.816558, -2.021660, 0.012304, 1.278610, -1.485710, 0.452747, 1.152960, -1.430970 };
    Coordinate9.point = m_point;
    Coordinate9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal3.m_parent = (X3dNode *)&IndexedFaceSet5;
    {
    static float m_vector[] = { -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940694, 0.149539, 0.304520, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.834986, -0.422306, 0.352783, 0.834986, -0.422306, 0.352783, 0.837532, -0.417196, 0.352827, 0.834986, -0.422306, 0.352783, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824726, 0.564327, 0.036905, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983866, 0.093656, 0.152433, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.919028, -0.327107, 0.219973, 0.919028, -0.327107, 0.219973, 0.919030, -0.327639, 0.219171, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857347, 0.447616, -0.254158, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.871918, -0.485933, -0.060240, 0.871918, -0.485933, -0.060240, 0.871819, -0.486088, -0.060419, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.748933, -0.523293, -0.406527, 0.748933, -0.523293, -0.406527, 0.748998, -0.522697, -0.407174, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.687379, -0.282285, -0.669197, 0.682617, -0.274976, -0.677069, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.373882, -0.393399, -0.839911, 0.373882, -0.393399, -0.839911, 0.372714, -0.393753, -0.840264, 0.043708, -0.372748, -0.926903, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.001617, -0.462617, -0.886557, 0.001617, -0.462617, -0.886557, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936815, 0.032489, 0.348312, -0.936814, 0.032493, 0.348315, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.753712, -0.525849, 0.394210, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.858943, 0.498866, 0.115538, -0.859015, 0.498704, 0.115703, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.909057, -0.352626, 0.221969, -0.909016, -0.354059, 0.219844, -0.909016, -0.354059, 0.219844, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.972656, 0.230360, -0.029579, -0.917591, -0.388391, 0.084727, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.926292, 0.088999, -0.366146, -0.914069, 0.304729, -0.267615, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.914069, 0.304729, -0.267615, -0.657343, -0.174205, -0.733180, -0.694040, -0.019618, -0.719669, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.053473, -0.461524, -0.885515, -0.465415, -0.809935, 0.356924, -0.465415, -0.809935, 0.356924, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.301321, -0.764284, 0.570154, 0.512027, -0.788035, 0.341804, 0.512027, -0.788035, 0.341804, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, 0.115620, -0.804235, -0.582956, -0.370219, -0.828569, -0.420014, 0.115620, -0.804235, -0.582956, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.117789, 0.819323, -0.561103, -0.725452, 0.412604, -0.550888, -0.112111, 0.280660, -0.953237, -0.115216, 0.288185, -0.950618, -0.115216, 0.288185, -0.950618, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.694040, -0.019618, -0.719669, 0.113269, -0.848919, -0.516243, 0.113269, -0.848919, -0.516243, 0.109595, 0.280897, -0.953460, 0.110751, 0.283846, -0.952452, 0.109595, 0.280897, -0.953460, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.285260, 0.728679, -0.622619, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.509719, 0.651105, -0.562360, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.046990, 0.842110, -0.537256 };
    Normal3.vector = m_vector;
    Normal3.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet5.color = NULL;
    IndexedFaceSet5.coord = &Coordinate9;
    IndexedFaceSet5.normal = &Normal3;
    IndexedFaceSet5.texCoord = NULL;
    IndexedFaceSet5.ccw = true;
    {
    static int m_colorIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet5.colorIndex = m_colorIndex;
    IndexedFaceSet5.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet5.colorPerVertex = true;
    IndexedFaceSet5.convex = true;
    {
    static int m_coordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet5.coordIndex = m_coordIndex;
    IndexedFaceSet5.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet5.creaseAngle = 3.104420;
    {
    static int m_normalIndex[] = { 0, 6, 12, -1, 13, 18, 1, -1, 19, 14, 24, -1, 25, 30, 20, -1, 36, 2, 21, -1, 22, 42, 37, -1, 43, 23, 31, -1, 32, 48, 44, -1, 49, 33, 54, -1, 55, 60, 50, -1, 61, 56, 66, -1, 67, 72, 62, -1, 76, 38, 45, -1, 46, 82, 77, -1, 83, 47, 51, -1, 52, 88, 84, -1, 89, 53, 63, -1, 64, 94, 90, -1, 95, 65, 73, -1, 74, 100, 96, -1, 103, 78, 85, -1, 86, 109, 104, -1, 110, 87, 91, -1, 92, 115, 111, -1, 116, 93, 97, -1, 98, 121, 117, -1, 122, 99, 101, -1, 102, 127, 123, -1, 130, 118, 124, -1, 125, 136, 131, -1, 137, 126, 128, -1, 129, 142, 138, -1, 145, 132, 139, -1, 140, 150, 146, -1, 151, 141, 143, -1, 144, 155, 152, -1, 158, 147, 153, -1, 159, 154, 156, -1, 160, 157, 164, -1, 3, 166, 7, -1, 167, 4, 172, -1, 173, 178, 168, -1, 179, 174, 184, -1, 39, 175, 5, -1, 176, 40, 190, -1, 191, 185, 177, -1, 186, 192, 196, -1, 197, 202, 187, -1, 203, 198, 208, -1, 209, 214, 204, -1, 215, 210, 220, -1, 79, 193, 41, -1, 194, 80, 224, -1, 225, 199, 195, -1, 200, 226, 230, -1, 231, 211, 201, -1, 212, 232, 236, -1, 237, 221, 213, -1, 222, 238, 242, -1, 105, 227, 81, -1, 228, 106, 245, -1, 246, 233, 229, -1, 234, 247, 251, -1, 252, 239, 235, -1, 240, 253, 257, -1, 258, 243, 241, -1, 244, 259, 263, -1, 266, 254, 248, -1, 272, 264, 260, -1, 265, 273, 278, -1, 281, 279, 274, -1, 280, 282, 286, -1, 161, 287, 283, -1, 162, 165, 288, -1, 223, 289, 216, -1, 291, 217, 290, -1, 75, 68, 297, -1, 299, 309, 314, -1, 300, 318, 310, -1, 301, 323, 319, -1, 302, 328, 324, -1, 303, 333, 329, -1, 339, 315, 311, -1, 312, 345, 340, -1, 346, 313, 320, -1, 321, 351, 347, -1, 352, 322, 325, -1, 326, 357, 353, -1, 358, 327, 330, -1, 331, 363, 359, -1, 364, 332, 334, -1, 335, 369, 365, -1, 8, 341, 348, -1, 349, 15, 9, -1, 16, 350, 354, -1, 355, 26, 17, -1, 27, 356, 360, -1, 361, 375, 28, -1, 376, 362, 366, -1, 367, 381, 377, -1, 382, 368, 370, -1, 371, 292, 383, -1, 34, 29, 378, -1, 379, 57, 35, -1, 58, 380, 384, -1, 385, 69, 59, -1, 70, 386, 293, -1, 304, 316, 387, -1, 305, 388, 392, -1, 306, 393, 397, -1, 307, 398, 402, -1, 308, 403, 336, -1, 342, 389, 317, -1, 390, 343, 407, -1, 408, 394, 391, -1, 395, 409, 413, -1, 414, 399, 396, -1, 400, 415, 419, -1, 420, 404, 401, -1, 405, 421, 425, -1, 426, 337, 406, -1, 338, 427, 372, -1, 10, 410, 344, -1, 411, 11, 169, -1, 170, 416, 412, -1, 417, 171, 180, -1, 181, 422, 418, -1, 423, 182, 431, -1, 432, 428, 424, -1, 429, 433, 437, -1, 438, 373, 430, -1, 374, 439, 294, -1, 188, 434, 183, -1, 435, 189, 205, -1, 206, 440, 436, -1, 441, 207, 218, -1, 219, 295, 442, -1, 71, 296, 298, -1, 443, 445, 450, -1, 453, 446, 459, -1, 444, 460, 447, -1, 464, 466, 469, -1, 467, 474, 475, -1, 476, 470, 468, -1, 112, 119, 477, -1, 120, 133, 478, -1, 471, 479, 134, -1, 255, 267, 454, -1, 261, 256, 455, -1, 275, 262, 456, -1, 107, 113, 481, -1, 249, 108, 482, -1, 268, 250, 483, -1, 114, 480, 484, -1, 451, 448, 269, -1, 449, 457, 270, -1, 485, 452, 271, -1, 284, 276, 461, -1, 163, 285, 462, -1, 277, 458, 463, -1, 465, 472, 148, -1, 473, 135, 149, -1 };
    IndexedFaceSet5.normalIndex = m_normalIndex;
    IndexedFaceSet5.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet5.normalPerVertex = true;
    IndexedFaceSet5.solid = true;
    {
    static int m_texCoordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet5.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet5.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet5.set_colorIndex = NULL;
    IndexedFaceSet5.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet5.set_coordIndex = NULL;
    IndexedFaceSet5.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet5.set_normalIndex = NULL;
    IndexedFaceSet5.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet5.set_texCoordIndex = NULL;
    IndexedFaceSet5.set_texCoordIndex_length = 0;
    }
    Shape5.appearance = &Appearance5;
    Shape5.geometry = &IndexedFaceSet5;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform4.rotation = m_rotation;
    Transform4.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform4.translation = m_translation;
    Transform4.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform4.bboxCenter = m_bboxCenter;
    Transform4.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform4.bboxSize = m_bboxSize;
    Transform4.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform4.center = m_center;
    Transform4.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform4.scale = m_scale;
    Transform4.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform4.scaleOrientation = m_scaleOrientation;
    Transform4.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape5 };
    Transform4.children = m_children;
    Transform4.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform4.addChildren = NULL;
    Transform4.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform4.removeChildren = NULL;
    Transform4.removeChildren_length = 0;
    }
    Transform5.route_sources =  NULL;
    Transform5.num_route_source = 0;
    Transform5.m_parent = (X3dNode *)NULL;
    Shape6.m_parent = (X3dNode *)NULL;
    Appearance6.m_parent = (X3dNode *)NULL;
    Material4.m_parent = (X3dNode *)NULL;
    Material4.ambientIntensity = 1;
    {
    static float m_diffuseColor[] = { 1, 1, 1 };
    Material4.diffuseColor = m_diffuseColor;
    Material4.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0.691489, 0.691489, 0.691489 };
    Material4.emissiveColor = m_emissiveColor;
    Material4.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material4.shininess = 1;
    {
    static float m_specularColor[] = { 0.765957, 0.765957, 0.765957 };
    Material4.specularColor = m_specularColor;
    Material4.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material4.transparency = 0.372340;
    Appearance6.material = &Material4;
    Appearance6.texture = NULL;
    Appearance6.textureTransform = NULL;
    IndexedFaceSet6.setGlName(5);
    IndexedFaceSet6.m_parent = (X3dNode *)NULL;
    Coordinate11.m_parent = (X3dNode *)&IndexedFaceSet6;
    {
    static float m_point[] = { 0.023775, 0.125771, 0.096370, -0.736057, 0.266519, -0.484712, 0.019864, 0.155999, -0.238411, -0.741419, 0.274624, -0.165225, 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.033239, 0.290582, 0.404469, -0.576839, 0.295092, 0.138182, 0.357975, 0.274623, 0.391110, 0.642283, 0.308596, 0.135106, 0.026537, 0.145128, 0.290011, 0.807795, 0.291706, -0.169135, 0.803608, 0.283498, -0.488598, -0.662406, 0.261798, -1.008430, 0.019557, 0.169360, -0.585595, -0.364349, 0.274033, -1.403870, 0.034997, 0.201297, -1.394460, 0.732023, 0.277202, -1.011950, 0.435755, 0.282992, -1.405890 };
    Coordinate11.point = m_point;
    Coordinate11.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal4.m_parent = (X3dNode *)&IndexedFaceSet6;
    {
    static float m_vector[] = { -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.115983, -0.989347, -0.087975, -0.197996, -0.979966, 0.021537, -0.141172, -0.989925, -0.010929, -0.115983, -0.989347, -0.087975, 0.131644, -0.987142, -0.090669, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, 0.127132, -0.991148, -0.038256, 0.157553, -0.987463, -0.009671, -0.228866, -0.968557, 0.097558, -0.229441, -0.968303, 0.098722, -0.229441, -0.968303, 0.098722, 0.003121, -0.997858, 0.065337, 0.003121, -0.997858, 0.065337, 0.057747, -0.997198, -0.047547, 0.057747, -0.997198, -0.047547, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, -0.193939, -0.928966, 0.315294, -0.193939, -0.928966, 0.315294, -0.194333, -0.929016, 0.314902, -0.046282, -0.996765, 0.065709, -0.056059, -0.979739, -0.192273, 0.204185, -0.934007, 0.293153, 0.205250, -0.934031, 0.292331, 0.205250, -0.934031, 0.292331, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.256610, -0.958616, 0.123319, 0.257004, -0.958385, 0.124286, 0.256610, -0.958616, 0.123319, 0.169604, -0.985103, -0.028410, 0.169720, -0.985078, -0.028591, 0.169604, -0.985103, -0.028410, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, -0.178067, -0.983112, -0.042216, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, 0.127132, -0.991148, -0.038256, 0.198666, -0.979446, -0.034880, 0.097470, -0.991352, -0.087874, -0.178067, -0.983112, -0.042216, -0.078733, -0.992819, -0.090062, 0.010487, -0.999174, -0.039251, 0.010487, -0.999174, -0.039251, 0.131753, -0.990507, -0.039208, 0.132081, -0.990482, -0.038739, 0.131753, -0.990507, -0.039208, 0.148313, -0.987028, -0.061478, 0.148313, -0.987028, -0.061478 };
    Normal4.vector = m_vector;
    Normal4.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet6.color = NULL;
    IndexedFaceSet6.coord = &Coordinate11;
    IndexedFaceSet6.normal = &Normal4;
    IndexedFaceSet6.texCoord = NULL;
    IndexedFaceSet6.ccw = true;
    {
    static int m_colorIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet6.colorIndex = m_colorIndex;
    IndexedFaceSet6.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet6.colorPerVertex = true;
    IndexedFaceSet6.convex = true;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet6.coordIndex = m_coordIndex;
    IndexedFaceSet6.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet6.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 0, 6, 9, -1, 7, 1, 15, -1, 18, 20, 22, -1, 28, 23, 21, -1, 19, 24, 31, -1, 33, 32, 25, -1, 36, 16, 2, -1, 26, 37, 34, -1, 27, 29, 38, -1, 17, 39, 30, -1, 40, 3, 42, -1, 43, 35, 41, -1, 4, 10, 45, -1, 46, 44, 5, -1, 48, 11, 8, -1, 51, 57, 59, -1, 12, 49, 52, -1, 58, 53, 50, -1, 13, 54, 61, -1, 62, 47, 14, -1, 55, 60, 64, -1, 65, 63, 56, -1 };
    IndexedFaceSet6.normalIndex = m_normalIndex;
    IndexedFaceSet6.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet6.normalPerVertex = true;
    IndexedFaceSet6.solid = true;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet6.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet6.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet6.set_colorIndex = NULL;
    IndexedFaceSet6.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet6.set_coordIndex = NULL;
    IndexedFaceSet6.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet6.set_normalIndex = NULL;
    IndexedFaceSet6.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet6.set_texCoordIndex = NULL;
    IndexedFaceSet6.set_texCoordIndex_length = 0;
    }
    Shape6.appearance = &Appearance6;
    Shape6.geometry = &IndexedFaceSet6;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform5.rotation = m_rotation;
    Transform5.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform5.translation = m_translation;
    Transform5.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform5.bboxCenter = m_bboxCenter;
    Transform5.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform5.bboxSize = m_bboxSize;
    Transform5.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform5.center = m_center;
    Transform5.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform5.scale = m_scale;
    Transform5.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform5.scaleOrientation = m_scaleOrientation;
    Transform5.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape6 };
    Transform5.children = m_children;
    Transform5.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform5.addChildren = NULL;
    Transform5.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform5.removeChildren = NULL;
    Transform5.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform2, &Transform3, &Transform4, &Transform5 };
    Group3.children = m_children;
    Group3.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group3.bboxCenter = m_bboxCenter;
    Group3.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group3.bboxSize = m_bboxSize;
    Group3.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group3.addChildren = NULL;
    Group3.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group3.removeChildren = NULL;
    Group3.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group3 };
    Inline2.children = m_children;
    Inline2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline2.bboxCenter = m_bboxCenter;
    Inline2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline2.bboxSize = m_bboxSize;
    Inline2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline2.addChildren = NULL;
    Inline2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline2.removeChildren = NULL;
    Inline2.removeChildren_length = 0;
    }
    Transform6.route_sources =  NULL;
    Transform6.num_route_source = 0;
    Transform6.m_parent = (X3dNode *)NULL;
    Transform7.route_sources =  NULL;
    Transform7.num_route_source = 0;
    Transform7.m_parent = (X3dNode *)NULL;
    Transform8.route_sources =  NULL;
    Transform8.num_route_source = 0;
    Transform8.m_parent = (X3dNode *)NULL;
    Shape7.m_parent = (X3dNode *)NULL;
    Appearance7.m_parent = (X3dNode *)NULL;
    Material5.m_parent = (X3dNode *)NULL;
    Material5.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material5.diffuseColor = m_diffuseColor;
    Material5.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material5.emissiveColor = m_emissiveColor;
    Material5.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material5.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material5.specularColor = m_specularColor;
    Material5.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material5.transparency = 0;
    Appearance7.material = &Material5;
    Appearance7.texture = NULL;
    Appearance7.textureTransform = NULL;
    IndexedFaceSet7.setGlName(6);
    IndexedFaceSet7.m_parent = (X3dNode *)NULL;
    Coordinate13.m_parent = (X3dNode *)&IndexedFaceSet7;
    {
    static float m_point[] = { 0, 0, -1, -0.382683, 0, -0.923880, -0.707107, 0, -0.707107, -0.923880, 0, -0.382683, -1, 0, 0, -0.923880, 0, 0.382684, -0.707107, 0, 0.707107, -0.382683, 0, 0.923880, 0, 0, 1, 0.382683, 0, 0.923880, 0.707107, 0, 0.707107, 0.923880, 0, 0.382683, 1, 0, 0, 0.923879, 0, -0.382684, 0.707107, 0, -0.707107, 0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, -0.363954, 0.309017, -0.878662, -0.672499, 0.309017, -0.672499, -0.878662, 0.309017, -0.363954, -0.951057, 0.309017, 0, -0.878662, 0.309017, 0.363954, -0.672499, 0.309017, 0.672499, -0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, 0.363954, 0.309017, 0.878662, 0.672499, 0.309017, 0.672498, 0.878662, 0.309017, 0.363953, 0.951057, 0.309017, 0, 0.878662, 0.309017, -0.363954, 0.672498, 0.309017, -0.672499, 0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, -0.309597, 0.587785, -0.747434, -0.572061, 0.587785, -0.572061, -0.747434, 0.587785, -0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, 0.309597, -0.572061, 0.587785, 0.572061, -0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, 0.309597, 0.587785, 0.747434, 0.572061, 0.587785, 0.572061, 0.747434, 0.587785, 0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, -0.309598, 0.572061, 0.587785, -0.572062, 0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, -0.224936, 0.809017, -0.543043, -0.415627, 0.809017, -0.415627, -0.543043, 0.809017, -0.224936, -0.587785, 0.809017, 0, -0.543043, 0.809017, 0.224936, -0.415627, 0.809017, 0.415627, -0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, 0.224936, 0.809017, 0.543043, 0.415627, 0.809017, 0.415627, 0.543043, 0.809017, 0.224935, 0.587785, 0.809017, 0, 0.543043, 0.809017, -0.224936, 0.415627, 0.809017, -0.415627, 0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, -0.118256, 0.951057, -0.285494, -0.218508, 0.951057, -0.218508, -0.285494, 0.951057, -0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, 0.118256, -0.218508, 0.951057, 0.218508, -0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, 0.118256, 0.951057, 0.285494, 0.218508, 0.951057, 0.218508, 0.285495, 0.951057, 0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, -0.118256, 0.218508, 0.951057, -0.218508, 0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate13.point = m_point;
    Coordinate13.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal5.m_parent = (X3dNode *)&IndexedFaceSet7;
    {
    static float m_vector[] = { -0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, -0.185864, 0.303889, -0.934401, -0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377972, 0.156433, -0.912505, -0.377972, 0.156433, -0.912505, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.698401, 0.156433, -0.698401, -0.698401, 0.156433, -0.698401, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.912505, 0.156434, -0.377972, -0.912505, 0.156434, -0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987689, 0.156434, 0, -0.987689, 0.156434, 0, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.912505, 0.156434, 0.377972, -0.912505, 0.156434, 0.377972, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.698401, 0.156434, 0.698402, -0.698401, 0.156434, 0.698402, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377971, 0.156434, 0.912505, -0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377973, 0.156433, 0.912505, 0.377973, 0.156433, 0.912505, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.698402, 0.156434, 0.698400, 0.698402, 0.156434, 0.698400, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.912505, 0.156435, 0.377972, 0.912505, 0.156435, 0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987688, 0.156434, 0, 0.987688, 0.156434, 0, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.912505, 0.156433, -0.377973, 0.912505, 0.156433, -0.377973, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.698401, 0.156434, -0.698401, 0.698401, 0.156434, -0.698401, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377973, 0.156435, -0.912505, 0.377973, 0.156435, -0.912505, 0.104474, 0.325528, -0.939743, 0.105800, 0.318418, -0.942028, 0.104474, 0.325528, -0.939743, 0.104474, 0.325528, -0.939743, 0.097141, 0.233062, -0.967598, 0.097141, 0.233062, -0.967598, -0.158828, 0.580683, -0.798486, -0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.090247, 0.600810, -0.794282, 0.091515, 0.594767, -0.798672, 0.090247, 0.600810, -0.794282, 0.090247, 0.600810, -0.794282, -0.116055, 0.803815, -0.583449, -0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.067135, 0.818259, -0.570916, 0.068222, 0.813901, -0.576985, 0.067135, 0.818259, -0.570916, 0.067135, 0.818259, -0.570916, -0.061299, 0.949355, -0.308167, -0.061299, 0.949355, -0.308167, 0, 1, 0, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137550, 0.933532, -0.331056, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253769, 0.933532, -0.253218, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331354, 0.933532, -0.136830, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, 0.000389, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331056, 0.933532, 0.137550, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253218, 0.933532, 0.253769, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.136830, 0.933532, 0.331354, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, 0.000389, 0.933532, 0.358494, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137550, 0.933532, 0.331056, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253768, 0.933532, 0.253219, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331354, 0.933532, 0.136830, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, -0.000391, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331056, 0.933532, -0.137550, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253218, 0.933532, -0.253769, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.136830, 0.933532, -0.331354, 0.042033, 0.942951, -0.330266, 0.042048, 0.942867, -0.330504, 0.042033, 0.942951, -0.330266, 0, 1, 0, -0.031102, 0.987211, -0.156357, -0.088569, 0.987211, -0.132553, -0.132553, 0.987211, -0.088569, -0.156357, 0.987211, -0.031102, -0.156357, 0.987211, 0.031102, -0.132553, 0.987211, 0.088569, -0.088569, 0.987211, 0.132553, -0.031102, 0.987211, 0.156357, 0.031102, 0.987211, 0.156357, 0.088569, 0.987211, 0.132553, 0.132553, 0.987211, 0.088570, 0.156357, 0.987211, 0.031101, 0.156357, 0.987211, -0.031102, 0.132553, 0.987211, -0.088569, 0.088569, 0.987211, -0.132553, 0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal5.vector = m_vector;
    Normal5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate3.m_parent = (X3dNode *)&IndexedFaceSet7;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate3.point = m_point;
    TextureCoordinate3.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet7.color = NULL;
    IndexedFaceSet7.coord = &Coordinate13;
    IndexedFaceSet7.normal = &Normal5;
    IndexedFaceSet7.texCoord = &TextureCoordinate3;
    IndexedFaceSet7.ccw = false;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet7.colorIndex = m_colorIndex;
    IndexedFaceSet7.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet7.colorPerVertex = true;
    IndexedFaceSet7.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet7.coordIndex = m_coordIndex;
    IndexedFaceSet7.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet7.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet7.normalIndex = m_normalIndex;
    IndexedFaceSet7.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet7.normalPerVertex = true;
    IndexedFaceSet7.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet7.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet7.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet7.set_colorIndex = NULL;
    IndexedFaceSet7.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet7.set_coordIndex = NULL;
    IndexedFaceSet7.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet7.set_normalIndex = NULL;
    IndexedFaceSet7.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet7.set_texCoordIndex = NULL;
    IndexedFaceSet7.set_texCoordIndex_length = 0;
    }
    Shape7.appearance = &Appearance7;
    Shape7.geometry = &IndexedFaceSet7;
    Transform9.route_sources =  NULL;
    Transform9.num_route_source = 0;
    Transform9.m_parent = (X3dNode *)NULL;
    Shape8.m_parent = (X3dNode *)NULL;
    Appearance8.m_parent = (X3dNode *)NULL;
    Material6.m_parent = (X3dNode *)NULL;
    Material6.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material6.diffuseColor = m_diffuseColor;
    Material6.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material6.emissiveColor = m_emissiveColor;
    Material6.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material6.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material6.specularColor = m_specularColor;
    Material6.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material6.transparency = 0;
    Appearance8.material = &Material6;
    Appearance8.texture = NULL;
    Appearance8.textureTransform = NULL;
    IndexedFaceSet8.setGlName(7);
    IndexedFaceSet8.m_parent = (X3dNode *)NULL;
    Coordinate15.m_parent = (X3dNode *)&IndexedFaceSet8;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate15.point = m_point;
    Coordinate15.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal6.m_parent = (X3dNode *)&IndexedFaceSet8;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal6.vector = m_vector;
    Normal6.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate5.m_parent = (X3dNode *)&IndexedFaceSet8;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate5.point = m_point;
    TextureCoordinate5.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet8.color = NULL;
    IndexedFaceSet8.coord = &Coordinate15;
    IndexedFaceSet8.normal = &Normal6;
    IndexedFaceSet8.texCoord = &TextureCoordinate5;
    IndexedFaceSet8.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet8.colorIndex = m_colorIndex;
    IndexedFaceSet8.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet8.colorPerVertex = true;
    IndexedFaceSet8.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet8.coordIndex = m_coordIndex;
    IndexedFaceSet8.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet8.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet8.normalIndex = m_normalIndex;
    IndexedFaceSet8.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet8.normalPerVertex = true;
    IndexedFaceSet8.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet8.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet8.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet8.set_colorIndex = NULL;
    IndexedFaceSet8.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet8.set_coordIndex = NULL;
    IndexedFaceSet8.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet8.set_normalIndex = NULL;
    IndexedFaceSet8.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet8.set_texCoordIndex = NULL;
    IndexedFaceSet8.set_texCoordIndex_length = 0;
    }
    Shape8.appearance = &Appearance8;
    Shape8.geometry = &IndexedFaceSet8;
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 0 };
    Transform9.rotation = m_rotation;
    Transform9.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.220000, 0.440000, 0.160000 };
    Transform9.translation = m_translation;
    Transform9.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform9.bboxCenter = m_bboxCenter;
    Transform9.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform9.bboxSize = m_bboxSize;
    Transform9.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform9.center = m_center;
    Transform9.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform9.scale = m_scale;
    Transform9.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform9.scaleOrientation = m_scaleOrientation;
    Transform9.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape8 };
    Transform9.children = m_children;
    Transform9.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform9.addChildren = NULL;
    Transform9.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform9.removeChildren = NULL;
    Transform9.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 1.578764 };
    Transform8.rotation = m_rotation;
    Transform8.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform8.translation = m_translation;
    Transform8.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform8.bboxCenter = m_bboxCenter;
    Transform8.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform8.bboxSize = m_bboxSize;
    Transform8.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform8.center = m_center;
    Transform8.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform8.scale = m_scale;
    Transform8.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform8.scaleOrientation = m_scaleOrientation;
    Transform8.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape7, &Transform9 };
    Transform8.children = m_children;
    Transform8.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform8.addChildren = NULL;
    Transform8.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform8.removeChildren = NULL;
    Transform8.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.565552, -0.792356, -0.228743, 0.826199 };
    Transform7.rotation = m_rotation;
    Transform7.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.109765, 0.715337, 1.294240 };
    Transform7.translation = m_translation;
    Transform7.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform7.bboxCenter = m_bboxCenter;
    Transform7.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform7.bboxSize = m_bboxSize;
    Transform7.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform7.center = m_center;
    Transform7.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform7.scale = m_scale;
    Transform7.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform7.scaleOrientation = m_scaleOrientation;
    Transform7.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform8 };
    Transform7.children = m_children;
    Transform7.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform7.addChildren = NULL;
    Transform7.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform7.removeChildren = NULL;
    Transform7.removeChildren_length = 0;
    }
    Transform10.route_sources =  NULL;
    Transform10.num_route_source = 0;
    Transform10.m_parent = (X3dNode *)NULL;
    Transform11.route_sources =  NULL;
    Transform11.num_route_source = 0;
    Transform11.m_parent = (X3dNode *)NULL;
    Shape9.m_parent = (X3dNode *)NULL;
    Appearance9.m_parent = (X3dNode *)NULL;
    Material7.m_parent = (X3dNode *)NULL;
    Material7.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material7.diffuseColor = m_diffuseColor;
    Material7.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material7.emissiveColor = m_emissiveColor;
    Material7.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material7.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material7.specularColor = m_specularColor;
    Material7.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material7.transparency = 0;
    Appearance9.material = &Material7;
    Appearance9.texture = NULL;
    Appearance9.textureTransform = NULL;
    IndexedFaceSet9.setGlName(8);
    IndexedFaceSet9.m_parent = (X3dNode *)NULL;
    Coordinate17.m_parent = (X3dNode *)&IndexedFaceSet9;
    {
    static float m_point[] = { 0, 0, -1, 0.382683, 0, -0.923880, 0.707107, 0, -0.707107, 0.923880, 0, -0.382683, 1, 0, 0, 0.923880, 0, 0.382684, 0.707107, 0, 0.707107, 0.382683, 0, 0.923880, 0, 0, 1, -0.382683, 0, 0.923880, -0.707107, 0, 0.707107, -0.923880, 0, 0.382683, -1, 0, 0, -0.923879, 0, -0.382684, -0.707107, 0, -0.707107, -0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, 0.363954, 0.309017, -0.878662, 0.672499, 0.309017, -0.672499, 0.878662, 0.309017, -0.363954, 0.951057, 0.309017, 0, 0.878662, 0.309017, 0.363954, 0.672499, 0.309017, 0.672499, 0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, -0.363954, 0.309017, 0.878662, -0.672499, 0.309017, 0.672498, -0.878662, 0.309017, 0.363953, -0.951057, 0.309017, 0, -0.878662, 0.309017, -0.363954, -0.672498, 0.309017, -0.672499, -0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, 0.309597, 0.587785, -0.747434, 0.572061, 0.587785, -0.572061, 0.747434, 0.587785, -0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, 0.309597, 0.572061, 0.587785, 0.572061, 0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, -0.309597, 0.587785, 0.747434, -0.572061, 0.587785, 0.572061, -0.747434, 0.587785, 0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, -0.309598, -0.572061, 0.587785, -0.572062, -0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, 0.224936, 0.809017, -0.543043, 0.415627, 0.809017, -0.415627, 0.543043, 0.809017, -0.224936, 0.587785, 0.809017, 0, 0.543043, 0.809017, 0.224936, 0.415627, 0.809017, 0.415627, 0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, -0.224936, 0.809017, 0.543043, -0.415627, 0.809017, 0.415627, -0.543043, 0.809017, 0.224935, -0.587785, 0.809017, 0, -0.543043, 0.809017, -0.224936, -0.415627, 0.809017, -0.415627, -0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, 0.118256, 0.951057, -0.285494, 0.218508, 0.951057, -0.218508, 0.285494, 0.951057, -0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, 0.118256, 0.218508, 0.951057, 0.218508, 0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, -0.118256, 0.951057, 0.285494, -0.218508, 0.951057, 0.218508, -0.285495, 0.951057, 0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, -0.118256, -0.218508, 0.951057, -0.218508, -0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate17.point = m_point;
    Coordinate17.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal7.m_parent = (X3dNode *)&IndexedFaceSet9;
    {
    static float m_vector[] = { 0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, 0.185864, 0.303889, -0.934401, 0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377972, 0.156433, -0.912505, 0.377972, 0.156433, -0.912505, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.698401, 0.156433, -0.698401, 0.698401, 0.156433, -0.698401, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.912505, 0.156434, -0.377972, 0.912505, 0.156434, -0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987689, 0.156434, 0, 0.987689, 0.156434, 0, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.912505, 0.156434, 0.377972, 0.912505, 0.156434, 0.377972, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.698401, 0.156434, 0.698402, 0.698401, 0.156434, 0.698402, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377971, 0.156434, 0.912505, 0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377973, 0.156433, 0.912505, -0.377973, 0.156433, 0.912505, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.698402, 0.156434, 0.698400, -0.698402, 0.156434, 0.698400, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.912505, 0.156435, 0.377972, -0.912505, 0.156435, 0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987688, 0.156434, 0, -0.987688, 0.156434, 0, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.912505, 0.156433, -0.377973, -0.912505, 0.156433, -0.377973, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.698401, 0.156434, -0.698401, -0.698401, 0.156434, -0.698401, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377973, 0.156435, -0.912505, -0.377973, 0.156435, -0.912505, -0.104474, 0.325528, -0.939743, -0.105800, 0.318418, -0.942028, -0.104474, 0.325528, -0.939743, -0.104474, 0.325528, -0.939743, -0.097141, 0.233062, -0.967598, -0.097141, 0.233062, -0.967598, 0.158828, 0.580683, -0.798486, 0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.090247, 0.600810, -0.794282, -0.091515, 0.594767, -0.798672, -0.090247, 0.600810, -0.794282, -0.090247, 0.600810, -0.794282, 0.116055, 0.803815, -0.583449, 0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.067135, 0.818259, -0.570916, -0.068222, 0.813901, -0.576985, -0.067135, 0.818259, -0.570916, -0.067135, 0.818259, -0.570916, 0.061299, 0.949355, -0.308167, 0.061299, 0.949355, -0.308167, 0, 1, 0, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137550, 0.933532, -0.331056, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253769, 0.933532, -0.253218, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331354, 0.933532, -0.136830, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, 0.000389, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331056, 0.933532, 0.137550, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253218, 0.933532, 0.253769, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.136830, 0.933532, 0.331354, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, -0.000389, 0.933532, 0.358494, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137550, 0.933532, 0.331056, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253768, 0.933532, 0.253219, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331354, 0.933532, 0.136830, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, -0.000391, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331056, 0.933532, -0.137550, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253218, 0.933532, -0.253769, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.136830, 0.933532, -0.331354, -0.042033, 0.942951, -0.330266, -0.042048, 0.942867, -0.330504, -0.042033, 0.942951, -0.330266, 0, 1, 0, 0.031102, 0.987211, -0.156357, 0.088569, 0.987211, -0.132553, 0.132553, 0.987211, -0.088569, 0.156357, 0.987211, -0.031102, 0.156357, 0.987211, 0.031102, 0.132553, 0.987211, 0.088569, 0.088569, 0.987211, 0.132553, 0.031102, 0.987211, 0.156357, -0.031102, 0.987211, 0.156357, -0.088569, 0.987211, 0.132553, -0.132553, 0.987211, 0.088570, -0.156357, 0.987211, 0.031101, -0.156357, 0.987211, -0.031102, -0.132553, 0.987211, -0.088569, -0.088569, 0.987211, -0.132553, -0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal7.vector = m_vector;
    Normal7.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate7.m_parent = (X3dNode *)&IndexedFaceSet9;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate7.point = m_point;
    TextureCoordinate7.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet9.color = NULL;
    IndexedFaceSet9.coord = &Coordinate17;
    IndexedFaceSet9.normal = &Normal7;
    IndexedFaceSet9.texCoord = &TextureCoordinate7;
    IndexedFaceSet9.ccw = true;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet9.colorIndex = m_colorIndex;
    IndexedFaceSet9.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet9.colorPerVertex = true;
    IndexedFaceSet9.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet9.coordIndex = m_coordIndex;
    IndexedFaceSet9.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet9.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet9.normalIndex = m_normalIndex;
    IndexedFaceSet9.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet9.normalPerVertex = true;
    IndexedFaceSet9.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet9.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet9.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet9.set_colorIndex = NULL;
    IndexedFaceSet9.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet9.set_coordIndex = NULL;
    IndexedFaceSet9.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet9.set_normalIndex = NULL;
    IndexedFaceSet9.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet9.set_texCoordIndex = NULL;
    IndexedFaceSet9.set_texCoordIndex_length = 0;
    }
    Shape9.appearance = &Appearance9;
    Shape9.geometry = &IndexedFaceSet9;
    Transform12.route_sources =  NULL;
    Transform12.num_route_source = 0;
    Transform12.m_parent = (X3dNode *)NULL;
    Shape10.m_parent = (X3dNode *)NULL;
    Appearance10.m_parent = (X3dNode *)NULL;
    Material8.m_parent = (X3dNode *)NULL;
    Material8.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material8.diffuseColor = m_diffuseColor;
    Material8.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material8.emissiveColor = m_emissiveColor;
    Material8.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material8.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material8.specularColor = m_specularColor;
    Material8.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material8.transparency = 0;
    Appearance10.material = &Material8;
    Appearance10.texture = NULL;
    Appearance10.textureTransform = NULL;
    IndexedFaceSet10.setGlName(9);
    IndexedFaceSet10.m_parent = (X3dNode *)NULL;
    Coordinate19.m_parent = (X3dNode *)&IndexedFaceSet10;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate19.point = m_point;
    Coordinate19.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal8.m_parent = (X3dNode *)&IndexedFaceSet10;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal8.vector = m_vector;
    Normal8.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate9.m_parent = (X3dNode *)&IndexedFaceSet10;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate9.point = m_point;
    TextureCoordinate9.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet10.color = NULL;
    IndexedFaceSet10.coord = &Coordinate19;
    IndexedFaceSet10.normal = &Normal8;
    IndexedFaceSet10.texCoord = &TextureCoordinate9;
    IndexedFaceSet10.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet10.colorIndex = m_colorIndex;
    IndexedFaceSet10.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet10.colorPerVertex = true;
    IndexedFaceSet10.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet10.coordIndex = m_coordIndex;
    IndexedFaceSet10.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet10.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet10.normalIndex = m_normalIndex;
    IndexedFaceSet10.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet10.normalPerVertex = true;
    IndexedFaceSet10.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet10.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet10.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet10.set_colorIndex = NULL;
    IndexedFaceSet10.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet10.set_coordIndex = NULL;
    IndexedFaceSet10.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet10.set_normalIndex = NULL;
    IndexedFaceSet10.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet10.set_texCoordIndex = NULL;
    IndexedFaceSet10.set_texCoordIndex_length = 0;
    }
    Shape10.appearance = &Appearance10;
    Shape10.geometry = &IndexedFaceSet10;
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, 0 };
    Transform12.rotation = m_rotation;
    Transform12.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.220000, 0.440000, 0.160000 };
    Transform12.translation = m_translation;
    Transform12.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform12.bboxCenter = m_bboxCenter;
    Transform12.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform12.bboxSize = m_bboxSize;
    Transform12.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform12.center = m_center;
    Transform12.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform12.scale = m_scale;
    Transform12.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform12.scaleOrientation = m_scaleOrientation;
    Transform12.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape10 };
    Transform12.children = m_children;
    Transform12.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform12.addChildren = NULL;
    Transform12.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform12.removeChildren = NULL;
    Transform12.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, -1.578764 };
    Transform11.rotation = m_rotation;
    Transform11.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform11.translation = m_translation;
    Transform11.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform11.bboxCenter = m_bboxCenter;
    Transform11.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform11.bboxSize = m_bboxSize;
    Transform11.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform11.center = m_center;
    Transform11.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform11.scale = m_scale;
    Transform11.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform11.scaleOrientation = m_scaleOrientation;
    Transform11.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape9, &Transform12 };
    Transform11.children = m_children;
    Transform11.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform11.addChildren = NULL;
    Transform11.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform11.removeChildren = NULL;
    Transform11.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.565552, -0.792356, -0.228743, -0.826199 };
    Transform10.rotation = m_rotation;
    Transform10.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.109765, 0.715337, 1.294240 };
    Transform10.translation = m_translation;
    Transform10.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform10.bboxCenter = m_bboxCenter;
    Transform10.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform10.bboxSize = m_bboxSize;
    Transform10.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform10.center = m_center;
    Transform10.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform10.scale = m_scale;
    Transform10.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform10.scaleOrientation = m_scaleOrientation;
    Transform10.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform11 };
    Transform10.children = m_children;
    Transform10.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform10.addChildren = NULL;
    Transform10.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform10.removeChildren = NULL;
    Transform10.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform6.rotation = m_rotation;
    Transform6.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0.120000 };
    Transform6.translation = m_translation;
    Transform6.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform6.bboxCenter = m_bboxCenter;
    Transform6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform6.bboxSize = m_bboxSize;
    Transform6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform6.center = m_center;
    Transform6.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform6.scale = m_scale;
    Transform6.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform6.scaleOrientation = m_scaleOrientation;
    Transform6.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform7, &Transform10 };
    Transform6.children = m_children;
    Transform6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform6.addChildren = NULL;
    Transform6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform6.removeChildren = NULL;
    Transform6.removeChildren_length = 0;
    }
    Inline5.m_parent = (X3dNode *)NULL;
    Inline3.m_parent = (X3dNode *)NULL;
    Group4.m_parent = (X3dNode *)&Inline3;
    Transform13.route_sources =  NULL;
    Transform13.num_route_source = 0;
    Transform13.m_parent = (X3dNode *)NULL;
    CORINTm_5.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_5.key = m_key;
    CORINTm_5.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.619297, 0.959445, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.779760, 0.966057, -0.506980, 1.645980, 0.964696, -0.929632, 2.257560, 1.170100, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890 };
    CORINTm_5.keyValue = m_keyValue;
    CORINTm_5.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_5.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_5.value_changed = NULL;
    CORINTm_5.value_changed_length = 0;
    }
    TIMERm_4.m_parent = (X3dNode *)NULL;
    TIMERm_4.cycleInterval = 2;
    TIMERm_4.enabled = true;
    TIMERm_4.loop = true;
    TIMERm_4.startTime = 0;
    TIMERm_4.stopTime = 0;
    TIMERm_4.cycleTime = 0;
    TIMERm_4.fraction_changed = 0;
    TIMERm_4.isActive = false;
    TIMERm_4.time = 0;
    Group5.m_parent = (X3dNode *)NULL;
    Background2.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background2.groundAngle = NULL;
    Background2.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background2.groundColor = m_groundColor;
    Background2.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background2.backUrl = NULL;
    Background2.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background2.rightUrl = NULL;
    Background2.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background2.frontUrl = NULL;
    Background2.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background2.leftUrl = NULL;
    Background2.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background2.topUrl = NULL;
    Background2.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background2.bottomUrl = NULL;
    Background2.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background2.skyAngle = NULL;
    Background2.skyAngle_length = 0;
    }
    Background2.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background2.skyColor = m_skyColor;
    Background2.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background2.set_bind = false;
    Background2.isBound = false;
    Shape11.m_parent = (X3dNode *)NULL;
    Appearance11.m_parent = (X3dNode *)NULL;
    Material9.m_parent = (X3dNode *)NULL;
    Material9.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material9.diffuseColor = m_diffuseColor;
    Material9.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material9.emissiveColor = m_emissiveColor;
    Material9.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material9.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material9.specularColor = m_specularColor;
    Material9.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material9.transparency = 6.382980e-2;
    Appearance11.material = &Material9;
    Appearance11.texture = NULL;
    Appearance11.textureTransform = NULL;
    IndexedFaceSet11.setGlName(10);
    IndexedFaceSet11.m_parent = (X3dNode *)NULL;
    COORD1m_5.m_parent = (X3dNode *)&IndexedFaceSet11;
    {
    static float m_point[] = { 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.780723, -0.294626, 1.000070, 0.734580, -0.981481, 1.700790, 0.307968, -0.506980, 1.580170, 0.280283, -1.021760, 2.071980, -0.277696, -0.976890 };
    COORD1m_5.point = m_point;
    COORD1m_5.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal29.m_parent = (X3dNode *)&IndexedFaceSet11;
    {
    static float m_vector[] = { 0.358806, 0.927849, -0.101757, 0.358806, 0.927849, -0.101757, 0.359718, 0.927470, -0.101994, 0.281824, 0.955978, -0.081744, 0.281824, 0.955978, -0.081744, 0.240191, 0.969864, -0.040901, 0.495155, 0.855218, -0.153047, 0.496190, 0.854566, -0.153340, 0.496190, 0.854566, -0.153340, 0.631845, 0.753624, -0.181169, 0.631845, 0.753624, -0.181169, 0.632281, 0.753233, -0.181277, 0.673713, 0.712478, -0.196177, 0.673713, 0.712478, -0.196177, 0.742001, 0.637272, -0.208134 };
    Normal29.vector = m_vector;
    Normal29.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet11.color = NULL;
    IndexedFaceSet11.coord = &COORD1m_5;
    IndexedFaceSet11.normal = &Normal29;
    IndexedFaceSet11.texCoord = NULL;
    IndexedFaceSet11.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet11.colorIndex = m_colorIndex;
    IndexedFaceSet11.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet11.colorPerVertex = true;
    IndexedFaceSet11.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet11.coordIndex = m_coordIndex;
    IndexedFaceSet11.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet11.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet11.normalIndex = m_normalIndex;
    IndexedFaceSet11.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet11.normalPerVertex = true;
    IndexedFaceSet11.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet11.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet11.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet11.set_colorIndex = NULL;
    IndexedFaceSet11.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet11.set_coordIndex = NULL;
    IndexedFaceSet11.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet11.set_normalIndex = NULL;
    IndexedFaceSet11.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet11.set_texCoordIndex = NULL;
    IndexedFaceSet11.set_texCoordIndex_length = 0;
    }
    Shape11.appearance = &Appearance11;
    Shape11.geometry = &IndexedFaceSet11;
    {
    static X3dNode* m_children[] = { &Background2, &Shape11 };
    Group5.children = m_children;
    Group5.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group5.bboxCenter = m_bboxCenter;
    Group5.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group5.bboxSize = m_bboxSize;
    Group5.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group5.addChildren = NULL;
    Group5.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group5.removeChildren = NULL;
    Group5.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform13.rotation = m_rotation;
    Transform13.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform13.translation = m_translation;
    Transform13.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform13.bboxCenter = m_bboxCenter;
    Transform13.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform13.bboxSize = m_bboxSize;
    Transform13.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform13.center = m_center;
    Transform13.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform13.scale = m_scale;
    Transform13.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform13.scaleOrientation = m_scaleOrientation;
    Transform13.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_5, &TIMERm_4, &Group5 };
    Transform13.children = m_children;
    Transform13.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform13.addChildren = NULL;
    Transform13.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform13.removeChildren = NULL;
    Transform13.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform13 };
    Group4.children = m_children;
    Group4.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group4.bboxCenter = m_bboxCenter;
    Group4.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group4.bboxSize = m_bboxSize;
    Group4.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group4.addChildren = NULL;
    Group4.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group4.removeChildren = NULL;
    Group4.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Group4 };
    Inline3.children = m_children;
    Inline3.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline3.bboxCenter = m_bboxCenter;
    Inline3.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline3.bboxSize = m_bboxSize;
    Inline3.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline3.addChildren = NULL;
    Inline3.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline3.removeChildren = NULL;
    Inline3.removeChildren_length = 0;
    }
    Inline4.m_parent = (X3dNode *)NULL;
    Group6.m_parent = (X3dNode *)&Inline4;
    Transform14.route_sources =  NULL;
    Transform14.num_route_source = 0;
    Transform14.m_parent = (X3dNode *)NULL;
    CORINTm_6.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_6.key = m_key;
    CORINTm_6.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.619297, 0.959445, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.779760, 0.966057, -0.506980, -1.645980, 0.964696, -0.929632, -2.257560, 1.170100, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890 };
    CORINTm_6.keyValue = m_keyValue;
    CORINTm_6.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_6.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_6.value_changed = NULL;
    CORINTm_6.value_changed_length = 0;
    }
    TIMERm_5.m_parent = (X3dNode *)NULL;
    TIMERm_5.cycleInterval = 2;
    TIMERm_5.enabled = true;
    TIMERm_5.loop = true;
    TIMERm_5.startTime = 0;
    TIMERm_5.stopTime = 0;
    TIMERm_5.cycleTime = 0;
    TIMERm_5.fraction_changed = 0;
    TIMERm_5.isActive = false;
    TIMERm_5.time = 0;
    Group7.m_parent = (X3dNode *)NULL;
    Background3.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background3.groundAngle = NULL;
    Background3.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background3.groundColor = m_groundColor;
    Background3.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background3.backUrl = NULL;
    Background3.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background3.rightUrl = NULL;
    Background3.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background3.frontUrl = NULL;
    Background3.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background3.leftUrl = NULL;
    Background3.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background3.topUrl = NULL;
    Background3.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background3.bottomUrl = NULL;
    Background3.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background3.skyAngle = NULL;
    Background3.skyAngle_length = 0;
    }
    Background3.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background3.skyColor = m_skyColor;
    Background3.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background3.set_bind = false;
    Background3.isBound = false;
    Shape12.m_parent = (X3dNode *)NULL;
    Appearance12.m_parent = (X3dNode *)NULL;
    Material10.m_parent = (X3dNode *)NULL;
    Material10.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material10.diffuseColor = m_diffuseColor;
    Material10.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material10.emissiveColor = m_emissiveColor;
    Material10.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material10.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material10.specularColor = m_specularColor;
    Material10.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material10.transparency = 6.382980e-2;
    Appearance12.material = &Material10;
    Appearance12.texture = NULL;
    Appearance12.textureTransform = NULL;
    IndexedFaceSet12.setGlName(11);
    IndexedFaceSet12.m_parent = (X3dNode *)NULL;
    COORD1m_7.m_parent = (X3dNode *)&IndexedFaceSet12;
    {
    static float m_point[] = { -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.780723, -0.294626, -1.000070, 0.734580, -0.981481, -1.700790, 0.307968, -0.506980, -1.580170, 0.280283, -1.021760, -2.071980, -0.277696, -0.976890 };
    COORD1m_7.point = m_point;
    COORD1m_7.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal30.m_parent = (X3dNode *)&IndexedFaceSet12;
    {
    static float m_vector[] = { 0.358806, -0.927849, 0.101757, 0.358806, -0.927849, 0.101757, 0.359718, -0.927470, 0.101994, 0.281824, -0.955978, 0.081744, 0.281824, -0.955978, 0.081744, 0.240191, -0.969864, 0.040901, 0.495155, -0.855218, 0.153047, 0.496190, -0.854566, 0.153340, 0.496190, -0.854566, 0.153340, 0.631845, -0.753624, 0.181169, 0.631845, -0.753624, 0.181169, 0.632281, -0.753233, 0.181277, 0.673713, -0.712478, 0.196177, 0.673713, -0.712478, 0.196177, 0.742001, -0.637272, 0.208134 };
    Normal30.vector = m_vector;
    Normal30.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet12.color = NULL;
    IndexedFaceSet12.coord = &COORD1m_7;
    IndexedFaceSet12.normal = &Normal30;
    IndexedFaceSet12.texCoord = NULL;
    IndexedFaceSet12.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet12.colorIndex = m_colorIndex;
    IndexedFaceSet12.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet12.colorPerVertex = true;
    IndexedFaceSet12.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet12.coordIndex = m_coordIndex;
    IndexedFaceSet12.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet12.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet12.normalIndex = m_normalIndex;
    IndexedFaceSet12.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet12.normalPerVertex = true;
    IndexedFaceSet12.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet12.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet12.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet12.set_colorIndex = NULL;
    IndexedFaceSet12.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet12.set_coordIndex = NULL;
    IndexedFaceSet12.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet12.set_normalIndex = NULL;
    IndexedFaceSet12.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet12.set_texCoordIndex = NULL;
    IndexedFaceSet12.set_texCoordIndex_length = 0;
    }
    Shape12.appearance = &Appearance12;
    Shape12.geometry = &IndexedFaceSet12;
    {
    static X3dNode* m_children[] = { &Background3, &Shape12 };
    Group7.children = m_children;
    Group7.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group7.bboxCenter = m_bboxCenter;
    Group7.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group7.bboxSize = m_bboxSize;
    Group7.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group7.addChildren = NULL;
    Group7.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group7.removeChildren = NULL;
    Group7.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform14.rotation = m_rotation;
    Transform14.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform14.translation = m_translation;
    Transform14.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform14.bboxCenter = m_bboxCenter;
    Transform14.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform14.bboxSize = m_bboxSize;
    Transform14.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform14.center = m_center;
    Transform14.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform14.scale = m_scale;
    Transform14.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform14.scaleOrientation = m_scaleOrientation;
    Transform14.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_6, &TIMERm_5, &Group7 };
    Transform14.children = m_children;
    Transform14.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform14.addChildren = NULL;
    Transform14.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform14.removeChildren = NULL;
    Transform14.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform14 };
    Group6.children = m_children;
    Group6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group6.bboxCenter = m_bboxCenter;
    Group6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group6.bboxSize = m_bboxSize;
    Group6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group6.addChildren = NULL;
    Group6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group6.removeChildren = NULL;
    Group6.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Group6 };
    Inline4.children = m_children;
    Inline4.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline4.bboxCenter = m_bboxCenter;
    Inline4.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline4.bboxSize = m_bboxSize;
    Inline4.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline4.addChildren = NULL;
    Inline4.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline4.removeChildren = NULL;
    Inline4.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Inline3, &Inline4 };
    Inline5.children = m_children;
    Inline5.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline5.bboxCenter = m_bboxCenter;
    Inline5.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline5.bboxSize = m_bboxSize;
    Inline5.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline5.addChildren = NULL;
    Inline5.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline5.removeChildren = NULL;
    Inline5.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform1.rotation = m_rotation;
    Transform1.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform1.translation = m_translation;
    Transform1.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform1.bboxCenter = m_bboxCenter;
    Transform1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform1.bboxSize = m_bboxSize;
    Transform1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform1.center = m_center;
    Transform1.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform1.scale = m_scale;
    Transform1.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform1.scaleOrientation = m_scaleOrientation;
    Transform1.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline2, &Transform6, &Inline5 };
    Transform1.children = m_children;
    Transform1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform1.addChildren = NULL;
    Transform1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform1.removeChildren = NULL;
    Transform1.removeChildren_length = 0;
    }
    WorldInfo2.m_parent = (X3dNode *)&Inline6;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "p_fly is a modification of the program tux_aqfh by Steve Baker", "tux_aqfh is GPL" };
    WorldInfo2.info = m_info;
    WorldInfo2.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo2.title = "Tux warbird 3D Game";
    {
    static X3dNode* m_children[] = { &Transform1, &WorldInfo2 };
    Inline6.children = m_children;
    Inline6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline6.bboxCenter = m_bboxCenter;
    Inline6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline6.bboxSize = m_bboxSize;
    Inline6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline6.addChildren = NULL;
    Inline6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline6.removeChildren = NULL;
    Inline6.removeChildren_length = 0;
    }
    PLANEm_SWITCH.m_parent = (X3dNode *)&HIDEm_THING;
    Inline7.m_parent = (X3dNode *)&PLANEm_SWITCH;
    Transform15.route_sources =  NULL;
    Transform15.num_route_source = 0;
    Transform15.m_parent = (X3dNode *)&Inline7;
    Shape13.m_parent = (X3dNode *)NULL;
    Appearance13.m_parent = (X3dNode *)NULL;
    Material11.m_parent = (X3dNode *)NULL;
    Material11.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material11.diffuseColor = m_diffuseColor;
    Material11.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material11.emissiveColor = m_emissiveColor;
    Material11.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material11.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material11.specularColor = m_specularColor;
    Material11.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material11.transparency = 0;
    Appearance13.material = &Material11;
    Appearance13.texture = NULL;
    Appearance13.textureTransform = NULL;
    IndexedFaceSet13.setGlName(12);
    IndexedFaceSet13.m_parent = (X3dNode *)NULL;
    Color7.m_parent = (X3dNode *)&IndexedFaceSet13;
    {
    static float m_color[] = { 1, 0, 0 };
    Color7.color = m_color;
    Color7.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate21.m_parent = (X3dNode *)&IndexedFaceSet13;
    {
    static float m_point[] = { 0, 2.145490, -2.315630, 0, 0.876689, -1.765810, 0, 1.553380, -0.553399, 0, 2.201880, -1.427460 };
    Coordinate21.point = m_point;
    Coordinate21.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal31.m_parent = (X3dNode *)&IndexedFaceSet13;
    {
    static float m_vector[] = { -1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0 };
    Normal31.vector = m_vector;
    Normal31.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet13.color = &Color7;
    IndexedFaceSet13.coord = &Coordinate21;
    IndexedFaceSet13.normal = &Normal31;
    IndexedFaceSet13.texCoord = NULL;
    IndexedFaceSet13.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet13.colorIndex = m_colorIndex;
    IndexedFaceSet13.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet13.colorPerVertex = false;
    IndexedFaceSet13.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet13.coordIndex = m_coordIndex;
    IndexedFaceSet13.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet13.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet13.normalIndex = m_normalIndex;
    IndexedFaceSet13.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet13.normalPerVertex = true;
    IndexedFaceSet13.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet13.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet13.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet13.set_colorIndex = NULL;
    IndexedFaceSet13.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet13.set_coordIndex = NULL;
    IndexedFaceSet13.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet13.set_normalIndex = NULL;
    IndexedFaceSet13.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet13.set_texCoordIndex = NULL;
    IndexedFaceSet13.set_texCoordIndex_length = 0;
    }
    Shape13.appearance = &Appearance13;
    Shape13.geometry = &IndexedFaceSet13;
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform15.rotation = m_rotation;
    Transform15.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform15.translation = m_translation;
    Transform15.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform15.bboxCenter = m_bboxCenter;
    Transform15.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform15.bboxSize = m_bboxSize;
    Transform15.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform15.center = m_center;
    Transform15.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform15.scale = m_scale;
    Transform15.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform15.scaleOrientation = m_scaleOrientation;
    Transform15.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape13 };
    Transform15.children = m_children;
    Transform15.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform15.addChildren = NULL;
    Transform15.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform15.removeChildren = NULL;
    Transform15.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Transform15 };
    Inline7.children = m_children;
    Inline7.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline7.bboxCenter = m_bboxCenter;
    Inline7.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline7.bboxSize = m_bboxSize;
    Inline7.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline7.addChildren = NULL;
    Inline7.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline7.removeChildren = NULL;
    Inline7.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Inline7 };
    PLANEm_SWITCH.children = m_children;
    PLANEm_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    PLANEm_SWITCH.whichChoice = -1;
    Transform16.route_sources =  NULL;
    Transform16.num_route_source = 0;
    Transform16.m_parent = (X3dNode *)&HIDEm_THING;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform16.rotation = m_rotation;
    Transform16.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -1.150000 };
    Transform16.translation = m_translation;
    Transform16.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform16.bboxCenter = m_bboxCenter;
    Transform16.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform16.bboxSize = m_bboxSize;
    Transform16.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform16.center = m_center;
    Transform16.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform16.scale = m_scale;
    Transform16.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform16.scaleOrientation = m_scaleOrientation;
    Transform16.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[1];
    Transform16.children = NULL;
    Transform16.children_length = 0;
    }
    {
    static X3dNode* m_addChildren[1];
    Transform16.addChildren = NULL;
    Transform16.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform16.removeChildren = NULL;
    Transform16.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    HIDEm_THING.rotation = m_rotation;
    HIDEm_THING.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    HIDEm_THING.translation = m_translation;
    HIDEm_THING.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    HIDEm_THING.bboxCenter = m_bboxCenter;
    HIDEm_THING.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    HIDEm_THING.bboxSize = m_bboxSize;
    HIDEm_THING.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    HIDEm_THING.center = m_center;
    HIDEm_THING.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    HIDEm_THING.scale = m_scale;
    HIDEm_THING.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    HIDEm_THING.scaleOrientation = m_scaleOrientation;
    HIDEm_THING.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline6, &PLANEm_SWITCH, &Transform16 };
    HIDEm_THING.children = m_children;
    HIDEm_THING.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    HIDEm_THING.addChildren = NULL;
    HIDEm_THING.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    HIDEm_THING.removeChildren = NULL;
    HIDEm_THING.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    UPm_DOWNm_AXIS.rotation = m_rotation;
    UPm_DOWNm_AXIS.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.translation = m_translation;
    UPm_DOWNm_AXIS.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.bboxCenter = m_bboxCenter;
    UPm_DOWNm_AXIS.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    UPm_DOWNm_AXIS.bboxSize = m_bboxSize;
    UPm_DOWNm_AXIS.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.center = m_center;
    UPm_DOWNm_AXIS.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    UPm_DOWNm_AXIS.scale = m_scale;
    UPm_DOWNm_AXIS.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    UPm_DOWNm_AXIS.scaleOrientation = m_scaleOrientation;
    UPm_DOWNm_AXIS.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &THINGm_SENSOR, &HIDEm_THING };
    UPm_DOWNm_AXIS.children = m_children;
    UPm_DOWNm_AXIS.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    UPm_DOWNm_AXIS.addChildren = NULL;
    UPm_DOWNm_AXIS.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    UPm_DOWNm_AXIS.removeChildren = NULL;
    UPm_DOWNm_AXIS.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    ROLLm_AXIS.rotation = m_rotation;
    ROLLm_AXIS.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    ROLLm_AXIS.translation = m_translation;
    ROLLm_AXIS.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    ROLLm_AXIS.bboxCenter = m_bboxCenter;
    ROLLm_AXIS.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    ROLLm_AXIS.bboxSize = m_bboxSize;
    ROLLm_AXIS.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    ROLLm_AXIS.center = m_center;
    ROLLm_AXIS.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    ROLLm_AXIS.scale = m_scale;
    ROLLm_AXIS.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    ROLLm_AXIS.scaleOrientation = m_scaleOrientation;
    ROLLm_AXIS.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &UPm_DOWNm_AXIS };
    ROLLm_AXIS.children = m_children;
    ROLLm_AXIS.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    ROLLm_AXIS.addChildren = NULL;
    ROLLm_AXIS.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    ROLLm_AXIS.removeChildren = NULL;
    ROLLm_AXIS.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    THING.rotation = m_rotation;
    THING.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    THING.translation = m_translation;
    THING.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    THING.bboxCenter = m_bboxCenter;
    THING.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    THING.bboxSize = m_bboxSize;
    THING.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    THING.center = m_center;
    THING.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    THING.scale = m_scale;
    THING.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    THING.scaleOrientation = m_scaleOrientation;
    THING.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &ROLLm_AXIS };
    THING.children = m_children;
    THING.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    THING.addChildren = NULL;
    THING.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    THING.removeChildren = NULL;
    THING.removeChildren_length = 0;
    }
    BULLETm_SWITCH.m_parent = (X3dNode *)&Group1;
    BULLETm_MOVE.route_sources =  NULL;
    BULLETm_MOVE.num_route_source = 0;
    BULLETm_MOVE.m_parent = (X3dNode *)&BULLETm_SWITCH;
    DUMMYm_BULLET.m_parent = (X3dNode *)&BULLETm_MOVE;
    DUMMYm_BULLET.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    DUMMYm_BULLET.hitNormal_changed = m_hitNormal_changed;
    DUMMYm_BULLET.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    DUMMYm_BULLET.hitPoint_changed = m_hitPoint_changed;
    DUMMYm_BULLET.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    DUMMYm_BULLET.hitTexCoord_changed = m_hitTexCoord_changed;
    DUMMYm_BULLET.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    DUMMYm_BULLET.isActive = false;
    DUMMYm_BULLET.isOver = false;
    DUMMYm_BULLET.touchTime = 0;
    Inline9.m_parent = (X3dNode *)&BULLETm_MOVE;
    Transform17.route_sources =  NULL;
    Transform17.num_route_source = 0;
    Transform17.m_parent = (X3dNode *)&Inline9;
    Inline8.m_parent = (X3dNode *)NULL;
    Shape14.m_parent = (X3dNode *)&Inline8;
    Appearance14.m_parent = (X3dNode *)NULL;
    Material12.m_parent = (X3dNode *)NULL;
    Material12.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material12.diffuseColor = m_diffuseColor;
    Material12.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material12.emissiveColor = m_emissiveColor;
    Material12.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material12.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material12.specularColor = m_specularColor;
    Material12.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material12.transparency = 0;
    ImageTexture2.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "cake.png" };
    ImageTexture2.url = m_url;
    ImageTexture2.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture2.repeatS = true;
    ImageTexture2.repeatT = true;
    Appearance14.material = &Material12;
    Appearance14.texture = &ImageTexture2;
    Appearance14.textureTransform = NULL;
    IndexedFaceSet14.setGlName(13);
    IndexedFaceSet14.m_parent = (X3dNode *)NULL;
    Coordinate23.m_parent = (X3dNode *)&IndexedFaceSet14;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate23.point = m_point;
    Coordinate23.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal9.m_parent = (X3dNode *)&IndexedFaceSet14;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal9.vector = m_vector;
    Normal9.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate11.m_parent = (X3dNode *)&IndexedFaceSet14;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate11.point = m_point;
    TextureCoordinate11.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet14.color = NULL;
    IndexedFaceSet14.coord = &Coordinate23;
    IndexedFaceSet14.normal = &Normal9;
    IndexedFaceSet14.texCoord = &TextureCoordinate11;
    IndexedFaceSet14.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet14.colorIndex = m_colorIndex;
    IndexedFaceSet14.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet14.colorPerVertex = false;
    IndexedFaceSet14.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet14.coordIndex = m_coordIndex;
    IndexedFaceSet14.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet14.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet14.normalIndex = m_normalIndex;
    IndexedFaceSet14.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet14.normalPerVertex = true;
    IndexedFaceSet14.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet14.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet14.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet14.set_colorIndex = NULL;
    IndexedFaceSet14.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet14.set_coordIndex = NULL;
    IndexedFaceSet14.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet14.set_normalIndex = NULL;
    IndexedFaceSet14.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet14.set_texCoordIndex = NULL;
    IndexedFaceSet14.set_texCoordIndex_length = 0;
    }
    Shape14.appearance = &Appearance14;
    Shape14.geometry = &IndexedFaceSet14;
    WorldInfo3.m_parent = (X3dNode *)&Inline8;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo3.info = m_info;
    WorldInfo3.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo3.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Shape14, &WorldInfo3 };
    Inline8.children = m_children;
    Inline8.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline8.bboxCenter = m_bboxCenter;
    Inline8.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline8.bboxSize = m_bboxSize;
    Inline8.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline8.addChildren = NULL;
    Inline8.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline8.removeChildren = NULL;
    Inline8.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform17.rotation = m_rotation;
    Transform17.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform17.translation = m_translation;
    Transform17.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform17.bboxCenter = m_bboxCenter;
    Transform17.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform17.bboxSize = m_bboxSize;
    Transform17.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform17.center = m_center;
    Transform17.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform17.scale = m_scale;
    Transform17.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform17.scaleOrientation = m_scaleOrientation;
    Transform17.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { NULL, &Inline8 };
    Transform17.children = m_children;
    Transform17.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform17.addChildren = NULL;
    Transform17.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform17.removeChildren = NULL;
    Transform17.removeChildren_length = 0;
    }
    WorldInfo4.m_parent = (X3dNode *)&Inline9;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo4.info = m_info;
    WorldInfo4.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo4.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { NULL, &Transform17, NULL, &WorldInfo4 };
    Inline9.children = m_children;
    Inline9.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline9.bboxCenter = m_bboxCenter;
    Inline9.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline9.bboxSize = m_bboxSize;
    Inline9.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline9.addChildren = NULL;
    Inline9.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline9.removeChildren = NULL;
    Inline9.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BULLETm_MOVE.rotation = m_rotation;
    BULLETm_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BULLETm_MOVE.translation = m_translation;
    BULLETm_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BULLETm_MOVE.bboxCenter = m_bboxCenter;
    BULLETm_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BULLETm_MOVE.bboxSize = m_bboxSize;
    BULLETm_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BULLETm_MOVE.center = m_center;
    BULLETm_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BULLETm_MOVE.scale = m_scale;
    BULLETm_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BULLETm_MOVE.scaleOrientation = m_scaleOrientation;
    BULLETm_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &DUMMYm_BULLET, &Inline9 };
    BULLETm_MOVE.children = m_children;
    BULLETm_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BULLETm_MOVE.addChildren = NULL;
    BULLETm_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BULLETm_MOVE.removeChildren = NULL;
    BULLETm_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BULLETm_MOVE };
    BULLETm_SWITCH.children = m_children;
    BULLETm_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BULLETm_SWITCH.whichChoice = -1;
    TIMER.m_parent = (X3dNode *)&Group1;
    TIMER.cycleInterval = 1;
    TIMER.enabled = true;
    TIMER.loop = true;
    TIMER.startTime = 0;
    TIMER.stopTime = 0;
    TIMER.cycleTime = 0;
    TIMER.fraction_changed = 0;
    TIMER.isActive = false;
    TIMER.time = 0;
    SOUND.m_parent = (X3dNode *)&Group1;
    SOUNDCLIP.m_parent = (X3dNode *)&SOUND;
    SOUNDCLIP.description = "away";
    SOUNDCLIP.loop = false;
    SOUNDCLIP.pitch = 1;
    SOUNDCLIP.startTime = 0;
    SOUNDCLIP.stopTime = 0;
    {
    static const char* m_url[] = { "multimedia/bubble.wav" };
    SOUNDCLIP.url = m_url;
    SOUNDCLIP.url_length = sizeof(m_url) / sizeof(const char*);
    }
    SOUNDCLIP.duration_changed = 0;
    SOUNDCLIP.isActive = false;
    {
    static float m_direction[] = { 0, 0, 1 };
    SOUND.direction = m_direction;
    SOUND.direction_length = sizeof(m_direction) / sizeof(float);
    }
    SOUND.intensity = 1;
    {
    static float m_location[] = { 0, 0, 0 };
    SOUND.location = m_location;
    SOUND.location_length = sizeof(m_location) / sizeof(float);
    }
    SOUND.maxBack = 500;
    SOUND.maxFront = 500;
    SOUND.minBack = 500;
    SOUND.minFront = 500;
    SOUND.priority = 0;
    SOUND.source = &SOUNDCLIP;
    SOUND.spatialize = true;
    NavigationInfo1.m_parent = (X3dNode *)&Group1;
    {
    static float m_avatarSize[] = { 0.250000, 1.600000, 0.750000 };
    NavigationInfo1.avatarSize = m_avatarSize;
    NavigationInfo1.avatarSize_length = sizeof(m_avatarSize) / sizeof(float);
    }
    NavigationInfo1.headlight = true;
    NavigationInfo1.speed = 1;
    {
    static const char* m_type[] = { "NONE" };
    NavigationInfo1.type = m_type;
    NavigationInfo1.type_length = sizeof(m_type) / sizeof(const char*);
    }
    NavigationInfo1.visibilityLimit = 0;
    NavigationInfo1.set_bind = false;
    NavigationInfo1.isBound = false;
    ROLLm_TURN.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()              \
      {              \
      rotation_out[0]=1;              \
      rotation_out[1]=0;              \
      rotation_out[2]=0;              \
      rotation_out[3]=0;              \
      }              \
   function rock_and_roll_flag_in(val)  \
      {  \
      rock_and_roll_flag=val;  \
      }  \
   function xvalue_in(val)    \
      {  \
      rotation_out[0]=0;  \
      if (rock_and_roll_flag==1)  \
         {  \
         rotation_out[1]=0;  \
         rotation_out[2]=1;  \
         }  \
      else  \
         {  \
         rotation_out[1]=1;  \
         rotation_out[2]=0;  \
         }  \
      rotation_out[3]=-val/1.0;  \
      }  \
   ", "c++:  \
   X3dScript_ROLL_TURN *self = (X3dScript_ROLL_TURN *)node;  \
   static bool ROLL_TURN_init = false;  \
   if (!ROLL_TURN_init) {  \
       self->rotation_out[0]=1;  \
       self->rotation_out[1]=0;  \
       self->rotation_out[2]=0;  \
       self->rotation_out[3]=0;  \
       ROLL_TURN_init = true;  \
   }           \
   self->rock_and_roll_flag = self->rock_and_roll_flag_in;  \
   if (self->rock_and_roll_flag==1) {  \
       self->rotation_out[1]=0;  \
       self->rotation_out[2]=1;  \
   } else {  \
       self->rotation_out[1]=1;  \
       self->rotation_out[2]=0;  \
   }  \
   self->rotation_out[0]=0;  \
   self->rotation_out[3]=self->xvalue_in * 2.0;  \
" };
    ROLLm_TURN.url = m_url;
    ROLLm_TURN.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROLLm_TURN.directOutput = false;
    ROLLm_TURN.mustEvaluate = false;
    ROLLm_TURN.rock_and_roll_flag = 0;
    ROLLm_TURN.xvalue_in = 0;
    ROLLm_TURN.rock_and_roll_flag_in = 0;
    {
    static float m_rotation_out[] = { 0, 0, 1, 0 };
    ROLLm_TURN.rotation_out = m_rotation_out;
    ROLLm_TURN.rotation_out_length = sizeof(m_rotation_out) / sizeof(float);
    }
    UPm_DOWNm_TURN.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()              \
      {              \
      rotation_out[0]=0;              \
      rotation_out[1]=0;              \
      rotation_out[2]=0;              \
      rotation_out[3]=0;              \
      }              \
   function yvalue_in(val)    \
      {  \
      rotation_out[0]=1;  \
      rotation_out[1]=0;  \
      rotation_out[2]=0;  \
      rotation_out[3]=-val/1.0;  \
      }  \
   ", "c++:  \
   X3dScript_UP_DOWN_TURN *self = (X3dScript_UP_DOWN_TURN *)node;  \
   static bool UP_DOWN_TURN_init = false;  \
   if (!UP_DOWN_TURN_init) {  \
       self->rotation_out[0]=0;  \
       self->rotation_out[1]=0;  \
       self->rotation_out[2]=0;  \
       self->rotation_out[3]=0;  \
       UP_DOWN_TURN_init = true;  \
   }           \
   self->rotation_out[0]=1;  \
   self->rotation_out[1]=0;  \
   self->rotation_out[2]=0;  \
   self->rotation_out[3]=self->yvalue_in * 2.0;  \
" };
    UPm_DOWNm_TURN.url = m_url;
    UPm_DOWNm_TURN.url_length = sizeof(m_url) / sizeof(const char*);
    }
    UPm_DOWNm_TURN.directOutput = false;
    UPm_DOWNm_TURN.mustEvaluate = false;
    UPm_DOWNm_TURN.yvalue_in = 0;
    {
    static float m_rotation_out[] = { 0, 0, 1, 0 };
    UPm_DOWNm_TURN.rotation_out = m_rotation_out;
    UPm_DOWNm_TURN.rotation_out_length = sizeof(m_rotation_out) / sizeof(float);
    }
    FLYm_FORWARD.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function initialize()  \
      {  \
      translation_out[0]=0;  \
      translation_out[1]=0;  \
      translation_out[2]=0;  \
      }  \
   function time_in(val)    \
      {  \
      translation_out[0]=0;  \
      translation_out[1]=0;  \
      translation_out[2]=0.15;  \
      time_out=val;  \
      }  \
   ", "c++:  \
   X3dScript_FLY_FORWARD *self = (X3dScript_FLY_FORWARD *)node;  \
   static bool FLY_FORWARD_init = false;  \
   if (!FLY_FORWARD_init) {  \
       self->translation_out[0]=0;  \
       self->translation_out[1]=0;  \
       self->translation_out[2]=0;  \
       FLY_FORWARD_init = true;  \
   }           \
   self->translation_out[0]=0;  \
   self->translation_out[1]=0;  \
   self->translation_out[2]=0.15;  \
   self->time_out = self->time_in;  \
" };
    FLYm_FORWARD.url = m_url;
    FLYm_FORWARD.url_length = sizeof(m_url) / sizeof(const char*);
    }
    FLYm_FORWARD.directOutput = false;
    FLYm_FORWARD.mustEvaluate = false;
    FLYm_FORWARD.time_in = 0;
    {
    static float m_translation_out[] = { 0, 0, 0 };
    FLYm_FORWARD.translation_out = m_translation_out;
    FLYm_FORWARD.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    FLYm_FORWARD.time_out = 0;
    FLYm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
  \
 // field SFNode metadata //  \
  \
 // field SFVec3f top_translation //  \
  \
 // field SFVec3f translation_old //  \
  \
 // field SFBool brake //  \
  \
 // field SFVec3f bullet_translation_start //  \
  \
 // field SFVec3f bullet_direction //  \
  \
 // field SFFloat bulletway_length //  \
  \
 // field SFTime old_time //  \
  \
 // field SFFloat factor //  \
  \
 // eventOut SFVec3f translation_out //  \
  \
 // eventOut SFVec3f top_translation_out //  \
  \
 // eventOut SFInt32 bullet_choice_out //  \
  \
 // eventOut SFVec3f bullet_translation_out //  \
  \
   function initialize()  \
      {  \
      bullet_choice_out=-1;  \
      }  \
   // stop movement   \
   function exit_time(val)  \
      {  \
      brake=TRUE;  \
      }  \
   // restart movement  \
   function enter_time(val)  \
      {  \
      brake=FALSE;  \
      }  \
   function time_in(val)  \
      {  \
      factor=5.0/Browser.getCurrentFrameRate();  \
      old_time=val;  \
      }  \
   function fire_in(val)  \
      {  \
      if (val)  \
         {  \
         bullet_translation_start=top_translation_out;  \
         // switch on bullet  \
         bullet_choice_out=0;  \
         bulletway_length=0;  \
         }  \
      }  \
   function drive_bullet(val)  \
      {  \
      if (bulletway_length==0)  \
         {  \
         bullet_direction=val;  \
         }  \
      if ((bullet_translation_out[2]>10) ||  \
          (bullet_translation_out[2]<-35))  \
         {  \
         bulletway_length=0;  \
         bullet_choice_out=-1;  \
         bullet_translation_out[2]=val[2];  \
         }  \
      else if (bulletway_length<45)  \
         {  \
         bullet_translation_out[0]=bullet_translation_start[0]-  \
                                   bulletway_length*bullet_direction[0];  \
         bullet_translation_out[1]=bullet_translation_start[1]-  \
                                   bulletway_length*bullet_direction[1];  \
         bullet_translation_out[2]=bullet_translation_start[2]-  \
                                   bulletway_length*bullet_direction[2];  \
         bulletway_length+=10.0*factor;  \
         }  \
      else  \
         {  \
         bulletway_length=0;  \
         bullet_choice_out=-1;  \
         bullet_translation_out[2]=val[2];  \
         }  \
      }  \
   //  \
   function translation_in(val)    \
      {  \
      top_translation[0]=translation_old[0]-val[0]*factor;  \
      top_translation[1]=translation_old[1]-val[1]*factor;  \
      top_translation[2]=translation_old[2]-val[2]*factor;  \
      if (top_translation[2]<-35)  \
          top_translation[2] = 7;  \
      if (top_translation[2]>7)  \
          top_translation[2] = -35;  \
      if (top_translation[0]<-7)  \
          top_translation[0] = 7;  \
      if (top_translation[0]>7)  \
          top_translation[0] = -7;  \
      if (top_translation[1]<-7)  \
          top_translation[1] = 7;  \
      if (top_translation[1]>7)  \
          top_translation[1] = -7;  \
      translation_old = top_translation;  \
      translation_out[0]=translation_old[0]-val[0]*factor;  \
      translation_out[1]=translation_old[1]-val[1]*factor;  \
      translation_out[2]=translation_old[2]-val[2]*factor;  \
      translation_old=translation_out;  \
      if (bullet_choice_out==0)  \
         drive_bullet(val);  \
      top_translation_out=top_translation;  \
      }  \
   ", "c++:  \
      X3dScript_FLY_ON *self = (X3dScript_FLY_ON *)node;  \
      static bool FLY_ON_init = false;  \
      if (!FLY_ON_init) {  \
           self->bullet_choice_out = -1;   \
           FLY_ON_init = true;  \
      }           \
/*  \
   function exit_time(val)  \
      {  \
      brake=TRUE;  \
      }  \
   // restart movement  \
   function enter_time(val)  \
      {  \
      brake=FALSE;  \
      }  \
*/  \
      self->factor = 0.06;  \
      self->old_time = self->time_in;  \
      if (self->fire_in)  \
         {  \
         self->bullet_translation_start = self->top_translation_out;  \
         // switch on bullet  \
         self->bullet_choice_out = 0;  \
         self->bulletway_length = 0;  \
         }  \
      self->top_translation[0] = self->translation_old[0] -  \
                                 self->translation_in[0] * self->factor;  \
      self->top_translation[1] = self->translation_old[1] -  \
                                 self->translation_in[1] * self->factor;  \
      self->top_translation[2] = self->translation_old[2] -  \
                                 self->translation_in[2] * self->factor;  \
      if (self->top_translation[2] < -35)  \
          self->top_translation[2] = 7;  \
      if (self->top_translation[2] > 7)  \
          self->top_translation[2] = -35;  \
      if (self->top_translation[0] < -7)  \
          self->top_translation[0] = 7;  \
      if (self->top_translation[0] > 7)  \
          self->top_translation[0] = -7;  \
      if (self->top_translation[1] < -7)  \
          self->top_translation[1] = 7;  \
      if (self->top_translation[1] > 7)  \
          self->top_translation[1] = -7;  \
      self->translation_old = self->top_translation;  \
      self->translation_out[0] = self->translation_old[0] -  \
                                 self->translation_in[0] * self->factor;  \
      self->translation_out[1] = self->translation_old[1] -  \
                                 self->translation_in[1] * self->factor;  \
      self->translation_out[2] = self->translation_old[2] -   \
                                 self->translation_in[2] * self->factor;  \
      self->translation_old[0] = self->translation_out[0];  \
      self->translation_old[1] = self->translation_out[1];  \
      self->translation_old[2] = self->translation_out[2];  \
      if (self->bullet_choice_out == 0)   \
         {  \
         if (self->bulletway_length == 0)  \
            {  \
            self->bullet_direction = self->translation_in;  \
            }  \
         if ((self->bullet_translation_out[2] > 10) ||  \
             (self->bullet_translation_out[2] < -35))  \
            {  \
            self->bulletway_length = 0;  \
            self->bullet_choice_out = -1;  \
            self->bullet_translation_out[2] = self->translation_in[2];  \
            }  \
         else if (self->bulletway_length < 45)  \
            {  \
            self->bullet_translation_out[0] = self->bullet_translation_start[0] -  \
                                              self->bulletway_length *  \
                                              self->bullet_direction[0];  \
            self->bullet_translation_out[1] = self->bullet_translation_start[1] -  \
                                              self->bulletway_length *  \
                                              self->bullet_direction[1];  \
            self->bullet_translation_out[2] = self->bullet_translation_start[2] -  \
                                              self->bulletway_length *  \
                                              self->bullet_direction[2];  \
            self->bulletway_length += 10.0 * self->factor;  \
            }  \
         else  \
            {  \
            self->bulletway_length = 0;  \
            self->bullet_choice_out = -1;  \
            self->bullet_translation_out[2] = self->translation_in[2];  \
            }  \
         }  \
      self->top_translation_out = self->top_translation;     \
" };
    FLYm_ON.url = m_url;
    FLYm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    FLYm_ON.directOutput = false;
    FLYm_ON.mustEvaluate = false;
    {
    static float m_top_translation[] = { -1, 0, 0 };
    FLYm_ON.top_translation = m_top_translation;
    FLYm_ON.top_translation_length = sizeof(m_top_translation) / sizeof(float);
    }
    {
    static float m_translation_old[] = { 0, 0, 0 };
    FLYm_ON.translation_old = m_translation_old;
    FLYm_ON.translation_old_length = sizeof(m_translation_old) / sizeof(float);
    }
    {
    static float m_bullet_translation_start[] = { 0, 0, 0 };
    FLYm_ON.bullet_translation_start = m_bullet_translation_start;
    FLYm_ON.bullet_translation_start_length = sizeof(m_bullet_translation_start) / sizeof(float);
    }
    {
    static float m_bullet_direction[] = { 0, 0, 0 };
    FLYm_ON.bullet_direction = m_bullet_direction;
    FLYm_ON.bullet_direction_length = sizeof(m_bullet_direction) / sizeof(float);
    }
    FLYm_ON.bulletway_length = 0;
    FLYm_ON.old_time = 0;
    FLYm_ON.factor = 10;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    FLYm_ON.translation_in = m_translation_in;
    FLYm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    FLYm_ON.time_in = 0;
    FLYm_ON.enter_time = 0;
    FLYm_ON.exit_time = 0;
    FLYm_ON.fire_in = false;
    {
    static float m_translation_out[] = { 0, 0, 0 };
    FLYm_ON.translation_out = m_translation_out;
    FLYm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    {
    static float m_top_translation_out[] = { 0, 0, 0 };
    FLYm_ON.top_translation_out = m_top_translation_out;
    FLYm_ON.top_translation_out_length = sizeof(m_top_translation_out) / sizeof(float);
    }
    FLYm_ON.bullet_choice_out = 0;
    {
    static float m_bullet_translation_out[] = { 0, 0, 0 };
    FLYm_ON.bullet_translation_out = m_bullet_translation_out;
    FLYm_ON.bullet_translation_out_length = sizeof(m_bullet_translation_out) / sizeof(float);
    }
    THINGm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0.1;  \
         hide_out[1]=0.1;  \
         hide_out[2]=0.1;  \
         }  \
      else  \
         {  \
         hide_out[0]=1.0;  \
         hide_out[1]=1.0;  \
         hide_out[2]=1.0;  \
         }  \
      }      \
   function fire_in(val)              \
      {  \
      if (val)  \
         {  \
         hide_out[0]=1;  \
         hide_out[1]=1;  \
         hide_out[2]=1;  \
         }                    \
      }  \
   ", "c++:  \
      X3dScript_THING_SCRIPT *self = (X3dScript_THING_SCRIPT *)node;  \
      if (self->isover_in)  \
         {  \
         self->hide_out[0]=0.1;  \
         self->hide_out[1]=0.1;  \
         self->hide_out[2]=0.1;  \
         }  \
      else  \
         {  \
         self->hide_out[0]=1.0;  \
         self->hide_out[1]=1.0;  \
         self->hide_out[2]=1.0;  \
         }  \
      if (self->fire_in)  \
         {  \
         self->hide_out[0]=1;  \
         self->hide_out[1]=1;  \
         self->hide_out[2]=1;  \
         }                    \
" };
    THINGm_SCRIPT.url = m_url;
    THINGm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    THINGm_SCRIPT.directOutput = false;
    THINGm_SCRIPT.mustEvaluate = false;
    THINGm_SCRIPT.isover_in = false;
    THINGm_SCRIPT.fire_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    THINGm_SCRIPT.hide_out = m_hide_out;
    THINGm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    Viewpoint1.m_parent = (X3dNode *)&Group1;
    Viewpoint1.description = "";
    Viewpoint1.fieldOfView = 0.785398;
    Viewpoint1.jump = true;
    {
    static float m_orientation[] = { 0.216404, 6.832186e-2, -0.973910, 0 };
    Viewpoint1.orientation = m_orientation;
    Viewpoint1.orientation_length = sizeof(m_orientation) / sizeof(float);
    }
    {
    static float m_position[] = { 1.070576e-4, -3.391865e-4, 10 };
    Viewpoint1.position = m_position;
    Viewpoint1.position_length = sizeof(m_position) / sizeof(float);
    }
    Viewpoint1.set_bind = false;
    Viewpoint1.bindTime = 0;
    Viewpoint1.isBound = false;
    Viewpoint2.m_parent = (X3dNode *)&Group1;
    Viewpoint2.description = "";
    Viewpoint2.fieldOfView = 0.785398;
    Viewpoint2.jump = true;
    {
    static float m_orientation[] = { 0, 0, 1, 0 };
    Viewpoint2.orientation = m_orientation;
    Viewpoint2.orientation_length = sizeof(m_orientation) / sizeof(float);
    }
    {
    static float m_position[] = { 0, 0, 10 };
    Viewpoint2.position = m_position;
    Viewpoint2.position_length = sizeof(m_position) / sizeof(float);
    }
    Viewpoint2.set_bind = false;
    Viewpoint2.bindTime = 0;
    Viewpoint2.isBound = false;
    WorldInfo5.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo5.info = m_info;
    WorldInfo5.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo5.title = "Tux warbird";
    Transform18.route_sources =  NULL;
    Transform18.num_route_source = 0;
    Transform18.m_parent = (X3dNode *)&Group1;
    Shape15.m_parent = (X3dNode *)&Transform18;
    Appearance15.m_parent = (X3dNode *)&Shape15;
    Material13.m_parent = (X3dNode *)&Appearance15;
    Material13.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material13.diffuseColor = m_diffuseColor;
    Material13.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material13.emissiveColor = m_emissiveColor;
    Material13.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material13.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material13.specularColor = m_specularColor;
    Material13.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material13.transparency = 0;
    Appearance15.material = &Material13;
    Appearance15.texture = NULL;
    Appearance15.textureTransform = NULL;
    Text1.m_parent = (X3dNode *)&Shape15;
    Text1.fontStyle = NULL;
    {
    static float m_length[1];
    Text1.length = NULL;
    Text1.length_length = 0;
    }
    Text1.maxExtent = 0;
    {
    static const char* m_string[] = { "" };
    Text1.string = m_string;
    Text1.string_length = sizeof(m_string) / sizeof(const char*);
    }
    Shape15.appearance = &Appearance15;
    Shape15.geometry = &Text1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform18.rotation = m_rotation;
    Transform18.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -5.552411, -2.600564, 0 };
    Transform18.translation = m_translation;
    Transform18.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform18.bboxCenter = m_bboxCenter;
    Transform18.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform18.bboxSize = m_bboxSize;
    Transform18.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform18.center = m_center;
    Transform18.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform18.scale = m_scale;
    Transform18.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform18.scaleOrientation = m_scaleOrientation;
    Transform18.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape15 };
    Transform18.children = m_children;
    Transform18.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform18.addChildren = NULL;
    Transform18.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform18.removeChildren = NULL;
    Transform18.removeChildren_length = 0;
    }
    Group8.m_parent = (X3dNode *)&Group1;
    Switch1.m_parent = (X3dNode *)&Group8;
    Transform19.route_sources =  NULL;
    Transform19.num_route_source = 0;
    Transform19.m_parent = (X3dNode *)&Switch1;
    Shape16.m_parent = (X3dNode *)&Transform19;
    Appearance16.m_parent = (X3dNode *)&Shape16;
    Material14.m_parent = (X3dNode *)&Appearance16;
    Material14.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material14.diffuseColor = m_diffuseColor;
    Material14.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material14.emissiveColor = m_emissiveColor;
    Material14.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material14.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material14.specularColor = m_specularColor;
    Material14.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material14.transparency = 0;
    Appearance16.material = &Material14;
    Appearance16.texture = NULL;
    Appearance16.textureTransform = NULL;
    Text1m_5.m_parent = (X3dNode *)&Shape16;
    Text1m_5.fontStyle = NULL;
    {
    static float m_length[1];
    Text1m_5.length = NULL;
    Text1m_5.length_length = 0;
    }
    Text1m_5.maxExtent = 0;
    {
    static const char* m_string[] = { "Resistance is futile. You will be assimilat...", "General Protection Fault in MSBILL32.DLL" };
    Text1m_5.string = m_string;
    Text1m_5.string_length = sizeof(m_string) / sizeof(const char*);
    }
    Shape16.appearance = &Appearance16;
    Shape16.geometry = &Text1m_5;
    DUMMYm_TEXT.m_parent = (X3dNode *)&Transform19;
    DUMMYm_TEXT.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    DUMMYm_TEXT.hitNormal_changed = m_hitNormal_changed;
    DUMMYm_TEXT.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    DUMMYm_TEXT.hitPoint_changed = m_hitPoint_changed;
    DUMMYm_TEXT.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    DUMMYm_TEXT.hitTexCoord_changed = m_hitTexCoord_changed;
    DUMMYm_TEXT.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    DUMMYm_TEXT.isActive = false;
    DUMMYm_TEXT.isOver = false;
    DUMMYm_TEXT.touchTime = 0;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform19.rotation = m_rotation;
    Transform19.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -10.787153, 0, -18.900000 };
    Transform19.translation = m_translation;
    Transform19.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform19.bboxCenter = m_bboxCenter;
    Transform19.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform19.bboxSize = m_bboxSize;
    Transform19.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform19.center = m_center;
    Transform19.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform19.scale = m_scale;
    Transform19.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform19.scaleOrientation = m_scaleOrientation;
    Transform19.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape16, &DUMMYm_TEXT };
    Transform19.children = m_children;
    Transform19.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform19.addChildren = NULL;
    Transform19.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform19.removeChildren = NULL;
    Transform19.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform19 };
    Switch1.children = m_children;
    Switch1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    Switch1.whichChoice = -1;
    {
    static X3dNode* m_children[] = { &Switch1 };
    Group8.children = m_children;
    Group8.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group8.bboxCenter = m_bboxCenter;
    Group8.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group8.bboxSize = m_bboxSize;
    Group8.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group8.addChildren = NULL;
    Group8.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group8.removeChildren = NULL;
    Group8.removeChildren_length = 0;
    }
    Shape17.m_parent = (X3dNode *)&Group1;
    Appearance17.m_parent = (X3dNode *)&Shape17;
    Material15.m_parent = (X3dNode *)&Appearance17;
    Material15.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0, 0, 1 };
    Material15.diffuseColor = m_diffuseColor;
    Material15.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material15.emissiveColor = m_emissiveColor;
    Material15.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material15.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material15.specularColor = m_specularColor;
    Material15.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material15.transparency = 0;
    Appearance17.material = &Material15;
    Appearance17.texture = NULL;
    Appearance17.textureTransform = NULL;
    IndexedFaceSet15.setGlName(14);
    IndexedFaceSet15.m_parent = (X3dNode *)&Shape17;
    COORDm_4.m_parent = (X3dNode *)&IndexedFaceSet15;
    {
    static float m_point[] = { -600, -600, -35, 600, -600, -35, -600, 600, -35, 600, 600, -35 };
    COORDm_4.point = m_point;
    COORDm_4.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal10.m_parent = (X3dNode *)&IndexedFaceSet15;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal10.vector = m_vector;
    Normal10.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet15.color = NULL;
    IndexedFaceSet15.coord = &COORDm_4;
    IndexedFaceSet15.normal = &Normal10;
    IndexedFaceSet15.texCoord = NULL;
    IndexedFaceSet15.ccw = true;
    {
    static int m_colorIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet15.colorIndex = m_colorIndex;
    IndexedFaceSet15.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet15.colorPerVertex = true;
    IndexedFaceSet15.convex = true;
    {
    static int m_coordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet15.coordIndex = m_coordIndex;
    IndexedFaceSet15.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet15.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet15.normalIndex = m_normalIndex;
    IndexedFaceSet15.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet15.normalPerVertex = true;
    IndexedFaceSet15.solid = true;
    {
    static int m_texCoordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet15.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet15.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet15.set_colorIndex = NULL;
    IndexedFaceSet15.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet15.set_coordIndex = NULL;
    IndexedFaceSet15.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet15.set_normalIndex = NULL;
    IndexedFaceSet15.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet15.set_texCoordIndex = NULL;
    IndexedFaceSet15.set_texCoordIndex_length = 0;
    }
    Shape17.appearance = &Appearance17;
    Shape17.geometry = &IndexedFaceSet15;
    BOX2m_HIDE.route_sources =  NULL;
    BOX2m_HIDE.num_route_source = 0;
    BOX2m_HIDE.m_parent = (X3dNode *)&Group1;
    BOX2m_SWITCH.m_parent = (X3dNode *)&BOX2m_HIDE;
    BOX2m_MOVE.route_sources =  NULL;
    BOX2m_MOVE.num_route_source = 0;
    BOX2m_MOVE.m_parent = (X3dNode *)&BOX2m_SWITCH;
    BOX2m_SENSOR.m_parent = (X3dNode *)&BOX2m_MOVE;
    BOX2m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX2m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX2m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX2m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX2m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX2m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX2m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX2m_SENSOR.isActive = false;
    BOX2m_SENSOR.isOver = false;
    BOX2m_SENSOR.touchTime = 0;
    Inline10.m_parent = (X3dNode *)&BOX2m_MOVE;
    Group9.m_parent = (X3dNode *)&Inline10;
    Shape18.m_parent = (X3dNode *)NULL;
    Appearance18.m_parent = (X3dNode *)NULL;
    Material16.m_parent = (X3dNode *)NULL;
    Material16.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material16.diffuseColor = m_diffuseColor;
    Material16.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material16.emissiveColor = m_emissiveColor;
    Material16.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material16.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material16.specularColor = m_specularColor;
    Material16.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material16.transparency = 0;
    Appearance18.material = &Material16;
    Appearance18.texture = NULL;
    Appearance18.textureTransform = NULL;
    IndexedFaceSet16.setGlName(15);
    IndexedFaceSet16.m_parent = (X3dNode *)NULL;
    Color9.m_parent = (X3dNode *)&IndexedFaceSet16;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color9.color = m_color;
    Color9.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate25.m_parent = (X3dNode *)&IndexedFaceSet16;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate25.point = m_point;
    Coordinate25.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal34.m_parent = (X3dNode *)&IndexedFaceSet16;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal34.vector = m_vector;
    Normal34.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet16.color = &Color9;
    IndexedFaceSet16.coord = &Coordinate25;
    IndexedFaceSet16.normal = &Normal34;
    IndexedFaceSet16.texCoord = NULL;
    IndexedFaceSet16.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 32321219, -1, 0, 0, 0, -1, 0, 6, 0, -1, 0, 0, 0, -1, 8, 11, 7, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet16.colorIndex = m_colorIndex;
    IndexedFaceSet16.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet16.colorPerVertex = false;
    IndexedFaceSet16.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet16.coordIndex = m_coordIndex;
    IndexedFaceSet16.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet16.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet16.normalIndex = m_normalIndex;
    IndexedFaceSet16.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet16.normalPerVertex = true;
    IndexedFaceSet16.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet16.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet16.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet16.set_colorIndex = NULL;
    IndexedFaceSet16.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet16.set_coordIndex = NULL;
    IndexedFaceSet16.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet16.set_normalIndex = NULL;
    IndexedFaceSet16.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet16.set_texCoordIndex = NULL;
    IndexedFaceSet16.set_texCoordIndex_length = 0;
    }
    Shape18.appearance = &Appearance18;
    Shape18.geometry = &IndexedFaceSet16;
    Shape19.m_parent = (X3dNode *)NULL;
    Appearance19.m_parent = (X3dNode *)NULL;
    Material17.m_parent = (X3dNode *)NULL;
    Material17.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material17.diffuseColor = m_diffuseColor;
    Material17.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material17.emissiveColor = m_emissiveColor;
    Material17.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material17.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material17.specularColor = m_specularColor;
    Material17.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material17.transparency = 0;
    ImageTexture3.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture3.url = m_url;
    ImageTexture3.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture3.repeatS = true;
    ImageTexture3.repeatT = true;
    Appearance19.material = &Material17;
    Appearance19.texture = &ImageTexture3;
    Appearance19.textureTransform = NULL;
    IndexedFaceSet17.setGlName(16);
    IndexedFaceSet17.m_parent = (X3dNode *)NULL;
    Coordinate27.m_parent = (X3dNode *)&IndexedFaceSet17;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate27.point = m_point;
    Coordinate27.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal35.m_parent = (X3dNode *)&IndexedFaceSet17;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal35.vector = m_vector;
    Normal35.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate13.m_parent = (X3dNode *)&IndexedFaceSet17;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate13.point = m_point;
    TextureCoordinate13.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet17.color = NULL;
    IndexedFaceSet17.coord = &Coordinate27;
    IndexedFaceSet17.normal = &Normal35;
    IndexedFaceSet17.texCoord = &TextureCoordinate13;
    IndexedFaceSet17.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet17.colorIndex = m_colorIndex;
    IndexedFaceSet17.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet17.colorPerVertex = false;
    IndexedFaceSet17.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet17.coordIndex = m_coordIndex;
    IndexedFaceSet17.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet17.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet17.normalIndex = m_normalIndex;
    IndexedFaceSet17.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet17.normalPerVertex = true;
    IndexedFaceSet17.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet17.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet17.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet17.set_colorIndex = NULL;
    IndexedFaceSet17.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet17.set_coordIndex = NULL;
    IndexedFaceSet17.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet17.set_normalIndex = NULL;
    IndexedFaceSet17.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet17.set_texCoordIndex = NULL;
    IndexedFaceSet17.set_texCoordIndex_length = 0;
    }
    Shape19.appearance = &Appearance19;
    Shape19.geometry = &IndexedFaceSet17;
    {
    static X3dNode* m_children[] = { NULL, &Shape18, NULL, &Shape19 };
    Group9.children = m_children;
    Group9.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group9.bboxCenter = m_bboxCenter;
    Group9.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group9.bboxSize = m_bboxSize;
    Group9.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group9.addChildren = NULL;
    Group9.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group9.removeChildren = NULL;
    Group9.removeChildren_length = 0;
    }
    WorldInfo6.m_parent = (X3dNode *)&Inline10;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo6.info = m_info;
    WorldInfo6.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo6.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group9, NULL, &WorldInfo6 };
    Inline10.children = m_children;
    Inline10.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline10.bboxCenter = m_bboxCenter;
    Inline10.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline10.bboxSize = m_bboxSize;
    Inline10.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline10.addChildren = NULL;
    Inline10.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline10.removeChildren = NULL;
    Inline10.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX2m_MOVE.rotation = m_rotation;
    BOX2m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX2m_MOVE.translation = m_translation;
    BOX2m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX2m_MOVE.bboxCenter = m_bboxCenter;
    BOX2m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX2m_MOVE.bboxSize = m_bboxSize;
    BOX2m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX2m_MOVE.center = m_center;
    BOX2m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX2m_MOVE.scale = m_scale;
    BOX2m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX2m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX2m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX2m_SENSOR, &Inline10 };
    BOX2m_MOVE.children = m_children;
    BOX2m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX2m_MOVE.addChildren = NULL;
    BOX2m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX2m_MOVE.removeChildren = NULL;
    BOX2m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX2m_MOVE };
    BOX2m_SWITCH.children = m_children;
    BOX2m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX2m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX2m_HIDE.rotation = m_rotation;
    BOX2m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX2m_HIDE.translation = m_translation;
    BOX2m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX2m_HIDE.bboxCenter = m_bboxCenter;
    BOX2m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX2m_HIDE.bboxSize = m_bboxSize;
    BOX2m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX2m_HIDE.center = m_center;
    BOX2m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX2m_HIDE.scale = m_scale;
    BOX2m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX2m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX2m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX2m_SWITCH };
    BOX2m_HIDE.children = m_children;
    BOX2m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX2m_HIDE.addChildren = NULL;
    BOX2m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX2m_HIDE.removeChildren = NULL;
    BOX2m_HIDE.removeChildren_length = 0;
    }
    BOX2m_HITm_CHECK.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function initialize()  \
         {  \
         choice_out=0;  \
         deleted_out=0;  \
         }  \
      function translation_in(val)    \
         {  \
         if (!deleted)  \
            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)  \
                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)  \
                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)  \
               )  \
               {  \
               choice_out=1;  \
               deleted_out=1;  \
               deleted=1;  \
               }  \
            else  \
               choice_out=0;  \
         else  \
            choice_out=1;  \
         }  \
           \
      function box_position(val)  \
         {  \
         box[0]=val[0];  \
         box[1]=val[1];  \
         box[2]=val[2];  \
         }    \
      ", "c++:  \
   X3dScript_BOX2_HIT_CHECK *self = (X3dScript_BOX2_HIT_CHECK *)node;  \
   static bool BOX2_HIT_CHECK_init = false;  \
   if (!BOX2_HIT_CHECK_init) {  \
       self->choice_out = 0;  \
       self->deleted_out = 0;  \
       BOX2_HIT_CHECK_init = true;  \
   } else if (!self->deleted)   \
      {  \
      self->box[0] = self->box_position[0];  \
      self->box[1] = self->box_position[1];  \
      self->box[2] = self->box_position[2];  \
      if (  \
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) &&   \
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) &&   \
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) &&   \
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) &&   \
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) &&   \
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)  \
         )  \
         {  \
         self->choice_out = 1;  \
         self->deleted_out = 1;  \
         self->deleted = 1;  \
         }  \
      else  \
         self->choice_out = 0;  \
   }  \
   else  \
      self->choice_out = 1;  \
" };
    BOX2m_HITm_CHECK.url = m_url;
    BOX2m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_HITm_CHECK.directOutput = false;
    BOX2m_HITm_CHECK.mustEvaluate = false;
    BOX2m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX2m_HITm_CHECK.box = m_box;
    BOX2m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX2m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX2m_HITm_CHECK.translation_in = m_translation_in;
    BOX2m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX2m_HITm_CHECK.box_position = m_box_position;
    BOX2m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX2m_HITm_CHECK.deleted_out = 0;
    BOX2m_HITm_CHECK.choice_out = 0;
    BOX2m_MOVEm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function nextsteps()  \
         {  \
         step_translation[0]=(Math.random()-0.5)*0.04;  \
         step_translation[1]=(Math.random()-0.5)*0.04;  \
         step_translation[2]=(Math.random()-0.5)*0.04;  \
         step=Math.random()*200+1;  \
         }  \
      function initialize()  \
         {  \
         translation_out[0]=Math.random()*8-4;  \
         translation_out[1]=Math.random()*6-3;  \
         translation_out[2]=Math.random()*-30-1;  \
         nextsteps();  \
         }  \
      function translation_in(val)    \
         {  \
         step--;  \
         if (step<=0)  \
            nextsteps();  \
         if (   (translation_out[0]+step_translation[0]<4)   \
             && (translation_out[0]+step_translation[0]>-4))   \
            translation_out[0]+=step_translation[0];  \
         if (   (translation_out[1]+step_translation[1]<3)   \
             && (translation_out[1]+step_translation[1]>-3))   \
             translation_out[1]+=step_translation[1];  \
         if (   (translation_out[2]+step_translation[2]<-1)   \
             && (translation_out[2]+step_translation[2]>-30))   \
            translation_out[2]+=step_translation[2];  \
         }  \
      ", "c++:  \
   X3dScript_BOX2_MOVE_ON *self = (X3dScript_BOX2_MOVE_ON *)node;  \
   static bool BOX2_MOVE_ON_init = false;  \
   if (!BOX2_MOVE_ON_init) {  \
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;  \
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;  \
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;  \
       BOX2_MOVE_ON_init = true;  \
   }           \
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   if ((self->translation_out[0] + self->step_translation[0] < 4) &&   \
       (self->translation_out[0] + self->step_translation[0] > -4))   \
       self->translation_out[0] += self->step_translation[0];  \
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&  \
       (self->translation_out[1] + self->step_translation[1] > -3))   \
       self->translation_out[1] += self->step_translation[1];  \
   if ((self->translation_out[2] + self->step_translation[2] < -1) &&   \
       (self->translation_out[2] + self->step_translation[2] > -30))   \
       self->translation_out[2] += self->step_translation[2];  \
" };
    BOX2m_MOVEm_ON.url = m_url;
    BOX2m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_MOVEm_ON.directOutput = false;
    BOX2m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.step_translation = m_step_translation;
    BOX2m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX2m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.translation_in = m_translation_in;
    BOX2m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.translation_out = m_translation_out;
    BOX2m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX2m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=100;  \
         }  \
      else  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=0;  \
         }  \
      }      \
   ", "c++:  \
   X3dScript_BOX2_HIDE_SCRIPT *self = (X3dScript_BOX2_HIDE_SCRIPT *)node;  \
   if (self->isover_in) {               \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 100;  \
   } else {  \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 0;  \
   }  \
" };
    BOX2m_HIDEm_SCRIPT.url = m_url;
    BOX2m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_HIDEm_SCRIPT.directOutput = false;
    BOX2m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX2m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX2m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX2m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX3m_HIDE.route_sources =  NULL;
    BOX3m_HIDE.num_route_source = 0;
    BOX3m_HIDE.m_parent = (X3dNode *)&Group1;
    BOX3m_SWITCH.m_parent = (X3dNode *)&BOX3m_HIDE;
    BOX3m_MOVE.route_sources =  NULL;
    BOX3m_MOVE.num_route_source = 0;
    BOX3m_MOVE.m_parent = (X3dNode *)&BOX3m_SWITCH;
    BOX3m_SENSOR.m_parent = (X3dNode *)&BOX3m_MOVE;
    BOX3m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX3m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX3m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX3m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX3m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX3m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX3m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX3m_SENSOR.isActive = false;
    BOX3m_SENSOR.isOver = false;
    BOX3m_SENSOR.touchTime = 0;
    Inline11.m_parent = (X3dNode *)&BOX3m_MOVE;
    Group10.m_parent = (X3dNode *)&Inline11;
    Shape20.m_parent = (X3dNode *)NULL;
    Appearance20.m_parent = (X3dNode *)NULL;
    Material18.m_parent = (X3dNode *)NULL;
    Material18.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material18.diffuseColor = m_diffuseColor;
    Material18.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material18.emissiveColor = m_emissiveColor;
    Material18.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material18.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material18.specularColor = m_specularColor;
    Material18.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material18.transparency = 0;
    Appearance20.material = &Material18;
    Appearance20.texture = NULL;
    Appearance20.textureTransform = NULL;
    IndexedFaceSet18.setGlName(17);
    IndexedFaceSet18.m_parent = (X3dNode *)NULL;
    Color11.m_parent = (X3dNode *)&IndexedFaceSet18;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color11.color = m_color;
    Color11.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate29.m_parent = (X3dNode *)&IndexedFaceSet18;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate29.point = m_point;
    Coordinate29.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal36.m_parent = (X3dNode *)&IndexedFaceSet18;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal36.vector = m_vector;
    Normal36.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet18.color = &Color11;
    IndexedFaceSet18.coord = &Coordinate29;
    IndexedFaceSet18.normal = &Normal36;
    IndexedFaceSet18.texCoord = NULL;
    IndexedFaceSet18.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 32416939, -1, 0, 0, 0, -1, 0, 6, 0, -1, 0, 0, 0, -1, 8, 11, 7, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet18.colorIndex = m_colorIndex;
    IndexedFaceSet18.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet18.colorPerVertex = false;
    IndexedFaceSet18.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet18.coordIndex = m_coordIndex;
    IndexedFaceSet18.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet18.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet18.normalIndex = m_normalIndex;
    IndexedFaceSet18.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet18.normalPerVertex = true;
    IndexedFaceSet18.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet18.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet18.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet18.set_colorIndex = NULL;
    IndexedFaceSet18.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet18.set_coordIndex = NULL;
    IndexedFaceSet18.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet18.set_normalIndex = NULL;
    IndexedFaceSet18.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet18.set_texCoordIndex = NULL;
    IndexedFaceSet18.set_texCoordIndex_length = 0;
    }
    Shape20.appearance = &Appearance20;
    Shape20.geometry = &IndexedFaceSet18;
    Shape21.m_parent = (X3dNode *)NULL;
    Appearance21.m_parent = (X3dNode *)NULL;
    Material19.m_parent = (X3dNode *)NULL;
    Material19.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material19.diffuseColor = m_diffuseColor;
    Material19.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material19.emissiveColor = m_emissiveColor;
    Material19.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material19.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material19.specularColor = m_specularColor;
    Material19.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material19.transparency = 0;
    ImageTexture4.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture4.url = m_url;
    ImageTexture4.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture4.repeatS = true;
    ImageTexture4.repeatT = true;
    Appearance21.material = &Material19;
    Appearance21.texture = &ImageTexture4;
    Appearance21.textureTransform = NULL;
    IndexedFaceSet19.setGlName(18);
    IndexedFaceSet19.m_parent = (X3dNode *)NULL;
    Coordinate31.m_parent = (X3dNode *)&IndexedFaceSet19;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate31.point = m_point;
    Coordinate31.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal37.m_parent = (X3dNode *)&IndexedFaceSet19;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal37.vector = m_vector;
    Normal37.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate15.m_parent = (X3dNode *)&IndexedFaceSet19;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate15.point = m_point;
    TextureCoordinate15.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet19.color = NULL;
    IndexedFaceSet19.coord = &Coordinate31;
    IndexedFaceSet19.normal = &Normal37;
    IndexedFaceSet19.texCoord = &TextureCoordinate15;
    IndexedFaceSet19.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet19.colorIndex = m_colorIndex;
    IndexedFaceSet19.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet19.colorPerVertex = false;
    IndexedFaceSet19.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet19.coordIndex = m_coordIndex;
    IndexedFaceSet19.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet19.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet19.normalIndex = m_normalIndex;
    IndexedFaceSet19.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet19.normalPerVertex = true;
    IndexedFaceSet19.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet19.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet19.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet19.set_colorIndex = NULL;
    IndexedFaceSet19.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet19.set_coordIndex = NULL;
    IndexedFaceSet19.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet19.set_normalIndex = NULL;
    IndexedFaceSet19.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet19.set_texCoordIndex = NULL;
    IndexedFaceSet19.set_texCoordIndex_length = 0;
    }
    Shape21.appearance = &Appearance21;
    Shape21.geometry = &IndexedFaceSet19;
    {
    static X3dNode* m_children[] = { NULL, &Shape20, NULL, &Shape21 };
    Group10.children = m_children;
    Group10.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group10.bboxCenter = m_bboxCenter;
    Group10.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group10.bboxSize = m_bboxSize;
    Group10.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group10.addChildren = NULL;
    Group10.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group10.removeChildren = NULL;
    Group10.removeChildren_length = 0;
    }
    WorldInfo7.m_parent = (X3dNode *)&Inline11;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo7.info = m_info;
    WorldInfo7.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo7.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group10, NULL, &WorldInfo7 };
    Inline11.children = m_children;
    Inline11.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline11.bboxCenter = m_bboxCenter;
    Inline11.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline11.bboxSize = m_bboxSize;
    Inline11.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline11.addChildren = NULL;
    Inline11.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline11.removeChildren = NULL;
    Inline11.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX3m_MOVE.rotation = m_rotation;
    BOX3m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX3m_MOVE.translation = m_translation;
    BOX3m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX3m_MOVE.bboxCenter = m_bboxCenter;
    BOX3m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX3m_MOVE.bboxSize = m_bboxSize;
    BOX3m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX3m_MOVE.center = m_center;
    BOX3m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX3m_MOVE.scale = m_scale;
    BOX3m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX3m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX3m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX3m_SENSOR, &Inline11 };
    BOX3m_MOVE.children = m_children;
    BOX3m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX3m_MOVE.addChildren = NULL;
    BOX3m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX3m_MOVE.removeChildren = NULL;
    BOX3m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX3m_MOVE };
    BOX3m_SWITCH.children = m_children;
    BOX3m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX3m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX3m_HIDE.rotation = m_rotation;
    BOX3m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX3m_HIDE.translation = m_translation;
    BOX3m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX3m_HIDE.bboxCenter = m_bboxCenter;
    BOX3m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX3m_HIDE.bboxSize = m_bboxSize;
    BOX3m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX3m_HIDE.center = m_center;
    BOX3m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX3m_HIDE.scale = m_scale;
    BOX3m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX3m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX3m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX3m_SWITCH };
    BOX3m_HIDE.children = m_children;
    BOX3m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX3m_HIDE.addChildren = NULL;
    BOX3m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX3m_HIDE.removeChildren = NULL;
    BOX3m_HIDE.removeChildren_length = 0;
    }
    BOX3m_HITm_CHECK.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function initialize()  \
         {  \
         choice_out=0;  \
         deleted_out=0;  \
         }  \
      function translation_in(val)    \
         {  \
         if (!deleted)  \
            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)  \
                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)  \
                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)  \
               )  \
               {  \
               choice_out=1;  \
               deleted_out=1;  \
               deleted=1;  \
               }  \
            else  \
               choice_out=0;  \
         else  \
            choice_out=1;  \
         }  \
           \
      function box_position(val)  \
         {  \
         box[0]=val[0];  \
         box[1]=val[1];  \
         box[2]=val[2];  \
         }    \
      ", "c++:  \
   X3dScript_BOX3_HIT_CHECK *self = (X3dScript_BOX3_HIT_CHECK *)node;  \
   static bool BOX3_HIT_CHECK_init = false;  \
   if (!BOX3_HIT_CHECK_init) {  \
       self->choice_out = 0;  \
       self->deleted_out = 9;  \
       BOX3_HIT_CHECK_init = true;  \
   } else if (!self->deleted)   \
      {  \
      self->box[0] = self->box_position[0];  \
      self->box[1] = self->box_position[1];  \
      self->box[2] = self->box_position[2];  \
      if (  \
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) &&   \
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) &&   \
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) &&   \
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) &&   \
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) &&   \
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)  \
         )  \
         {  \
         self->choice_out = 1;  \
         self->deleted_out = 1;  \
         self->deleted = 1;  \
         }  \
      else  \
         self->choice_out = 0;  \
   }  \
   else  \
      self->choice_out = 1;  \
" };
    BOX3m_HITm_CHECK.url = m_url;
    BOX3m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_HITm_CHECK.directOutput = false;
    BOX3m_HITm_CHECK.mustEvaluate = false;
    BOX3m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX3m_HITm_CHECK.box = m_box;
    BOX3m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX3m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX3m_HITm_CHECK.translation_in = m_translation_in;
    BOX3m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX3m_HITm_CHECK.box_position = m_box_position;
    BOX3m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX3m_HITm_CHECK.deleted_out = 0;
    BOX3m_HITm_CHECK.choice_out = 0;
    BOX3m_MOVEm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function nextsteps()  \
         {  \
         step_translation[0]=(Math.random()-0.5)*0.04;  \
         step_translation[1]=(Math.random()-0.5)*0.04;  \
         step_translation[2]=(Math.random()-0.5)*0.04;  \
         step=Math.random()*200+1;  \
         }  \
      function initialize()  \
         {  \
         translation_out[0]=Math.random()*8-4;  \
         translation_out[1]=Math.random()*6-3;  \
         translation_out[2]=Math.random()*-30-1;  \
         nextsteps();  \
         }  \
      function translation_in(val)    \
         {  \
         step--;  \
         if (step<=0)  \
            nextsteps();  \
         if (   (translation_out[0]+step_translation[0]<4)   \
             && (translation_out[0]+step_translation[0]>-4))   \
            translation_out[0]+=step_translation[0];  \
         if (   (translation_out[1]+step_translation[1]<3)   \
             && (translation_out[1]+step_translation[1]>-3))   \
             translation_out[1]+=step_translation[1];  \
         if (   (translation_out[2]+step_translation[2]<-1)   \
             && (translation_out[2]+step_translation[2]>-30))   \
            translation_out[2]+=step_translation[2];  \
         }  \
      ", "c++:  \
   X3dScript_BOX3_MOVE_ON *self = (X3dScript_BOX3_MOVE_ON *)node;  \
   static bool BOX3_MOVE_ON_init = false;  \
   if (!BOX3_MOVE_ON_init) {  \
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;  \
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;  \
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;  \
       BOX3_MOVE_ON_init = true;  \
   }           \
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   if ((self->translation_out[0] + self->step_translation[0] < 4) &&   \
       (self->translation_out[0] + self->step_translation[0] > -4))   \
       self->translation_out[0] += self->step_translation[0];  \
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&  \
       (self->translation_out[1] + self->step_translation[1] > -3))   \
       self->translation_out[1] += self->step_translation[1];  \
   if ((self->translation_out[2] + self->step_translation[2] < -1) &&   \
       (self->translation_out[2] + self->step_translation[2] > -30))   \
       self->translation_out[2] += self->step_translation[2];  \
" };
    BOX3m_MOVEm_ON.url = m_url;
    BOX3m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_MOVEm_ON.directOutput = false;
    BOX3m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.step_translation = m_step_translation;
    BOX3m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX3m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.translation_in = m_translation_in;
    BOX3m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.translation_out = m_translation_out;
    BOX3m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX3m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=100;  \
         }  \
      else  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=0;  \
         }  \
      }      \
   ", "c++:  \
   X3dScript_BOX3_HIDE_SCRIPT *self = (X3dScript_BOX3_HIDE_SCRIPT *)node;  \
   if (self->isover_in) {               \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 100;  \
   } else {  \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 0;  \
   }  \
" };
    BOX3m_HIDEm_SCRIPT.url = m_url;
    BOX3m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_HIDEm_SCRIPT.directOutput = false;
    BOX3m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX3m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX3m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX3m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX4m_HIDE.route_sources =  NULL;
    BOX4m_HIDE.num_route_source = 0;
    BOX4m_HIDE.m_parent = (X3dNode *)&Group1;
    BOX4m_SWITCH.m_parent = (X3dNode *)&BOX4m_HIDE;
    BOX4m_MOVE.route_sources =  NULL;
    BOX4m_MOVE.num_route_source = 0;
    BOX4m_MOVE.m_parent = (X3dNode *)&BOX4m_SWITCH;
    BOX4m_SENSOR.m_parent = (X3dNode *)&BOX4m_MOVE;
    BOX4m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX4m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX4m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX4m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX4m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX4m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX4m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX4m_SENSOR.isActive = false;
    BOX4m_SENSOR.isOver = false;
    BOX4m_SENSOR.touchTime = 0;
    Inline12.m_parent = (X3dNode *)&BOX4m_MOVE;
    Group11.m_parent = (X3dNode *)&Inline12;
    Shape22.m_parent = (X3dNode *)NULL;
    Appearance22.m_parent = (X3dNode *)NULL;
    Material20.m_parent = (X3dNode *)NULL;
    Material20.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material20.diffuseColor = m_diffuseColor;
    Material20.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material20.emissiveColor = m_emissiveColor;
    Material20.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material20.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material20.specularColor = m_specularColor;
    Material20.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material20.transparency = 0;
    Appearance22.material = &Material20;
    Appearance22.texture = NULL;
    Appearance22.textureTransform = NULL;
    IndexedFaceSet20.setGlName(19);
    IndexedFaceSet20.m_parent = (X3dNode *)NULL;
    Color13.m_parent = (X3dNode *)&IndexedFaceSet20;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color13.color = m_color;
    Color13.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate33.m_parent = (X3dNode *)&IndexedFaceSet20;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate33.point = m_point;
    Coordinate33.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal38.m_parent = (X3dNode *)&IndexedFaceSet20;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal38.vector = m_vector;
    Normal38.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet20.color = &Color13;
    IndexedFaceSet20.coord = &Coordinate33;
    IndexedFaceSet20.normal = &Normal38;
    IndexedFaceSet20.texCoord = NULL;
    IndexedFaceSet20.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 32526340, -1, 0, 0, 0, -1, 0, 6, 0, -1, 0, 0, 0, -1, 8, 11, 7, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet20.colorIndex = m_colorIndex;
    IndexedFaceSet20.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet20.colorPerVertex = false;
    IndexedFaceSet20.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet20.coordIndex = m_coordIndex;
    IndexedFaceSet20.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet20.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet20.normalIndex = m_normalIndex;
    IndexedFaceSet20.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet20.normalPerVertex = true;
    IndexedFaceSet20.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet20.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet20.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet20.set_colorIndex = NULL;
    IndexedFaceSet20.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet20.set_coordIndex = NULL;
    IndexedFaceSet20.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet20.set_normalIndex = NULL;
    IndexedFaceSet20.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet20.set_texCoordIndex = NULL;
    IndexedFaceSet20.set_texCoordIndex_length = 0;
    }
    Shape22.appearance = &Appearance22;
    Shape22.geometry = &IndexedFaceSet20;
    Shape23.m_parent = (X3dNode *)NULL;
    Appearance23.m_parent = (X3dNode *)NULL;
    Material21.m_parent = (X3dNode *)NULL;
    Material21.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material21.diffuseColor = m_diffuseColor;
    Material21.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material21.emissiveColor = m_emissiveColor;
    Material21.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material21.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material21.specularColor = m_specularColor;
    Material21.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material21.transparency = 0;
    ImageTexture5.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture5.url = m_url;
    ImageTexture5.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture5.repeatS = true;
    ImageTexture5.repeatT = true;
    Appearance23.material = &Material21;
    Appearance23.texture = &ImageTexture5;
    Appearance23.textureTransform = NULL;
    IndexedFaceSet21.setGlName(20);
    IndexedFaceSet21.m_parent = (X3dNode *)NULL;
    Coordinate35.m_parent = (X3dNode *)&IndexedFaceSet21;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate35.point = m_point;
    Coordinate35.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal39.m_parent = (X3dNode *)&IndexedFaceSet21;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal39.vector = m_vector;
    Normal39.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate17.m_parent = (X3dNode *)&IndexedFaceSet21;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate17.point = m_point;
    TextureCoordinate17.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet21.color = NULL;
    IndexedFaceSet21.coord = &Coordinate35;
    IndexedFaceSet21.normal = &Normal39;
    IndexedFaceSet21.texCoord = &TextureCoordinate17;
    IndexedFaceSet21.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet21.colorIndex = m_colorIndex;
    IndexedFaceSet21.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet21.colorPerVertex = false;
    IndexedFaceSet21.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet21.coordIndex = m_coordIndex;
    IndexedFaceSet21.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet21.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet21.normalIndex = m_normalIndex;
    IndexedFaceSet21.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet21.normalPerVertex = true;
    IndexedFaceSet21.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet21.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet21.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet21.set_colorIndex = NULL;
    IndexedFaceSet21.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet21.set_coordIndex = NULL;
    IndexedFaceSet21.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet21.set_normalIndex = NULL;
    IndexedFaceSet21.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet21.set_texCoordIndex = NULL;
    IndexedFaceSet21.set_texCoordIndex_length = 0;
    }
    Shape23.appearance = &Appearance23;
    Shape23.geometry = &IndexedFaceSet21;
    {
    static X3dNode* m_children[] = { NULL, &Shape22, NULL, &Shape23 };
    Group11.children = m_children;
    Group11.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group11.bboxCenter = m_bboxCenter;
    Group11.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group11.bboxSize = m_bboxSize;
    Group11.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group11.addChildren = NULL;
    Group11.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group11.removeChildren = NULL;
    Group11.removeChildren_length = 0;
    }
    WorldInfo8.m_parent = (X3dNode *)&Inline12;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo8.info = m_info;
    WorldInfo8.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo8.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group11, NULL, &WorldInfo8 };
    Inline12.children = m_children;
    Inline12.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline12.bboxCenter = m_bboxCenter;
    Inline12.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline12.bboxSize = m_bboxSize;
    Inline12.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline12.addChildren = NULL;
    Inline12.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline12.removeChildren = NULL;
    Inline12.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX4m_MOVE.rotation = m_rotation;
    BOX4m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX4m_MOVE.translation = m_translation;
    BOX4m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX4m_MOVE.bboxCenter = m_bboxCenter;
    BOX4m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX4m_MOVE.bboxSize = m_bboxSize;
    BOX4m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX4m_MOVE.center = m_center;
    BOX4m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX4m_MOVE.scale = m_scale;
    BOX4m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX4m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX4m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX4m_SENSOR, &Inline12 };
    BOX4m_MOVE.children = m_children;
    BOX4m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX4m_MOVE.addChildren = NULL;
    BOX4m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX4m_MOVE.removeChildren = NULL;
    BOX4m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX4m_MOVE };
    BOX4m_SWITCH.children = m_children;
    BOX4m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX4m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX4m_HIDE.rotation = m_rotation;
    BOX4m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX4m_HIDE.translation = m_translation;
    BOX4m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX4m_HIDE.bboxCenter = m_bboxCenter;
    BOX4m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX4m_HIDE.bboxSize = m_bboxSize;
    BOX4m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX4m_HIDE.center = m_center;
    BOX4m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX4m_HIDE.scale = m_scale;
    BOX4m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX4m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX4m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX4m_SWITCH };
    BOX4m_HIDE.children = m_children;
    BOX4m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX4m_HIDE.addChildren = NULL;
    BOX4m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX4m_HIDE.removeChildren = NULL;
    BOX4m_HIDE.removeChildren_length = 0;
    }
    BOX4m_HITm_CHECK.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function initialize()  \
         {  \
         choice_out=0;  \
         deleted_out=0;  \
         }  \
      function translation_in(val)    \
         {  \
         if (!deleted)  \
            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)  \
                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)  \
                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)  \
               )  \
               {  \
               choice_out=1;  \
               deleted_out=1;  \
               deleted=1;  \
               }  \
            else  \
               choice_out=0;  \
         else  \
            choice_out=1;  \
         }  \
           \
      function box_position(val)  \
         {  \
         box[0]=val[0];  \
         box[1]=val[1];  \
         box[2]=val[2];  \
         }    \
      ", "c++:  \
   X3dScript_BOX4_HIT_CHECK *self = (X3dScript_BOX4_HIT_CHECK *)node;  \
   static bool BOX4_HIT_CHECK_init = false;  \
   if (!BOX4_HIT_CHECK_init) {  \
       self->choice_out = 0;  \
       self->deleted_out = 0;  \
       BOX4_HIT_CHECK_init = true;  \
   } else if (!self->deleted)   \
      {  \
      self->box[0] = self->box_position[0];  \
      self->box[1] = self->box_position[1];  \
      self->box[2] = self->box_position[2];  \
      if (  \
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) &&   \
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) &&   \
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) &&   \
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) &&   \
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) &&   \
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)  \
         )  \
         {  \
         self->choice_out = 1;  \
         self->deleted_out = 1;  \
         self->deleted = 1;  \
         }  \
      else  \
         self->choice_out = 0;  \
   }  \
   else  \
      self->choice_out = 1;  \
" };
    BOX4m_HITm_CHECK.url = m_url;
    BOX4m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_HITm_CHECK.directOutput = false;
    BOX4m_HITm_CHECK.mustEvaluate = false;
    BOX4m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX4m_HITm_CHECK.box = m_box;
    BOX4m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX4m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX4m_HITm_CHECK.translation_in = m_translation_in;
    BOX4m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX4m_HITm_CHECK.box_position = m_box_position;
    BOX4m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX4m_HITm_CHECK.deleted_out = 0;
    BOX4m_HITm_CHECK.choice_out = 0;
    BOX4m_MOVEm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function nextsteps()  \
         {  \
         step_translation[0]=(Math.random()-0.5)*0.04;  \
         step_translation[1]=(Math.random()-0.5)*0.04;  \
         step_translation[2]=(Math.random()-0.5)*0.04;  \
         step=Math.random()*200+1;  \
         }  \
      function initialize()  \
         {  \
         translation_out[0]=Math.random()*8-4;  \
         translation_out[1]=Math.random()*6-3;  \
         translation_out[2]=Math.random()*-30-1;  \
         nextsteps();  \
         }  \
      function translation_in(val)    \
         {  \
         step--;  \
         if (step<=0)  \
            nextsteps();  \
         if (   (translation_out[0]+step_translation[0]<4)   \
             && (translation_out[0]+step_translation[0]>-4))   \
            translation_out[0]+=step_translation[0];  \
         if (   (translation_out[1]+step_translation[1]<3)   \
             && (translation_out[1]+step_translation[1]>-3))   \
             translation_out[1]+=step_translation[1];  \
         if (   (translation_out[2]+step_translation[2]<-1)   \
             && (translation_out[2]+step_translation[2]>-30))   \
            translation_out[2]+=step_translation[2];  \
         }  \
      ", "c++:  \
   X3dScript_BOX4_MOVE_ON *self = (X3dScript_BOX4_MOVE_ON *)node;  \
   static bool BOX4_MOVE_ON_init = false;  \
   if (!BOX4_MOVE_ON_init) {  \
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;  \
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;  \
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;  \
       BOX4_MOVE_ON_init = true;  \
   }           \
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   if ((self->translation_out[0] + self->step_translation[0] < 4) &&   \
       (self->translation_out[0] + self->step_translation[0] > -4))   \
       self->translation_out[0] += self->step_translation[0];  \
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&  \
       (self->translation_out[1] + self->step_translation[1] > -3))   \
       self->translation_out[1] += self->step_translation[1];  \
   if ((self->translation_out[2] + self->step_translation[2] < -1) &&   \
       (self->translation_out[2] + self->step_translation[2] > -30))   \
       self->translation_out[2] += self->step_translation[2];  \
" };
    BOX4m_MOVEm_ON.url = m_url;
    BOX4m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_MOVEm_ON.directOutput = false;
    BOX4m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.step_translation = m_step_translation;
    BOX4m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX4m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.translation_in = m_translation_in;
    BOX4m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.translation_out = m_translation_out;
    BOX4m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX4m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=100;  \
         }  \
      else  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=0;  \
         }  \
      }      \
   ", "c++:  \
   X3dScript_BOX4_HIDE_SCRIPT *self = (X3dScript_BOX4_HIDE_SCRIPT *)node;  \
   if (self->isover_in) {               \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 100;  \
   } else {  \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 0;  \
   }  \
" };
    BOX4m_HIDEm_SCRIPT.url = m_url;
    BOX4m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_HIDEm_SCRIPT.directOutput = false;
    BOX4m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX4m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX4m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX4m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX5m_HIDE.route_sources =  NULL;
    BOX5m_HIDE.num_route_source = 0;
    BOX5m_HIDE.m_parent = (X3dNode *)&Group1;
    BOX5m_SWITCH.m_parent = (X3dNode *)&BOX5m_HIDE;
    BOX5m_MOVE.route_sources =  NULL;
    BOX5m_MOVE.num_route_source = 0;
    BOX5m_MOVE.m_parent = (X3dNode *)&BOX5m_SWITCH;
    BOX5m_SENSOR.m_parent = (X3dNode *)&BOX5m_MOVE;
    BOX5m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX5m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX5m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX5m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX5m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX5m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX5m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX5m_SENSOR.isActive = false;
    BOX5m_SENSOR.isOver = false;
    BOX5m_SENSOR.touchTime = 0;
    Inline13.m_parent = (X3dNode *)&BOX5m_MOVE;
    Group12.m_parent = (X3dNode *)&Inline13;
    Shape24.m_parent = (X3dNode *)NULL;
    Appearance24.m_parent = (X3dNode *)NULL;
    Material22.m_parent = (X3dNode *)NULL;
    Material22.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material22.diffuseColor = m_diffuseColor;
    Material22.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material22.emissiveColor = m_emissiveColor;
    Material22.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material22.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material22.specularColor = m_specularColor;
    Material22.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material22.transparency = 0;
    Appearance24.material = &Material22;
    Appearance24.texture = NULL;
    Appearance24.textureTransform = NULL;
    IndexedFaceSet22.setGlName(21);
    IndexedFaceSet22.m_parent = (X3dNode *)NULL;
    Color15.m_parent = (X3dNode *)&IndexedFaceSet22;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color15.color = m_color;
    Color15.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate37.m_parent = (X3dNode *)&IndexedFaceSet22;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate37.point = m_point;
    Coordinate37.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal40.m_parent = (X3dNode *)&IndexedFaceSet22;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal40.vector = m_vector;
    Normal40.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet22.color = &Color15;
    IndexedFaceSet22.coord = &Coordinate37;
    IndexedFaceSet22.normal = &Normal40;
    IndexedFaceSet22.texCoord = NULL;
    IndexedFaceSet22.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 32635470, -1, 0, 0, 0, -1, 0, 6, 0, -1, 0, 0, 0, -1, 8, 11, 7, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet22.colorIndex = m_colorIndex;
    IndexedFaceSet22.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet22.colorPerVertex = false;
    IndexedFaceSet22.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet22.coordIndex = m_coordIndex;
    IndexedFaceSet22.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet22.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet22.normalIndex = m_normalIndex;
    IndexedFaceSet22.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet22.normalPerVertex = true;
    IndexedFaceSet22.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet22.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet22.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet22.set_colorIndex = NULL;
    IndexedFaceSet22.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet22.set_coordIndex = NULL;
    IndexedFaceSet22.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet22.set_normalIndex = NULL;
    IndexedFaceSet22.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet22.set_texCoordIndex = NULL;
    IndexedFaceSet22.set_texCoordIndex_length = 0;
    }
    Shape24.appearance = &Appearance24;
    Shape24.geometry = &IndexedFaceSet22;
    Shape25.m_parent = (X3dNode *)NULL;
    Appearance25.m_parent = (X3dNode *)NULL;
    Material23.m_parent = (X3dNode *)NULL;
    Material23.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material23.diffuseColor = m_diffuseColor;
    Material23.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material23.emissiveColor = m_emissiveColor;
    Material23.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material23.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material23.specularColor = m_specularColor;
    Material23.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material23.transparency = 0;
    ImageTexture6.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture6.url = m_url;
    ImageTexture6.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture6.repeatS = true;
    ImageTexture6.repeatT = true;
    Appearance25.material = &Material23;
    Appearance25.texture = &ImageTexture6;
    Appearance25.textureTransform = NULL;
    IndexedFaceSet23.setGlName(22);
    IndexedFaceSet23.m_parent = (X3dNode *)NULL;
    Coordinate39.m_parent = (X3dNode *)&IndexedFaceSet23;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate39.point = m_point;
    Coordinate39.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal41.m_parent = (X3dNode *)&IndexedFaceSet23;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal41.vector = m_vector;
    Normal41.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate19.m_parent = (X3dNode *)&IndexedFaceSet23;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate19.point = m_point;
    TextureCoordinate19.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet23.color = NULL;
    IndexedFaceSet23.coord = &Coordinate39;
    IndexedFaceSet23.normal = &Normal41;
    IndexedFaceSet23.texCoord = &TextureCoordinate19;
    IndexedFaceSet23.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet23.colorIndex = m_colorIndex;
    IndexedFaceSet23.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet23.colorPerVertex = false;
    IndexedFaceSet23.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet23.coordIndex = m_coordIndex;
    IndexedFaceSet23.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet23.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet23.normalIndex = m_normalIndex;
    IndexedFaceSet23.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet23.normalPerVertex = true;
    IndexedFaceSet23.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet23.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet23.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet23.set_colorIndex = NULL;
    IndexedFaceSet23.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet23.set_coordIndex = NULL;
    IndexedFaceSet23.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet23.set_normalIndex = NULL;
    IndexedFaceSet23.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet23.set_texCoordIndex = NULL;
    IndexedFaceSet23.set_texCoordIndex_length = 0;
    }
    Shape25.appearance = &Appearance25;
    Shape25.geometry = &IndexedFaceSet23;
    {
    static X3dNode* m_children[] = { NULL, &Shape24, NULL, &Shape25 };
    Group12.children = m_children;
    Group12.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group12.bboxCenter = m_bboxCenter;
    Group12.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group12.bboxSize = m_bboxSize;
    Group12.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group12.addChildren = NULL;
    Group12.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group12.removeChildren = NULL;
    Group12.removeChildren_length = 0;
    }
    WorldInfo9.m_parent = (X3dNode *)&Inline13;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo9.info = m_info;
    WorldInfo9.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo9.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group12, NULL, &WorldInfo9 };
    Inline13.children = m_children;
    Inline13.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline13.bboxCenter = m_bboxCenter;
    Inline13.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline13.bboxSize = m_bboxSize;
    Inline13.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline13.addChildren = NULL;
    Inline13.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline13.removeChildren = NULL;
    Inline13.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX5m_MOVE.rotation = m_rotation;
    BOX5m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX5m_MOVE.translation = m_translation;
    BOX5m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX5m_MOVE.bboxCenter = m_bboxCenter;
    BOX5m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX5m_MOVE.bboxSize = m_bboxSize;
    BOX5m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX5m_MOVE.center = m_center;
    BOX5m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX5m_MOVE.scale = m_scale;
    BOX5m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX5m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX5m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX5m_SENSOR, &Inline13 };
    BOX5m_MOVE.children = m_children;
    BOX5m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX5m_MOVE.addChildren = NULL;
    BOX5m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX5m_MOVE.removeChildren = NULL;
    BOX5m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX5m_MOVE };
    BOX5m_SWITCH.children = m_children;
    BOX5m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX5m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX5m_HIDE.rotation = m_rotation;
    BOX5m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX5m_HIDE.translation = m_translation;
    BOX5m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX5m_HIDE.bboxCenter = m_bboxCenter;
    BOX5m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX5m_HIDE.bboxSize = m_bboxSize;
    BOX5m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX5m_HIDE.center = m_center;
    BOX5m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX5m_HIDE.scale = m_scale;
    BOX5m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX5m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX5m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX5m_SWITCH };
    BOX5m_HIDE.children = m_children;
    BOX5m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX5m_HIDE.addChildren = NULL;
    BOX5m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX5m_HIDE.removeChildren = NULL;
    BOX5m_HIDE.removeChildren_length = 0;
    }
    BOX5m_HITm_CHECK.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function initialize()  \
         {  \
         choice_out=0;  \
         deleted_out=0;  \
         }  \
      function translation_in(val)    \
         {  \
         if (!deleted)  \
            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)  \
                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)  \
                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)  \
               )  \
               {  \
               choice_out=1;  \
               deleted_out=1;  \
               deleted=1;  \
               }  \
            else  \
               choice_out=0;  \
         else  \
            choice_out=1;  \
         }  \
           \
      function box_position(val)  \
         {  \
         box[0]=val[0];  \
         box[1]=val[1];  \
         box[2]=val[2];  \
         }    \
      ", "c++:  \
   X3dScript_BOX5_HIT_CHECK *self = (X3dScript_BOX5_HIT_CHECK *)node;  \
   static bool BOX5_HIT_CHECK_init = false;  \
   if (!BOX5_HIT_CHECK_init) {  \
       self->choice_out = 0;  \
       self->deleted_out = 0;  \
       BOX5_HIT_CHECK_init = true;  \
   } else if (!self->deleted)   \
      {  \
      self->box[0] = self->box_position[0];  \
      self->box[1] = self->box_position[1];  \
      self->box[2] = self->box_position[2];  \
      if (  \
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) &&   \
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) &&   \
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) &&   \
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) &&   \
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) &&   \
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)  \
         )  \
         {  \
         self->choice_out = 1;  \
         self->deleted_out = 1;  \
         self->deleted = 1;  \
         }  \
      else  \
         self->choice_out = 0;  \
   }  \
   else  \
      self->choice_out = 1;  \
" };
    BOX5m_HITm_CHECK.url = m_url;
    BOX5m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_HITm_CHECK.directOutput = false;
    BOX5m_HITm_CHECK.mustEvaluate = false;
    BOX5m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX5m_HITm_CHECK.box = m_box;
    BOX5m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX5m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX5m_HITm_CHECK.translation_in = m_translation_in;
    BOX5m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX5m_HITm_CHECK.box_position = m_box_position;
    BOX5m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX5m_HITm_CHECK.deleted_out = 0;
    BOX5m_HITm_CHECK.choice_out = 0;
    BOX5m_MOVEm_ON.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
      function nextsteps()  \
         {  \
         step_translation[0]=(Math.random()-0.5)*0.04;  \
         step_translation[1]=(Math.random()-0.5)*0.04;  \
         step_translation[2]=(Math.random()-0.5)*0.04;  \
         step=Math.random()*200+1;  \
         }  \
      function initialize()  \
         {  \
         translation_out[0]=Math.random()*8-4;  \
         translation_out[1]=Math.random()*6-3;  \
         translation_out[2]=Math.random()*-30-1;  \
         nextsteps();  \
         }  \
      function translation_in(val)    \
         {  \
         step--;  \
         if (step<=0)  \
            nextsteps();  \
         if (   (translation_out[0]+step_translation[0]<4)   \
             && (translation_out[0]+step_translation[0]>-4))   \
            translation_out[0]+=step_translation[0];  \
         if (   (translation_out[1]+step_translation[1]<3)   \
             && (translation_out[1]+step_translation[1]>-3))   \
             translation_out[1]+=step_translation[1];  \
         if (   (translation_out[2]+step_translation[2]<-1)   \
             && (translation_out[2]+step_translation[2]>-30))   \
            translation_out[2]+=step_translation[2];  \
         }  \
      ", "c++:  \
   X3dScript_BOX5_MOVE_ON *self = (X3dScript_BOX5_MOVE_ON *)node;  \
   static bool BOX5_MOVE_ON_init = false;  \
   if (!BOX5_MOVE_ON_init) {  \
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;  \
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;  \
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;  \
       BOX5_MOVE_ON_init = true;  \
   }           \
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;  \
   if ((self->translation_out[0] + self->step_translation[0] < 4) &&   \
       (self->translation_out[0] + self->step_translation[0] > -4))   \
       self->translation_out[0] += self->step_translation[0];  \
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&  \
       (self->translation_out[1] + self->step_translation[1] > -3))   \
       self->translation_out[1] += self->step_translation[1];  \
   if ((self->translation_out[2] + self->step_translation[2] < -1) &&   \
       (self->translation_out[2] + self->step_translation[2] > -30))   \
       self->translation_out[2] += self->step_translation[2];  \
" };
    BOX5m_MOVEm_ON.url = m_url;
    BOX5m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_MOVEm_ON.directOutput = false;
    BOX5m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.step_translation = m_step_translation;
    BOX5m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX5m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.translation_in = m_translation_in;
    BOX5m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.translation_out = m_translation_out;
    BOX5m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX5m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group1;
    {
    static const char* m_url[] = { "javascript:  \
   function isover_in(val)              \
      {              \
      if (val)  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=100;  \
         }  \
      else  \
         {  \
         hide_out[0]=0;  \
         hide_out[1]=0;  \
         hide_out[2]=0;  \
         }  \
      }      \
   ", "c++:  \
   X3dScript_BOX5_HIDE_SCRIPT *self = (X3dScript_BOX5_HIDE_SCRIPT *)node;  \
   if (self->isover_in) {               \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 100;  \
   } else {  \
       self->hide_out[0] = 0;  \
       self->hide_out[1] = 0;  \
       self->hide_out[2] = 0;  \
   }  \
" };
    BOX5m_HIDEm_SCRIPT.url = m_url;
    BOX5m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_HIDEm_SCRIPT.directOutput = false;
    BOX5m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX5m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX5m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX5m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_HIDE, &BOX1m_HITm_CHECK, &BOX1m_MOVEm_ON, &BOX1m_HIDEm_SCRIPT, &STEER, &WALLm_SENSOR, &WALLm_SENSOR2, &ROTm_TRANS1, &ROTm_TRANS2, &Background1, &WALLm_FIRE, &THING, &BULLETm_SWITCH, &TIMER, &SOUND, &NavigationInfo1, &ROLLm_TURN, &UPm_DOWNm_TURN, &FLYm_FORWARD, &FLYm_ON, &THINGm_SCRIPT, &Viewpoint1, &Viewpoint2, &WorldInfo5, &Transform18, &Group8, &Shape17, &BOX2m_HIDE, &BOX2m_HITm_CHECK, &BOX2m_MOVEm_ON, &BOX2m_HIDEm_SCRIPT, &BOX3m_HIDE, &BOX3m_HITm_CHECK, &BOX3m_MOVEm_ON, &BOX3m_HIDEm_SCRIPT, &BOX4m_HIDE, &BOX4m_HITm_CHECK, &BOX4m_MOVEm_ON, &BOX4m_HIDEm_SCRIPT, &BOX5m_HIDE, &BOX5m_HITm_CHECK, &BOX5m_MOVEm_ON, &BOX5m_HIDEm_SCRIPT };
    Group1.children = m_children;
    Group1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group1.bboxCenter = m_bboxCenter;
    Group1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group1.bboxSize = m_bboxSize;
    Group1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group1.addChildren = NULL;
    Group1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group1.removeChildren = NULL;
    Group1.removeChildren_length = 0;
    }
    Transform20.route_sources =  NULL;
    Transform20.num_route_source = 0;
    Transform20.m_parent = (X3dNode *)&Switch2;
    Inline19.m_parent = (X3dNode *)&Transform20;
    Background4.m_parent = (X3dNode *)&Inline19;
    {
    static float m_groundAngle[1];
    Background4.groundAngle = NULL;
    Background4.groundAngle_length = 0;
    }
    {
    static float m_groundColor[1];
    Background4.groundColor = NULL;
    Background4.groundColor_length = 0;
    }
    {
    static const char* m_backUrl[1];
    Background4.backUrl = NULL;
    Background4.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background4.rightUrl = NULL;
    Background4.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background4.frontUrl = NULL;
    Background4.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background4.leftUrl = NULL;
    Background4.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background4.topUrl = NULL;
    Background4.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background4.bottomUrl = NULL;
    Background4.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background4.skyAngle = NULL;
    Background4.skyAngle_length = 0;
    }
    Background4.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background4.skyColor = m_skyColor;
    Background4.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background4.set_bind = false;
    Background4.isBound = false;
    Anchor1.m_parent = (X3dNode *)&Inline19;
    Billboard1.m_parent = (X3dNode *)NULL;
    Shape26.m_parent = (X3dNode *)NULL;
    Appearance26.m_parent = (X3dNode *)NULL;
    Material24.m_parent = (X3dNode *)NULL;
    Material24.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0, 0, 1 };
    Material24.diffuseColor = m_diffuseColor;
    Material24.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material24.emissiveColor = m_emissiveColor;
    Material24.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material24.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material24.specularColor = m_specularColor;
    Material24.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material24.transparency = 0;
    Appearance26.material = &Material24;
    Appearance26.texture = NULL;
    Appearance26.textureTransform = NULL;
    IndexedFaceSet24.setGlName(23);
    IndexedFaceSet24.m_parent = (X3dNode *)NULL;
    COORDm_6.m_parent = (X3dNode *)&IndexedFaceSet24;
    {
    static float m_point[] = { -600, -600, -35, 600, -600, -35, -600, 600, -35, 600, 600, -35 };
    COORDm_6.point = m_point;
    COORDm_6.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal42.m_parent = (X3dNode *)&IndexedFaceSet24;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal42.vector = m_vector;
    Normal42.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet24.color = NULL;
    IndexedFaceSet24.coord = &COORDm_6;
    IndexedFaceSet24.normal = &Normal42;
    IndexedFaceSet24.texCoord = NULL;
    IndexedFaceSet24.ccw = true;
    {
    static int m_colorIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet24.colorIndex = m_colorIndex;
    IndexedFaceSet24.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet24.colorPerVertex = true;
    IndexedFaceSet24.convex = true;
    {
    static int m_coordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet24.coordIndex = m_coordIndex;
    IndexedFaceSet24.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet24.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet24.normalIndex = m_normalIndex;
    IndexedFaceSet24.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet24.normalPerVertex = true;
    IndexedFaceSet24.solid = true;
    {
    static int m_texCoordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet24.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet24.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet24.set_colorIndex = NULL;
    IndexedFaceSet24.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet24.set_coordIndex = NULL;
    IndexedFaceSet24.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet24.set_normalIndex = NULL;
    IndexedFaceSet24.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet24.set_texCoordIndex = NULL;
    IndexedFaceSet24.set_texCoordIndex_length = 0;
    }
    Shape26.appearance = &Appearance26;
    Shape26.geometry = &IndexedFaceSet24;
    {
    static float m_axisOfRotation[] = { 0, 1, 0 };
    Billboard1.axisOfRotation = m_axisOfRotation;
    Billboard1.axisOfRotation_length = sizeof(m_axisOfRotation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape26 };
    Billboard1.children = m_children;
    Billboard1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Billboard1.bboxCenter = m_bboxCenter;
    Billboard1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Billboard1.bboxSize = m_bboxSize;
    Billboard1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Billboard1.addChildren = NULL;
    Billboard1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Billboard1.removeChildren = NULL;
    Billboard1.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Billboard1 };
    Anchor1.children = m_children;
    Anchor1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Anchor1.bboxCenter = m_bboxCenter;
    Anchor1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Anchor1.bboxSize = m_bboxSize;
    Anchor1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    Anchor1.description = "restart game";
    {
    static const char* m_parameter[1];
    Anchor1.parameter = NULL;
    Anchor1.parameter_length = 0;
    }
    {
    static const char* m_url[] = { "warbird.wrl" };
    Anchor1.url = m_url;
    Anchor1.url_length = sizeof(m_url) / sizeof(const char*);
    }
    {
    static X3dNode* m_addChildren[1];
    Anchor1.addChildren = NULL;
    Anchor1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Anchor1.removeChildren = NULL;
    Anchor1.removeChildren_length = 0;
    }
    Anchor2.m_parent = (X3dNode *)&Inline19;
    SCHRIFTm_6.route_sources =  NULL;
    SCHRIFTm_6.num_route_source = 0;
    SCHRIFTm_6.m_parent = (X3dNode *)NULL;
    Transform21.route_sources =  NULL;
    Transform21.num_route_source = 0;
    Transform21.m_parent = (X3dNode *)NULL;
    Inline18.m_parent = (X3dNode *)NULL;
    Transform22.route_sources =  NULL;
    Transform22.num_route_source = 0;
    Transform22.m_parent = (X3dNode *)&Inline18;
    Inline14.m_parent = (X3dNode *)NULL;
    Group13.m_parent = (X3dNode *)&Inline14;
    Transform23.route_sources =  NULL;
    Transform23.num_route_source = 0;
    Transform23.m_parent = (X3dNode *)NULL;
    Shape27.m_parent = (X3dNode *)NULL;
    Appearance27.m_parent = (X3dNode *)NULL;
    A0m_3.m_parent = (X3dNode *)NULL;
    A0m_3.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_3.diffuseColor = m_diffuseColor;
    A0m_3.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_3.emissiveColor = m_emissiveColor;
    A0m_3.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_3.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_3.specularColor = m_specularColor;
    A0m_3.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_3.transparency = 0;
    Appearance27.material = &A0m_3;
    Appearance27.texture = NULL;
    Appearance27.textureTransform = NULL;
    IndexedFaceSet25.setGlName(24);
    IndexedFaceSet25.m_parent = (X3dNode *)NULL;
    Color17.m_parent = (X3dNode *)&IndexedFaceSet25;
    {
    static float m_color[] = { 1, 0.793614, 0.087319, 1, 0.638174, 0.077397, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.754465, 0.084820, 1, 0.754465, 0.084820, 1, 0.775771, 0.086180, 1, 0.756482, 0.084949, 1, 0.756482, 0.084949, 1, 0.777047, 0.086262 };
    Color17.color = m_color;
    Color17.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate41.m_parent = (X3dNode *)&IndexedFaceSet25;
    {
    static float m_point[] = { 0.150260, 0.330511, 1.553120, 0.256526, 0.628258, 1.205410, 0.043506, 0.580147, 1.404830, 0.041913, 0.281121, 1.582420, -0.098818, 0.305979, 1.539120, -0.172704, 0.627974, 1.205410, -0.237476, 0.530979, 1.149410, -0.066433, 0.330228, 1.553120, 0.043506, 0.580146, 1.404830, 0.041913, 0.257624, 1.511120, 0.041913, 0.257624, 1.511120, 0.150260, 0.282014, 1.525120, 0.321306, 0.531263, 1.149410, 0.041913, 0.281121, 1.582420, -0.066433, 0.281730, 1.525120, 0.182646, 0.306263, 1.539120, 0.090713, 0.547383, 1.392800, 0.248053, 0.604517, 1.233140, 0.248850, 0.594803, 1.240080, 0.054951, 0.553382, 1.420730, 0.026772, 0.542105, 1.427400, -0.006386, 0.549079, 1.391450, -0.160300, 0.593221, 1.246000, -0.165083, 0.594759, 1.239830, 0.231697, 0.398694, 1.194280, 0.202474, 0.389779, 1.198610, 0.041913, 0.328005, 1.091790, -0.121436, 0.391855, 1.193190, 0.041913, 0.328657, 1.087900, -0.146267, 0.396108, 1.200800 };
    Coordinate41.point = m_point;
    Coordinate41.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal11.m_parent = (X3dNode *)&IndexedFaceSet25;
    {
    static float m_vector[] = { 0.240520, -0.178855, 0.954024, 0.241302, -0.186810, 0.952300, 0.240520, -0.178855, 0.954024, 0.240520, -0.178855, 0.954024, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.293132, 0.577824, 0.761704, 0.293132, 0.577824, 0.761704, -0.000257, 0.510630, 0.859801, -0.540597, 0.550255, 0.636376, -0.540597, 0.550255, 0.636376, -0.627971, 0.485889, 0.607918, -0.741730, 0.393026, 0.543477, -0.741730, 0.393026, 0.543477, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, 0.273372, -0.961689, -0.020544, 0.273372, -0.961689, -0.020544, 0.274033, -0.961490, -0.021035, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, -0.237959, -0.155430, 0.958758, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.001166, 0.510636, 0.859796, -0.278860, 0.577795, 0.767066, -0.278860, 0.577795, 0.767066, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.871407, -0.488674, 0.042975, -0.871407, -0.488674, 0.042975, -0.867993, -0.495110, 0.038144, -0.168427, -0.936187, 0.308521, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.947812, -0.246434, 0.202292, 0.949402, -0.233910, 0.209575, 0.949402, -0.233910, 0.209575, 0.947812, -0.246434, 0.202292, 0.539886, 0.550710, 0.636586, 0.539886, 0.550710, 0.636586, 0.765835, 0.369611, 0.526198, 0.765835, 0.369611, 0.526198, 0.765820, 0.369627, 0.526210, 0.741377, 0.393368, 0.543712, 0.741377, 0.393368, 0.543712, 0.502593, 0.598896, 0.623477, 0.502593, 0.598896, 0.623477, -0.489526, 0.601019, 0.631775, -0.489526, 0.601019, 0.631775, 0.578916, -0.803148, -0.140751, 0.578916, -0.803148, -0.140751, 0.344423, -0.915028, -0.209994, 0.344423, -0.915028, -0.209994, 0.439693, -0.885759, -0.148667, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, -0.438743, -0.886217, -0.148742, -0.829634, -0.558259, 0.007340, -0.242173, -0.932158, -0.269134 };
    Normal11.vector = m_vector;
    Normal11.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet25.color = &Color17;
    IndexedFaceSet25.coord = &Coordinate41;
    IndexedFaceSet25.normal = &Normal11;
    IndexedFaceSet25.texCoord = NULL;
    IndexedFaceSet25.ccw = true;
    {
    static int m_colorIndex[] = { 1, 0, 11, 0, -1, 16, 0, 19, 0, 0, -1, 1, 1, 0, -1, 1, 0, 13, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 1, 0, 1, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 7, 0, 0, 9, 2, -1, 5, 0, 6, -1, 0, 0, 8, -1, 0, 3, 8, -1, 10, 0, 4, -1, 15, 0, 12, -1, 0, 14, 17, -1, 0, 18, 0, -1, 0, 23, 0, -1, 22, 0, 1, 25, -1, 1, 26, 24, -1, 1, 27, 29, -1, 0, 0, 21, -1, 28, 1, 0, 20, -1 };
    IndexedFaceSet25.colorIndex = m_colorIndex;
    IndexedFaceSet25.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet25.colorPerVertex = true;
    IndexedFaceSet25.convex = true;
    {
    static int m_coordIndex[] = { 3, 0, 19, 2, -1, 22, 5, 23, 4, 7, -1, 3, 10, 11, -1, 13, 8, 20, 7, -1, 13, 7, 4, -1, 13, 4, 14, -1, 6, 14, 4, -1, 13, 14, 9, -1, 3, 15, 0, -1, 3, 11, 15, -1, 12, 15, 11, -1, 17, 0, 15, 18, 1, -1, 16, 0, 17, -1, 15, 12, 18, -1, 12, 1, 18, -1, 19, 0, 16, -1, 21, 7, 20, -1, 7, 21, 22, -1, 4, 23, 6, -1, 11, 24, 12, -1, 24, 11, 10, 25, -1, 10, 26, 25, -1, 9, 27, 28, -1, 14, 6, 29, -1, 27, 9, 14, 29, -1 };
    IndexedFaceSet25.coordIndex = m_coordIndex;
    IndexedFaceSet25.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet25.creaseAngle = 2.942970;
    {
    static int m_normalIndex[] = { 0, 4, 9, 11, -1, 12, 14, 15, 17, 22, -1, 1, 27, 30, -1, 35, 39, 40, 23, -1, 36, 24, 18, -1, 37, 19, 42, -1, 47, 43, 20, -1, 38, 44, 50, -1, 2, 53, 5, -1, 3, 31, 54, -1, 58, 55, 32, -1, 62, 6, 56, 64, 67, -1, 69, 7, 63, -1, 57, 59, 65, -1, 60, 68, 66, -1, 10, 8, 70, -1, 71, 25, 41, -1, 26, 72, 13, -1, 21, 16, 48, -1, 33, 73, 61, -1, 74, 34, 28, 75, -1, 29, 77, 76, -1, 51, 78, 80, -1, 45, 49, 81, -1, 79, 52, 46, 82, -1 };
    IndexedFaceSet25.normalIndex = m_normalIndex;
    IndexedFaceSet25.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet25.normalPerVertex = true;
    IndexedFaceSet25.solid = true;
    {
    static int m_texCoordIndex[1];
    IndexedFaceSet25.texCoordIndex = NULL;
    IndexedFaceSet25.texCoordIndex_length = 0;
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet25.set_colorIndex = NULL;
    IndexedFaceSet25.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet25.set_coordIndex = NULL;
    IndexedFaceSet25.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet25.set_normalIndex = NULL;
    IndexedFaceSet25.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet25.set_texCoordIndex = NULL;
    IndexedFaceSet25.set_texCoordIndex_length = 0;
    }
    Shape27.appearance = &Appearance27;
    Shape27.geometry = &IndexedFaceSet25;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform23.rotation = m_rotation;
    Transform23.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -2.980230e-8, 5.005030e-3, -3.011850e-2 };
    Transform23.translation = m_translation;
    Transform23.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform23.bboxCenter = m_bboxCenter;
    Transform23.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform23.bboxSize = m_bboxSize;
    Transform23.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform23.center = m_center;
    Transform23.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform23.scale = m_scale;
    Transform23.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform23.scaleOrientation = m_scaleOrientation;
    Transform23.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape27 };
    Transform23.children = m_children;
    Transform23.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform23.addChildren = NULL;
    Transform23.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform23.removeChildren = NULL;
    Transform23.removeChildren_length = 0;
    }
    Transform24.route_sources =  NULL;
    Transform24.num_route_source = 0;
    Transform24.m_parent = (X3dNode *)NULL;
    Shape28.m_parent = (X3dNode *)NULL;
    Appearance28.m_parent = (X3dNode *)NULL;
    A0m_4.m_parent = (X3dNode *)NULL;
    A0m_4.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_4.diffuseColor = m_diffuseColor;
    A0m_4.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_4.emissiveColor = m_emissiveColor;
    A0m_4.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_4.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_4.specularColor = m_specularColor;
    A0m_4.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_4.transparency = 0;
    Appearance28.material = &A0m_4;
    Appearance28.texture = NULL;
    Appearance28.textureTransform = NULL;
    IndexedFaceSet26.setGlName(25);
    IndexedFaceSet26.m_parent = (X3dNode *)NULL;
    Color19.m_parent = (X3dNode *)&IndexedFaceSet26;
    {
    static float m_color[] = { 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794 };
    Color19.color = m_color;
    Color19.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate43.m_parent = (X3dNode *)&IndexedFaceSet26;
    {
    static float m_point[] = { 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180, 0.041613, 0.179757, -1.951840, 0.275296, 0.046906, -1.173840, 0.607257, 0.161629, -1.835830, 0.041613, 0.179757, -1.951840, -0.524042, 0.158129, -1.836490, -0.192141, 0.045295, -1.174140, 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180 };
    Coordinate43.point = m_point;
    Coordinate43.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal12.m_parent = (X3dNode *)&IndexedFaceSet26;
    {
    static float m_vector[] = { -0.958763, 0.012792, 0.283919, -0.181027, 0.455152, 0.871817, -0.110099, 0.301903, 0.946960, -0.110099, 0.301903, 0.946960, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.698330, -0.710853, 0.083806, -0.181027, 0.455152, 0.871817, -0.698330, -0.710853, 0.083806, 0.875466, 0.215616, 0.432514, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, 0.003126, -0.985533, -0.169456, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, 0.003126, -0.985533, -0.169456, -0.896416, 0.017432, 0.442870, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112 };
    Normal12.vector = m_vector;
    Normal12.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet26.color = &Color19;
    IndexedFaceSet26.coord = &Coordinate43;
    IndexedFaceSet26.normal = &Normal12;
    IndexedFaceSet26.texCoord = NULL;
    IndexedFaceSet26.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 1, 1, 0, -1, 0, 1, 1, -1, 2, 3, 2, -1, 5, 4, 4, -1 };
    IndexedFaceSet26.colorIndex = m_colorIndex;
    IndexedFaceSet26.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet26.colorPerVertex = true;
    IndexedFaceSet26.convex = true;
    {
    static int m_coordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet26.coordIndex = m_coordIndex;
    IndexedFaceSet26.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet26.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 2, 4, 7, -1, 2, 7, 0, -1, 3, 10, 12, -1, 3, 12, 5, -1, 11, 13, 8, -1, 11, 8, 1, -1, 17, 19, 22, -1, 17, 22, 15, -1, 25, 27, 20, -1, 25, 20, 18, -1, 26, 28, 23, -1, 26, 23, 16, -1, 14, 9, 6, -1, 29, 24, 21, -1, 30, 31, 32, -1, 33, 34, 35, -1 };
    IndexedFaceSet26.normalIndex = m_normalIndex;
    IndexedFaceSet26.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet26.normalPerVertex = true;
    IndexedFaceSet26.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet26.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet26.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet26.set_colorIndex = NULL;
    IndexedFaceSet26.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet26.set_coordIndex = NULL;
    IndexedFaceSet26.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet26.set_normalIndex = NULL;
    IndexedFaceSet26.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet26.set_texCoordIndex = NULL;
    IndexedFaceSet26.set_texCoordIndex_length = 0;
    }
    Shape28.appearance = &Appearance28;
    Shape28.geometry = &IndexedFaceSet26;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform24.rotation = m_rotation;
    Transform24.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0.179843, -9.580490e-2 };
    Transform24.translation = m_translation;
    Transform24.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform24.bboxCenter = m_bboxCenter;
    Transform24.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform24.bboxSize = m_bboxSize;
    Transform24.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform24.center = m_center;
    Transform24.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform24.scale = m_scale;
    Transform24.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform24.scaleOrientation = m_scaleOrientation;
    Transform24.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape28 };
    Transform24.children = m_children;
    Transform24.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform24.addChildren = NULL;
    Transform24.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform24.removeChildren = NULL;
    Transform24.removeChildren_length = 0;
    }
    Transform25.route_sources =  NULL;
    Transform25.num_route_source = 0;
    Transform25.m_parent = (X3dNode *)NULL;
    Shape29.m_parent = (X3dNode *)NULL;
    Appearance29.m_parent = (X3dNode *)NULL;
    Material25.m_parent = (X3dNode *)NULL;
    Material25.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material25.diffuseColor = m_diffuseColor;
    Material25.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material25.emissiveColor = m_emissiveColor;
    Material25.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material25.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material25.specularColor = m_specularColor;
    Material25.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material25.transparency = 6.382980e-2;
    Appearance29.material = &Material25;
    Appearance29.texture = NULL;
    Appearance29.textureTransform = NULL;
    IndexedFaceSet27.setGlName(26);
    IndexedFaceSet27.m_parent = (X3dNode *)NULL;
    Coordinate45.m_parent = (X3dNode *)&IndexedFaceSet27;
    {
    static float m_point[] = { 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.357975, 0.274623, 0.391110, 0.435755, 0.282992, -1.405890, 0.732023, 0.277202, -1.011950, 0.803608, 0.283498, -0.488598, 0.807795, 0.291706, -0.169135, 0.642283, 0.308596, 0.135106, -0.576839, 0.295092, 0.138182, -0.741419, 0.274624, -0.165225, -0.736057, 0.266519, -0.484712, -0.662406, 0.261798, -1.008430, -0.364349, 0.274033, -1.403870, 0.034173, 1.521740, 0.072271, 0.032379, 1.258290, 0.402776, 0.324052, 1.234020, 0.434518, 0.382959, 1.442650, 0.114314, 0.462972, 1.076210, 0.512842, 0.634192, 1.091710, 0.196483, 0.036047, 1.641430, -0.319814, 0.468123, 1.540110, -0.266019, 0.780864, 1.161780, -0.151701, 0.753362, 0.779925, 0.302416, 0.875071, 0.752977, 0.006324, 0.646152, 0.462049, 0.391531, 0.036992, 1.662050, -0.688097, 0.522545, 1.549060, -0.628109, 0.873437, 1.148710, -0.497152, 0.978519, 0.692729, -0.320894, 0.036779, 1.562450, -1.012960, 0.537744, 1.451000, -0.953720, 0.879328, 1.083370, -0.818803, 0.983199, 0.632646, -0.644566, 0.672590, 0.889258, -1.304530, 0.877021, 0.531470, -1.119920, 0.439314, 0.644779, -1.532240, 0.477913, 0.461302, -1.461330, 0.028006, 0.456359, -1.522590, 0.025463, 0.201297, -1.389500, -0.259292, 1.233740, 0.434518, -0.316266, 1.434790, 0.116078, -0.398215, 1.075930, 0.512842, -0.569430, 1.091430, 0.196483, -0.398142, 1.530440, -0.263833, -0.718144, 1.145240, -0.147918, -0.688602, 0.779642, 0.302416, -0.820289, 0.734310, 0.010603, -0.581395, 0.461765, 0.391532, -0.450917, 1.538220, -0.625652, -0.809539, 1.130170, -0.492904, -0.923472, 0.671821, -0.316093, -0.466519, 1.439820, -0.951186, -0.815262, 1.064720, -0.814525, -0.927883, 0.611639, -0.639742, -0.391565, 1.162260, -1.429880, -0.821517, 0.512768, -1.115630, -0.422165, 0.451259, -1.459060, 0.032379, 1.315590, 1.240890, 0.228842, 1.352260, 0.975027, 0.032379, 1.384890, 0.907439, 0.314142, 1.285900, 1.111910, 0.314142, 1.207010, 1.274610, 0.228842, 1.140650, 1.411480, 0.032379, 1.107740, 1.479070, 0.032379, 1.284680, 0.661614, 0.313972, 1.236840, 0.760583, 0.438872, 1.139660, 0.961005, 0.438872, 0.959292, 1.192490, 0.313972, 0.916734, 1.377380, 0.032379, 0.878853, 1.498640, 0.457846, 0.598657, 1.016000, 0.325246, 0.449644, 1.073590, -0.164085, 1.351980, 0.975027, -0.249383, 1.285620, 1.111910, -0.249383, 1.206730, 1.274610, -0.164085, 1.140370, 1.411480, -0.249218, 1.236560, 0.760583, -0.374117, 1.139380, 0.961005, -0.374117, 0.959003, 1.192490, -0.249218, 0.916450, 1.377380, -0.412717, 0.598373, 1.016000, -0.277239, 0.449360, 1.073590, 0.030998, 0.455945, -1.522540, 0.200163, 0.782040, -2.021660, 0.022803, 0.816558, -2.021660, -0.153194, 0.791256, -2.021660, 0.030997, 0.455945, -1.522540, -0.638224, 0.898115, -1.302900, -0.394524, 0.650450, -1.531170, 0.012304, 1.278610, -1.485710, 0.022803, 0.816558, -2.021660, 0.012304, 1.278610, -1.485710, 0.452747, 1.152960, -1.430970 };
    Coordinate45.point = m_point;
    Coordinate45.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal13.m_parent = (X3dNode *)&IndexedFaceSet27;
    {
    static float m_vector[] = { -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940694, 0.149539, 0.304520, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.834986, -0.422306, 0.352783, 0.834986, -0.422306, 0.352783, 0.837532, -0.417196, 0.352827, 0.834986, -0.422306, 0.352783, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824726, 0.564327, 0.036905, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983866, 0.093656, 0.152433, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.919028, -0.327107, 0.219973, 0.919028, -0.327107, 0.219973, 0.919030, -0.327639, 0.219171, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857347, 0.447616, -0.254158, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.871918, -0.485933, -0.060240, 0.871918, -0.485933, -0.060240, 0.871819, -0.486088, -0.060419, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.748933, -0.523293, -0.406527, 0.748933, -0.523293, -0.406527, 0.748998, -0.522697, -0.407174, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.687379, -0.282285, -0.669197, 0.682617, -0.274976, -0.677069, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.373882, -0.393399, -0.839911, 0.373882, -0.393399, -0.839911, 0.372714, -0.393753, -0.840264, 0.043708, -0.372748, -0.926903, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.001617, -0.462617, -0.886557, 0.001617, -0.462617, -0.886557, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936815, 0.032489, 0.348312, -0.936814, 0.032493, 0.348315, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.753712, -0.525849, 0.394210, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.858943, 0.498866, 0.115538, -0.859015, 0.498704, 0.115703, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.909057, -0.352626, 0.221969, -0.909016, -0.354059, 0.219844, -0.909016, -0.354059, 0.219844, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.972656, 0.230360, -0.029579, -0.917591, -0.388391, 0.084727, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.926292, 0.088999, -0.366146, -0.914069, 0.304729, -0.267615, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.914069, 0.304729, -0.267615, -0.657343, -0.174205, -0.733180, -0.694040, -0.019618, -0.719669, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.053473, -0.461524, -0.885515, -0.465415, -0.809935, 0.356924, -0.465415, -0.809935, 0.356924, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.301321, -0.764284, 0.570154, 0.512027, -0.788035, 0.341804, 0.512027, -0.788035, 0.341804, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, 0.115620, -0.804235, -0.582956, -0.370219, -0.828569, -0.420014, 0.115620, -0.804235, -0.582956, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.117789, 0.819323, -0.561103, -0.725452, 0.412604, -0.550888, -0.112111, 0.280660, -0.953237, -0.115216, 0.288185, -0.950618, -0.115216, 0.288185, -0.950618, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.694040, -0.019618, -0.719669, 0.113269, -0.848919, -0.516243, 0.113269, -0.848919, -0.516243, 0.109595, 0.280897, -0.953460, 0.110751, 0.283846, -0.952452, 0.109595, 0.280897, -0.953460, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.285260, 0.728679, -0.622619, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.509719, 0.651105, -0.562360, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.046990, 0.842110, -0.537256 };
    Normal13.vector = m_vector;
    Normal13.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet27.color = NULL;
    IndexedFaceSet27.coord = &Coordinate45;
    IndexedFaceSet27.normal = &Normal13;
    IndexedFaceSet27.texCoord = NULL;
    IndexedFaceSet27.ccw = true;
    {
    static int m_colorIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet27.colorIndex = m_colorIndex;
    IndexedFaceSet27.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet27.colorPerVertex = true;
    IndexedFaceSet27.convex = true;
    {
    static int m_coordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet27.coordIndex = m_coordIndex;
    IndexedFaceSet27.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet27.creaseAngle = 3.104420;
    {
    static int m_normalIndex[] = { 0, 6, 12, -1, 13, 18, 1, -1, 19, 14, 24, -1, 25, 30, 20, -1, 36, 2, 21, -1, 22, 42, 37, -1, 43, 23, 31, -1, 32, 48, 44, -1, 49, 33, 54, -1, 55, 60, 50, -1, 61, 56, 66, -1, 67, 72, 62, -1, 76, 38, 45, -1, 46, 82, 77, -1, 83, 47, 51, -1, 52, 88, 84, -1, 89, 53, 63, -1, 64, 94, 90, -1, 95, 65, 73, -1, 74, 100, 96, -1, 103, 78, 85, -1, 86, 109, 104, -1, 110, 87, 91, -1, 92, 115, 111, -1, 116, 93, 97, -1, 98, 121, 117, -1, 122, 99, 101, -1, 102, 127, 123, -1, 130, 118, 124, -1, 125, 136, 131, -1, 137, 126, 128, -1, 129, 142, 138, -1, 145, 132, 139, -1, 140, 150, 146, -1, 151, 141, 143, -1, 144, 155, 152, -1, 158, 147, 153, -1, 159, 154, 156, -1, 160, 157, 164, -1, 3, 166, 7, -1, 167, 4, 172, -1, 173, 178, 168, -1, 179, 174, 184, -1, 39, 175, 5, -1, 176, 40, 190, -1, 191, 185, 177, -1, 186, 192, 196, -1, 197, 202, 187, -1, 203, 198, 208, -1, 209, 214, 204, -1, 215, 210, 220, -1, 79, 193, 41, -1, 194, 80, 224, -1, 225, 199, 195, -1, 200, 226, 230, -1, 231, 211, 201, -1, 212, 232, 236, -1, 237, 221, 213, -1, 222, 238, 242, -1, 105, 227, 81, -1, 228, 106, 245, -1, 246, 233, 229, -1, 234, 247, 251, -1, 252, 239, 235, -1, 240, 253, 257, -1, 258, 243, 241, -1, 244, 259, 263, -1, 266, 254, 248, -1, 272, 264, 260, -1, 265, 273, 278, -1, 281, 279, 274, -1, 280, 282, 286, -1, 161, 287, 283, -1, 162, 165, 288, -1, 223, 289, 216, -1, 291, 217, 290, -1, 75, 68, 297, -1, 299, 309, 314, -1, 300, 318, 310, -1, 301, 323, 319, -1, 302, 328, 324, -1, 303, 333, 329, -1, 339, 315, 311, -1, 312, 345, 340, -1, 346, 313, 320, -1, 321, 351, 347, -1, 352, 322, 325, -1, 326, 357, 353, -1, 358, 327, 330, -1, 331, 363, 359, -1, 364, 332, 334, -1, 335, 369, 365, -1, 8, 341, 348, -1, 349, 15, 9, -1, 16, 350, 354, -1, 355, 26, 17, -1, 27, 356, 360, -1, 361, 375, 28, -1, 376, 362, 366, -1, 367, 381, 377, -1, 382, 368, 370, -1, 371, 292, 383, -1, 34, 29, 378, -1, 379, 57, 35, -1, 58, 380, 384, -1, 385, 69, 59, -1, 70, 386, 293, -1, 304, 316, 387, -1, 305, 388, 392, -1, 306, 393, 397, -1, 307, 398, 402, -1, 308, 403, 336, -1, 342, 389, 317, -1, 390, 343, 407, -1, 408, 394, 391, -1, 395, 409, 413, -1, 414, 399, 396, -1, 400, 415, 419, -1, 420, 404, 401, -1, 405, 421, 425, -1, 426, 337, 406, -1, 338, 427, 372, -1, 10, 410, 344, -1, 411, 11, 169, -1, 170, 416, 412, -1, 417, 171, 180, -1, 181, 422, 418, -1, 423, 182, 431, -1, 432, 428, 424, -1, 429, 433, 437, -1, 438, 373, 430, -1, 374, 439, 294, -1, 188, 434, 183, -1, 435, 189, 205, -1, 206, 440, 436, -1, 441, 207, 218, -1, 219, 295, 442, -1, 71, 296, 298, -1, 443, 445, 450, -1, 453, 446, 459, -1, 444, 460, 447, -1, 464, 466, 469, -1, 467, 474, 475, -1, 476, 470, 468, -1, 112, 119, 477, -1, 120, 133, 478, -1, 471, 479, 134, -1, 255, 267, 454, -1, 261, 256, 455, -1, 275, 262, 456, -1, 107, 113, 481, -1, 249, 108, 482, -1, 268, 250, 483, -1, 114, 480, 484, -1, 451, 448, 269, -1, 449, 457, 270, -1, 485, 452, 271, -1, 284, 276, 461, -1, 163, 285, 462, -1, 277, 458, 463, -1, 465, 472, 148, -1, 473, 135, 149, -1 };
    IndexedFaceSet27.normalIndex = m_normalIndex;
    IndexedFaceSet27.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet27.normalPerVertex = true;
    IndexedFaceSet27.solid = true;
    {
    static int m_texCoordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet27.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet27.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet27.set_colorIndex = NULL;
    IndexedFaceSet27.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet27.set_coordIndex = NULL;
    IndexedFaceSet27.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet27.set_normalIndex = NULL;
    IndexedFaceSet27.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet27.set_texCoordIndex = NULL;
    IndexedFaceSet27.set_texCoordIndex_length = 0;
    }
    Shape29.appearance = &Appearance29;
    Shape29.geometry = &IndexedFaceSet27;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform25.rotation = m_rotation;
    Transform25.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform25.translation = m_translation;
    Transform25.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform25.bboxCenter = m_bboxCenter;
    Transform25.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform25.bboxSize = m_bboxSize;
    Transform25.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform25.center = m_center;
    Transform25.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform25.scale = m_scale;
    Transform25.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform25.scaleOrientation = m_scaleOrientation;
    Transform25.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape29 };
    Transform25.children = m_children;
    Transform25.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform25.addChildren = NULL;
    Transform25.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform25.removeChildren = NULL;
    Transform25.removeChildren_length = 0;
    }
    Transform26.route_sources =  NULL;
    Transform26.num_route_source = 0;
    Transform26.m_parent = (X3dNode *)NULL;
    Shape30.m_parent = (X3dNode *)NULL;
    Appearance30.m_parent = (X3dNode *)NULL;
    Material26.m_parent = (X3dNode *)NULL;
    Material26.ambientIntensity = 1;
    {
    static float m_diffuseColor[] = { 1, 1, 1 };
    Material26.diffuseColor = m_diffuseColor;
    Material26.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0.691489, 0.691489, 0.691489 };
    Material26.emissiveColor = m_emissiveColor;
    Material26.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material26.shininess = 1;
    {
    static float m_specularColor[] = { 0.765957, 0.765957, 0.765957 };
    Material26.specularColor = m_specularColor;
    Material26.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material26.transparency = 0.372340;
    Appearance30.material = &Material26;
    Appearance30.texture = NULL;
    Appearance30.textureTransform = NULL;
    IndexedFaceSet28.setGlName(27);
    IndexedFaceSet28.m_parent = (X3dNode *)NULL;
    Coordinate47.m_parent = (X3dNode *)&IndexedFaceSet28;
    {
    static float m_point[] = { 0.023775, 0.125771, 0.096370, -0.736057, 0.266519, -0.484712, 0.019864, 0.155999, -0.238411, -0.741419, 0.274624, -0.165225, 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.033239, 0.290582, 0.404469, -0.576839, 0.295092, 0.138182, 0.357975, 0.274623, 0.391110, 0.642283, 0.308596, 0.135106, 0.026537, 0.145128, 0.290011, 0.807795, 0.291706, -0.169135, 0.803608, 0.283498, -0.488598, -0.662406, 0.261798, -1.008430, 0.019557, 0.169360, -0.585595, -0.364349, 0.274033, -1.403870, 0.034997, 0.201297, -1.394460, 0.732023, 0.277202, -1.011950, 0.435755, 0.282992, -1.405890 };
    Coordinate47.point = m_point;
    Coordinate47.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal14.m_parent = (X3dNode *)&IndexedFaceSet28;
    {
    static float m_vector[] = { -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.115983, -0.989347, -0.087975, -0.197996, -0.979966, 0.021537, -0.141172, -0.989925, -0.010929, -0.115983, -0.989347, -0.087975, 0.131644, -0.987142, -0.090669, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, 0.127132, -0.991148, -0.038256, 0.157553, -0.987463, -0.009671, -0.228866, -0.968557, 0.097558, -0.229441, -0.968303, 0.098722, -0.229441, -0.968303, 0.098722, 0.003121, -0.997858, 0.065337, 0.003121, -0.997858, 0.065337, 0.057747, -0.997198, -0.047547, 0.057747, -0.997198, -0.047547, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, -0.193939, -0.928966, 0.315294, -0.193939, -0.928966, 0.315294, -0.194333, -0.929016, 0.314902, -0.046282, -0.996765, 0.065709, -0.056059, -0.979739, -0.192273, 0.204185, -0.934007, 0.293153, 0.205250, -0.934031, 0.292331, 0.205250, -0.934031, 0.292331, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.256610, -0.958616, 0.123319, 0.257004, -0.958385, 0.124286, 0.256610, -0.958616, 0.123319, 0.169604, -0.985103, -0.028410, 0.169720, -0.985078, -0.028591, 0.169604, -0.985103, -0.028410, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, -0.178067, -0.983112, -0.042216, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, 0.127132, -0.991148, -0.038256, 0.198666, -0.979446, -0.034880, 0.097470, -0.991352, -0.087874, -0.178067, -0.983112, -0.042216, -0.078733, -0.992819, -0.090062, 0.010487, -0.999174, -0.039251, 0.010487, -0.999174, -0.039251, 0.131753, -0.990507, -0.039208, 0.132081, -0.990482, -0.038739, 0.131753, -0.990507, -0.039208, 0.148313, -0.987028, -0.061478, 0.148313, -0.987028, -0.061478 };
    Normal14.vector = m_vector;
    Normal14.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet28.color = NULL;
    IndexedFaceSet28.coord = &Coordinate47;
    IndexedFaceSet28.normal = &Normal14;
    IndexedFaceSet28.texCoord = NULL;
    IndexedFaceSet28.ccw = true;
    {
    static int m_colorIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet28.colorIndex = m_colorIndex;
    IndexedFaceSet28.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet28.colorPerVertex = true;
    IndexedFaceSet28.convex = true;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet28.coordIndex = m_coordIndex;
    IndexedFaceSet28.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet28.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 0, 6, 9, -1, 7, 1, 15, -1, 18, 20, 22, -1, 28, 23, 21, -1, 19, 24, 31, -1, 33, 32, 25, -1, 36, 16, 2, -1, 26, 37, 34, -1, 27, 29, 38, -1, 17, 39, 30, -1, 40, 3, 42, -1, 43, 35, 41, -1, 4, 10, 45, -1, 46, 44, 5, -1, 48, 11, 8, -1, 51, 57, 59, -1, 12, 49, 52, -1, 58, 53, 50, -1, 13, 54, 61, -1, 62, 47, 14, -1, 55, 60, 64, -1, 65, 63, 56, -1 };
    IndexedFaceSet28.normalIndex = m_normalIndex;
    IndexedFaceSet28.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet28.normalPerVertex = true;
    IndexedFaceSet28.solid = true;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet28.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet28.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet28.set_colorIndex = NULL;
    IndexedFaceSet28.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet28.set_coordIndex = NULL;
    IndexedFaceSet28.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet28.set_normalIndex = NULL;
    IndexedFaceSet28.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet28.set_texCoordIndex = NULL;
    IndexedFaceSet28.set_texCoordIndex_length = 0;
    }
    Shape30.appearance = &Appearance30;
    Shape30.geometry = &IndexedFaceSet28;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform26.rotation = m_rotation;
    Transform26.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform26.translation = m_translation;
    Transform26.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform26.bboxCenter = m_bboxCenter;
    Transform26.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform26.bboxSize = m_bboxSize;
    Transform26.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform26.center = m_center;
    Transform26.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform26.scale = m_scale;
    Transform26.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform26.scaleOrientation = m_scaleOrientation;
    Transform26.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape30 };
    Transform26.children = m_children;
    Transform26.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform26.addChildren = NULL;
    Transform26.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform26.removeChildren = NULL;
    Transform26.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform23, &Transform24, &Transform25, &Transform26 };
    Group13.children = m_children;
    Group13.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group13.bboxCenter = m_bboxCenter;
    Group13.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group13.bboxSize = m_bboxSize;
    Group13.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group13.addChildren = NULL;
    Group13.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group13.removeChildren = NULL;
    Group13.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group13 };
    Inline14.children = m_children;
    Inline14.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline14.bboxCenter = m_bboxCenter;
    Inline14.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline14.bboxSize = m_bboxSize;
    Inline14.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline14.addChildren = NULL;
    Inline14.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline14.removeChildren = NULL;
    Inline14.removeChildren_length = 0;
    }
    Transform27.route_sources =  NULL;
    Transform27.num_route_source = 0;
    Transform27.m_parent = (X3dNode *)NULL;
    Transform28.route_sources =  NULL;
    Transform28.num_route_source = 0;
    Transform28.m_parent = (X3dNode *)NULL;
    Transform29.route_sources =  NULL;
    Transform29.num_route_source = 0;
    Transform29.m_parent = (X3dNode *)NULL;
    Shape31.m_parent = (X3dNode *)NULL;
    Appearance31.m_parent = (X3dNode *)NULL;
    Material27.m_parent = (X3dNode *)NULL;
    Material27.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material27.diffuseColor = m_diffuseColor;
    Material27.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material27.emissiveColor = m_emissiveColor;
    Material27.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material27.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material27.specularColor = m_specularColor;
    Material27.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material27.transparency = 0;
    Appearance31.material = &Material27;
    Appearance31.texture = NULL;
    Appearance31.textureTransform = NULL;
    IndexedFaceSet29.setGlName(28);
    IndexedFaceSet29.m_parent = (X3dNode *)NULL;
    Coordinate49.m_parent = (X3dNode *)&IndexedFaceSet29;
    {
    static float m_point[] = { 0, 0, -1, -0.382683, 0, -0.923880, -0.707107, 0, -0.707107, -0.923880, 0, -0.382683, -1, 0, 0, -0.923880, 0, 0.382684, -0.707107, 0, 0.707107, -0.382683, 0, 0.923880, 0, 0, 1, 0.382683, 0, 0.923880, 0.707107, 0, 0.707107, 0.923880, 0, 0.382683, 1, 0, 0, 0.923879, 0, -0.382684, 0.707107, 0, -0.707107, 0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, -0.363954, 0.309017, -0.878662, -0.672499, 0.309017, -0.672499, -0.878662, 0.309017, -0.363954, -0.951057, 0.309017, 0, -0.878662, 0.309017, 0.363954, -0.672499, 0.309017, 0.672499, -0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, 0.363954, 0.309017, 0.878662, 0.672499, 0.309017, 0.672498, 0.878662, 0.309017, 0.363953, 0.951057, 0.309017, 0, 0.878662, 0.309017, -0.363954, 0.672498, 0.309017, -0.672499, 0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, -0.309597, 0.587785, -0.747434, -0.572061, 0.587785, -0.572061, -0.747434, 0.587785, -0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, 0.309597, -0.572061, 0.587785, 0.572061, -0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, 0.309597, 0.587785, 0.747434, 0.572061, 0.587785, 0.572061, 0.747434, 0.587785, 0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, -0.309598, 0.572061, 0.587785, -0.572062, 0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, -0.224936, 0.809017, -0.543043, -0.415627, 0.809017, -0.415627, -0.543043, 0.809017, -0.224936, -0.587785, 0.809017, 0, -0.543043, 0.809017, 0.224936, -0.415627, 0.809017, 0.415627, -0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, 0.224936, 0.809017, 0.543043, 0.415627, 0.809017, 0.415627, 0.543043, 0.809017, 0.224935, 0.587785, 0.809017, 0, 0.543043, 0.809017, -0.224936, 0.415627, 0.809017, -0.415627, 0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, -0.118256, 0.951057, -0.285494, -0.218508, 0.951057, -0.218508, -0.285494, 0.951057, -0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, 0.118256, -0.218508, 0.951057, 0.218508, -0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, 0.118256, 0.951057, 0.285494, 0.218508, 0.951057, 0.218508, 0.285495, 0.951057, 0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, -0.118256, 0.218508, 0.951057, -0.218508, 0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate49.point = m_point;
    Coordinate49.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal15.m_parent = (X3dNode *)&IndexedFaceSet29;
    {
    static float m_vector[] = { -0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, -0.185864, 0.303889, -0.934401, -0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377972, 0.156433, -0.912505, -0.377972, 0.156433, -0.912505, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.698401, 0.156433, -0.698401, -0.698401, 0.156433, -0.698401, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.912505, 0.156434, -0.377972, -0.912505, 0.156434, -0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987689, 0.156434, 0, -0.987689, 0.156434, 0, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.912505, 0.156434, 0.377972, -0.912505, 0.156434, 0.377972, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.698401, 0.156434, 0.698402, -0.698401, 0.156434, 0.698402, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377971, 0.156434, 0.912505, -0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377973, 0.156433, 0.912505, 0.377973, 0.156433, 0.912505, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.698402, 0.156434, 0.698400, 0.698402, 0.156434, 0.698400, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.912505, 0.156435, 0.377972, 0.912505, 0.156435, 0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987688, 0.156434, 0, 0.987688, 0.156434, 0, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.912505, 0.156433, -0.377973, 0.912505, 0.156433, -0.377973, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.698401, 0.156434, -0.698401, 0.698401, 0.156434, -0.698401, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377973, 0.156435, -0.912505, 0.377973, 0.156435, -0.912505, 0.104474, 0.325528, -0.939743, 0.105800, 0.318418, -0.942028, 0.104474, 0.325528, -0.939743, 0.104474, 0.325528, -0.939743, 0.097141, 0.233062, -0.967598, 0.097141, 0.233062, -0.967598, -0.158828, 0.580683, -0.798486, -0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.090247, 0.600810, -0.794282, 0.091515, 0.594767, -0.798672, 0.090247, 0.600810, -0.794282, 0.090247, 0.600810, -0.794282, -0.116055, 0.803815, -0.583449, -0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.067135, 0.818259, -0.570916, 0.068222, 0.813901, -0.576985, 0.067135, 0.818259, -0.570916, 0.067135, 0.818259, -0.570916, -0.061299, 0.949355, -0.308167, -0.061299, 0.949355, -0.308167, 0, 1, 0, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137550, 0.933532, -0.331056, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253769, 0.933532, -0.253218, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331354, 0.933532, -0.136830, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, 0.000389, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331056, 0.933532, 0.137550, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253218, 0.933532, 0.253769, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.136830, 0.933532, 0.331354, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, 0.000389, 0.933532, 0.358494, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137550, 0.933532, 0.331056, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253768, 0.933532, 0.253219, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331354, 0.933532, 0.136830, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, -0.000391, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331056, 0.933532, -0.137550, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253218, 0.933532, -0.253769, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.136830, 0.933532, -0.331354, 0.042033, 0.942951, -0.330266, 0.042048, 0.942867, -0.330504, 0.042033, 0.942951, -0.330266, 0, 1, 0, -0.031102, 0.987211, -0.156357, -0.088569, 0.987211, -0.132553, -0.132553, 0.987211, -0.088569, -0.156357, 0.987211, -0.031102, -0.156357, 0.987211, 0.031102, -0.132553, 0.987211, 0.088569, -0.088569, 0.987211, 0.132553, -0.031102, 0.987211, 0.156357, 0.031102, 0.987211, 0.156357, 0.088569, 0.987211, 0.132553, 0.132553, 0.987211, 0.088570, 0.156357, 0.987211, 0.031101, 0.156357, 0.987211, -0.031102, 0.132553, 0.987211, -0.088569, 0.088569, 0.987211, -0.132553, 0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal15.vector = m_vector;
    Normal15.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate21.m_parent = (X3dNode *)&IndexedFaceSet29;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate21.point = m_point;
    TextureCoordinate21.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet29.color = NULL;
    IndexedFaceSet29.coord = &Coordinate49;
    IndexedFaceSet29.normal = &Normal15;
    IndexedFaceSet29.texCoord = &TextureCoordinate21;
    IndexedFaceSet29.ccw = false;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet29.colorIndex = m_colorIndex;
    IndexedFaceSet29.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet29.colorPerVertex = true;
    IndexedFaceSet29.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet29.coordIndex = m_coordIndex;
    IndexedFaceSet29.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet29.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet29.normalIndex = m_normalIndex;
    IndexedFaceSet29.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet29.normalPerVertex = true;
    IndexedFaceSet29.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet29.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet29.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet29.set_colorIndex = NULL;
    IndexedFaceSet29.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet29.set_coordIndex = NULL;
    IndexedFaceSet29.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet29.set_normalIndex = NULL;
    IndexedFaceSet29.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet29.set_texCoordIndex = NULL;
    IndexedFaceSet29.set_texCoordIndex_length = 0;
    }
    Shape31.appearance = &Appearance31;
    Shape31.geometry = &IndexedFaceSet29;
    Transform30.route_sources =  NULL;
    Transform30.num_route_source = 0;
    Transform30.m_parent = (X3dNode *)NULL;
    Shape32.m_parent = (X3dNode *)NULL;
    Appearance32.m_parent = (X3dNode *)NULL;
    Material28.m_parent = (X3dNode *)NULL;
    Material28.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material28.diffuseColor = m_diffuseColor;
    Material28.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material28.emissiveColor = m_emissiveColor;
    Material28.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material28.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material28.specularColor = m_specularColor;
    Material28.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material28.transparency = 0;
    Appearance32.material = &Material28;
    Appearance32.texture = NULL;
    Appearance32.textureTransform = NULL;
    IndexedFaceSet30.setGlName(29);
    IndexedFaceSet30.m_parent = (X3dNode *)NULL;
    Coordinate51.m_parent = (X3dNode *)&IndexedFaceSet30;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate51.point = m_point;
    Coordinate51.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal16.m_parent = (X3dNode *)&IndexedFaceSet30;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal16.vector = m_vector;
    Normal16.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate23.m_parent = (X3dNode *)&IndexedFaceSet30;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate23.point = m_point;
    TextureCoordinate23.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet30.color = NULL;
    IndexedFaceSet30.coord = &Coordinate51;
    IndexedFaceSet30.normal = &Normal16;
    IndexedFaceSet30.texCoord = &TextureCoordinate23;
    IndexedFaceSet30.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet30.colorIndex = m_colorIndex;
    IndexedFaceSet30.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet30.colorPerVertex = true;
    IndexedFaceSet30.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet30.coordIndex = m_coordIndex;
    IndexedFaceSet30.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet30.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet30.normalIndex = m_normalIndex;
    IndexedFaceSet30.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet30.normalPerVertex = true;
    IndexedFaceSet30.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet30.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet30.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet30.set_colorIndex = NULL;
    IndexedFaceSet30.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet30.set_coordIndex = NULL;
    IndexedFaceSet30.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet30.set_normalIndex = NULL;
    IndexedFaceSet30.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet30.set_texCoordIndex = NULL;
    IndexedFaceSet30.set_texCoordIndex_length = 0;
    }
    Shape32.appearance = &Appearance32;
    Shape32.geometry = &IndexedFaceSet30;
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 0 };
    Transform30.rotation = m_rotation;
    Transform30.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.220000, 0.440000, 0.160000 };
    Transform30.translation = m_translation;
    Transform30.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform30.bboxCenter = m_bboxCenter;
    Transform30.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform30.bboxSize = m_bboxSize;
    Transform30.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform30.center = m_center;
    Transform30.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform30.scale = m_scale;
    Transform30.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform30.scaleOrientation = m_scaleOrientation;
    Transform30.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape32 };
    Transform30.children = m_children;
    Transform30.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform30.addChildren = NULL;
    Transform30.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform30.removeChildren = NULL;
    Transform30.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 1.578764 };
    Transform29.rotation = m_rotation;
    Transform29.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform29.translation = m_translation;
    Transform29.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform29.bboxCenter = m_bboxCenter;
    Transform29.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform29.bboxSize = m_bboxSize;
    Transform29.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform29.center = m_center;
    Transform29.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform29.scale = m_scale;
    Transform29.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform29.scaleOrientation = m_scaleOrientation;
    Transform29.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape31, &Transform30 };
    Transform29.children = m_children;
    Transform29.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform29.addChildren = NULL;
    Transform29.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform29.removeChildren = NULL;
    Transform29.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.565552, -0.792356, -0.228743, 0.826199 };
    Transform28.rotation = m_rotation;
    Transform28.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.109765, 0.715337, 1.294240 };
    Transform28.translation = m_translation;
    Transform28.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform28.bboxCenter = m_bboxCenter;
    Transform28.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform28.bboxSize = m_bboxSize;
    Transform28.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform28.center = m_center;
    Transform28.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform28.scale = m_scale;
    Transform28.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform28.scaleOrientation = m_scaleOrientation;
    Transform28.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform29 };
    Transform28.children = m_children;
    Transform28.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform28.addChildren = NULL;
    Transform28.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform28.removeChildren = NULL;
    Transform28.removeChildren_length = 0;
    }
    Transform31.route_sources =  NULL;
    Transform31.num_route_source = 0;
    Transform31.m_parent = (X3dNode *)NULL;
    Transform32.route_sources =  NULL;
    Transform32.num_route_source = 0;
    Transform32.m_parent = (X3dNode *)NULL;
    Shape33.m_parent = (X3dNode *)NULL;
    Appearance33.m_parent = (X3dNode *)NULL;
    Material29.m_parent = (X3dNode *)NULL;
    Material29.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material29.diffuseColor = m_diffuseColor;
    Material29.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material29.emissiveColor = m_emissiveColor;
    Material29.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material29.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material29.specularColor = m_specularColor;
    Material29.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material29.transparency = 0;
    Appearance33.material = &Material29;
    Appearance33.texture = NULL;
    Appearance33.textureTransform = NULL;
    IndexedFaceSet31.setGlName(30);
    IndexedFaceSet31.m_parent = (X3dNode *)NULL;
    Coordinate53.m_parent = (X3dNode *)&IndexedFaceSet31;
    {
    static float m_point[] = { 0, 0, -1, 0.382683, 0, -0.923880, 0.707107, 0, -0.707107, 0.923880, 0, -0.382683, 1, 0, 0, 0.923880, 0, 0.382684, 0.707107, 0, 0.707107, 0.382683, 0, 0.923880, 0, 0, 1, -0.382683, 0, 0.923880, -0.707107, 0, 0.707107, -0.923880, 0, 0.382683, -1, 0, 0, -0.923879, 0, -0.382684, -0.707107, 0, -0.707107, -0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, 0.363954, 0.309017, -0.878662, 0.672499, 0.309017, -0.672499, 0.878662, 0.309017, -0.363954, 0.951057, 0.309017, 0, 0.878662, 0.309017, 0.363954, 0.672499, 0.309017, 0.672499, 0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, -0.363954, 0.309017, 0.878662, -0.672499, 0.309017, 0.672498, -0.878662, 0.309017, 0.363953, -0.951057, 0.309017, 0, -0.878662, 0.309017, -0.363954, -0.672498, 0.309017, -0.672499, -0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, 0.309597, 0.587785, -0.747434, 0.572061, 0.587785, -0.572061, 0.747434, 0.587785, -0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, 0.309597, 0.572061, 0.587785, 0.572061, 0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, -0.309597, 0.587785, 0.747434, -0.572061, 0.587785, 0.572061, -0.747434, 0.587785, 0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, -0.309598, -0.572061, 0.587785, -0.572062, -0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, 0.224936, 0.809017, -0.543043, 0.415627, 0.809017, -0.415627, 0.543043, 0.809017, -0.224936, 0.587785, 0.809017, 0, 0.543043, 0.809017, 0.224936, 0.415627, 0.809017, 0.415627, 0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, -0.224936, 0.809017, 0.543043, -0.415627, 0.809017, 0.415627, -0.543043, 0.809017, 0.224935, -0.587785, 0.809017, 0, -0.543043, 0.809017, -0.224936, -0.415627, 0.809017, -0.415627, -0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, 0.118256, 0.951057, -0.285494, 0.218508, 0.951057, -0.218508, 0.285494, 0.951057, -0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, 0.118256, 0.218508, 0.951057, 0.218508, 0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, -0.118256, 0.951057, 0.285494, -0.218508, 0.951057, 0.218508, -0.285495, 0.951057, 0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, -0.118256, -0.218508, 0.951057, -0.218508, -0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate53.point = m_point;
    Coordinate53.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal17.m_parent = (X3dNode *)&IndexedFaceSet31;
    {
    static float m_vector[] = { 0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, 0.185864, 0.303889, -0.934401, 0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377972, 0.156433, -0.912505, 0.377972, 0.156433, -0.912505, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.698401, 0.156433, -0.698401, 0.698401, 0.156433, -0.698401, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.912505, 0.156434, -0.377972, 0.912505, 0.156434, -0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987689, 0.156434, 0, 0.987689, 0.156434, 0, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.912505, 0.156434, 0.377972, 0.912505, 0.156434, 0.377972, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.698401, 0.156434, 0.698402, 0.698401, 0.156434, 0.698402, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377971, 0.156434, 0.912505, 0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377973, 0.156433, 0.912505, -0.377973, 0.156433, 0.912505, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.698402, 0.156434, 0.698400, -0.698402, 0.156434, 0.698400, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.912505, 0.156435, 0.377972, -0.912505, 0.156435, 0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987688, 0.156434, 0, -0.987688, 0.156434, 0, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.912505, 0.156433, -0.377973, -0.912505, 0.156433, -0.377973, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.698401, 0.156434, -0.698401, -0.698401, 0.156434, -0.698401, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377973, 0.156435, -0.912505, -0.377973, 0.156435, -0.912505, -0.104474, 0.325528, -0.939743, -0.105800, 0.318418, -0.942028, -0.104474, 0.325528, -0.939743, -0.104474, 0.325528, -0.939743, -0.097141, 0.233062, -0.967598, -0.097141, 0.233062, -0.967598, 0.158828, 0.580683, -0.798486, 0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.090247, 0.600810, -0.794282, -0.091515, 0.594767, -0.798672, -0.090247, 0.600810, -0.794282, -0.090247, 0.600810, -0.794282, 0.116055, 0.803815, -0.583449, 0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.067135, 0.818259, -0.570916, -0.068222, 0.813901, -0.576985, -0.067135, 0.818259, -0.570916, -0.067135, 0.818259, -0.570916, 0.061299, 0.949355, -0.308167, 0.061299, 0.949355, -0.308167, 0, 1, 0, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137550, 0.933532, -0.331056, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253769, 0.933532, -0.253218, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331354, 0.933532, -0.136830, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, 0.000389, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331056, 0.933532, 0.137550, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253218, 0.933532, 0.253769, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.136830, 0.933532, 0.331354, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, -0.000389, 0.933532, 0.358494, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137550, 0.933532, 0.331056, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253768, 0.933532, 0.253219, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331354, 0.933532, 0.136830, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, -0.000391, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331056, 0.933532, -0.137550, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253218, 0.933532, -0.253769, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.136830, 0.933532, -0.331354, -0.042033, 0.942951, -0.330266, -0.042048, 0.942867, -0.330504, -0.042033, 0.942951, -0.330266, 0, 1, 0, 0.031102, 0.987211, -0.156357, 0.088569, 0.987211, -0.132553, 0.132553, 0.987211, -0.088569, 0.156357, 0.987211, -0.031102, 0.156357, 0.987211, 0.031102, 0.132553, 0.987211, 0.088569, 0.088569, 0.987211, 0.132553, 0.031102, 0.987211, 0.156357, -0.031102, 0.987211, 0.156357, -0.088569, 0.987211, 0.132553, -0.132553, 0.987211, 0.088570, -0.156357, 0.987211, 0.031101, -0.156357, 0.987211, -0.031102, -0.132553, 0.987211, -0.088569, -0.088569, 0.987211, -0.132553, -0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal17.vector = m_vector;
    Normal17.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate25.m_parent = (X3dNode *)&IndexedFaceSet31;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate25.point = m_point;
    TextureCoordinate25.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet31.color = NULL;
    IndexedFaceSet31.coord = &Coordinate53;
    IndexedFaceSet31.normal = &Normal17;
    IndexedFaceSet31.texCoord = &TextureCoordinate25;
    IndexedFaceSet31.ccw = true;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet31.colorIndex = m_colorIndex;
    IndexedFaceSet31.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet31.colorPerVertex = true;
    IndexedFaceSet31.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet31.coordIndex = m_coordIndex;
    IndexedFaceSet31.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet31.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet31.normalIndex = m_normalIndex;
    IndexedFaceSet31.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet31.normalPerVertex = true;
    IndexedFaceSet31.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet31.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet31.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet31.set_colorIndex = NULL;
    IndexedFaceSet31.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet31.set_coordIndex = NULL;
    IndexedFaceSet31.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet31.set_normalIndex = NULL;
    IndexedFaceSet31.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet31.set_texCoordIndex = NULL;
    IndexedFaceSet31.set_texCoordIndex_length = 0;
    }
    Shape33.appearance = &Appearance33;
    Shape33.geometry = &IndexedFaceSet31;
    Transform33.route_sources =  NULL;
    Transform33.num_route_source = 0;
    Transform33.m_parent = (X3dNode *)NULL;
    Shape34.m_parent = (X3dNode *)NULL;
    Appearance34.m_parent = (X3dNode *)NULL;
    Material30.m_parent = (X3dNode *)NULL;
    Material30.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material30.diffuseColor = m_diffuseColor;
    Material30.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material30.emissiveColor = m_emissiveColor;
    Material30.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material30.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material30.specularColor = m_specularColor;
    Material30.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material30.transparency = 0;
    Appearance34.material = &Material30;
    Appearance34.texture = NULL;
    Appearance34.textureTransform = NULL;
    IndexedFaceSet32.setGlName(31);
    IndexedFaceSet32.m_parent = (X3dNode *)NULL;
    Coordinate55.m_parent = (X3dNode *)&IndexedFaceSet32;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate55.point = m_point;
    Coordinate55.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal18.m_parent = (X3dNode *)&IndexedFaceSet32;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal18.vector = m_vector;
    Normal18.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate27.m_parent = (X3dNode *)&IndexedFaceSet32;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate27.point = m_point;
    TextureCoordinate27.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet32.color = NULL;
    IndexedFaceSet32.coord = &Coordinate55;
    IndexedFaceSet32.normal = &Normal18;
    IndexedFaceSet32.texCoord = &TextureCoordinate27;
    IndexedFaceSet32.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet32.colorIndex = m_colorIndex;
    IndexedFaceSet32.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet32.colorPerVertex = true;
    IndexedFaceSet32.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet32.coordIndex = m_coordIndex;
    IndexedFaceSet32.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet32.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet32.normalIndex = m_normalIndex;
    IndexedFaceSet32.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet32.normalPerVertex = true;
    IndexedFaceSet32.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet32.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet32.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet32.set_colorIndex = NULL;
    IndexedFaceSet32.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet32.set_coordIndex = NULL;
    IndexedFaceSet32.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet32.set_normalIndex = NULL;
    IndexedFaceSet32.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet32.set_texCoordIndex = NULL;
    IndexedFaceSet32.set_texCoordIndex_length = 0;
    }
    Shape34.appearance = &Appearance34;
    Shape34.geometry = &IndexedFaceSet32;
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, 0 };
    Transform33.rotation = m_rotation;
    Transform33.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.220000, 0.440000, 0.160000 };
    Transform33.translation = m_translation;
    Transform33.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform33.bboxCenter = m_bboxCenter;
    Transform33.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform33.bboxSize = m_bboxSize;
    Transform33.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform33.center = m_center;
    Transform33.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform33.scale = m_scale;
    Transform33.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform33.scaleOrientation = m_scaleOrientation;
    Transform33.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape34 };
    Transform33.children = m_children;
    Transform33.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform33.addChildren = NULL;
    Transform33.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform33.removeChildren = NULL;
    Transform33.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, -1.578764 };
    Transform32.rotation = m_rotation;
    Transform32.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform32.translation = m_translation;
    Transform32.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform32.bboxCenter = m_bboxCenter;
    Transform32.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform32.bboxSize = m_bboxSize;
    Transform32.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform32.center = m_center;
    Transform32.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform32.scale = m_scale;
    Transform32.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform32.scaleOrientation = m_scaleOrientation;
    Transform32.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape33, &Transform33 };
    Transform32.children = m_children;
    Transform32.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform32.addChildren = NULL;
    Transform32.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform32.removeChildren = NULL;
    Transform32.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.565552, -0.792356, -0.228743, -0.826199 };
    Transform31.rotation = m_rotation;
    Transform31.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.109765, 0.715337, 1.294240 };
    Transform31.translation = m_translation;
    Transform31.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform31.bboxCenter = m_bboxCenter;
    Transform31.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform31.bboxSize = m_bboxSize;
    Transform31.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform31.center = m_center;
    Transform31.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform31.scale = m_scale;
    Transform31.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform31.scaleOrientation = m_scaleOrientation;
    Transform31.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform32 };
    Transform31.children = m_children;
    Transform31.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform31.addChildren = NULL;
    Transform31.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform31.removeChildren = NULL;
    Transform31.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform27.rotation = m_rotation;
    Transform27.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0.120000 };
    Transform27.translation = m_translation;
    Transform27.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform27.bboxCenter = m_bboxCenter;
    Transform27.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform27.bboxSize = m_bboxSize;
    Transform27.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform27.center = m_center;
    Transform27.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform27.scale = m_scale;
    Transform27.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform27.scaleOrientation = m_scaleOrientation;
    Transform27.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform28, &Transform31 };
    Transform27.children = m_children;
    Transform27.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform27.addChildren = NULL;
    Transform27.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform27.removeChildren = NULL;
    Transform27.removeChildren_length = 0;
    }
    Inline17.m_parent = (X3dNode *)NULL;
    Inline15.m_parent = (X3dNode *)NULL;
    Group14.m_parent = (X3dNode *)&Inline15;
    Transform34.route_sources =  NULL;
    Transform34.num_route_source = 0;
    Transform34.m_parent = (X3dNode *)NULL;
    CORINTm_7.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_7.key = m_key;
    CORINTm_7.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.619297, 0.959445, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.779760, 0.966057, -0.506980, 1.645980, 0.964696, -0.929632, 2.257560, 1.170100, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890 };
    CORINTm_7.keyValue = m_keyValue;
    CORINTm_7.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_7.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_7.value_changed = NULL;
    CORINTm_7.value_changed_length = 0;
    }
    TIMERm_6.m_parent = (X3dNode *)NULL;
    TIMERm_6.cycleInterval = 2;
    TIMERm_6.enabled = true;
    TIMERm_6.loop = true;
    TIMERm_6.startTime = 0;
    TIMERm_6.stopTime = 0;
    TIMERm_6.cycleTime = 0;
    TIMERm_6.fraction_changed = 0;
    TIMERm_6.isActive = false;
    TIMERm_6.time = 0;
    Group15.m_parent = (X3dNode *)NULL;
    Background5.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background5.groundAngle = NULL;
    Background5.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background5.groundColor = m_groundColor;
    Background5.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background5.backUrl = NULL;
    Background5.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background5.rightUrl = NULL;
    Background5.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background5.frontUrl = NULL;
    Background5.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background5.leftUrl = NULL;
    Background5.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background5.topUrl = NULL;
    Background5.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background5.bottomUrl = NULL;
    Background5.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background5.skyAngle = NULL;
    Background5.skyAngle_length = 0;
    }
    Background5.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background5.skyColor = m_skyColor;
    Background5.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background5.set_bind = false;
    Background5.isBound = false;
    Shape35.m_parent = (X3dNode *)NULL;
    Appearance35.m_parent = (X3dNode *)NULL;
    Material31.m_parent = (X3dNode *)NULL;
    Material31.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material31.diffuseColor = m_diffuseColor;
    Material31.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material31.emissiveColor = m_emissiveColor;
    Material31.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material31.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material31.specularColor = m_specularColor;
    Material31.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material31.transparency = 6.382980e-2;
    Appearance35.material = &Material31;
    Appearance35.texture = NULL;
    Appearance35.textureTransform = NULL;
    IndexedFaceSet33.setGlName(32);
    IndexedFaceSet33.m_parent = (X3dNode *)NULL;
    COORD1m_9.m_parent = (X3dNode *)&IndexedFaceSet33;
    {
    static float m_point[] = { 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.780723, -0.294626, 1.000070, 0.734580, -0.981481, 1.700790, 0.307968, -0.506980, 1.580170, 0.280283, -1.021760, 2.071980, -0.277696, -0.976890 };
    COORD1m_9.point = m_point;
    COORD1m_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal51.m_parent = (X3dNode *)&IndexedFaceSet33;
    {
    static float m_vector[] = { 0.358806, 0.927849, -0.101757, 0.358806, 0.927849, -0.101757, 0.359718, 0.927470, -0.101994, 0.281824, 0.955978, -0.081744, 0.281824, 0.955978, -0.081744, 0.240191, 0.969864, -0.040901, 0.495155, 0.855218, -0.153047, 0.496190, 0.854566, -0.153340, 0.496190, 0.854566, -0.153340, 0.631845, 0.753624, -0.181169, 0.631845, 0.753624, -0.181169, 0.632281, 0.753233, -0.181277, 0.673713, 0.712478, -0.196177, 0.673713, 0.712478, -0.196177, 0.742001, 0.637272, -0.208134 };
    Normal51.vector = m_vector;
    Normal51.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet33.color = NULL;
    IndexedFaceSet33.coord = &COORD1m_9;
    IndexedFaceSet33.normal = &Normal51;
    IndexedFaceSet33.texCoord = NULL;
    IndexedFaceSet33.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet33.colorIndex = m_colorIndex;
    IndexedFaceSet33.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet33.colorPerVertex = true;
    IndexedFaceSet33.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet33.coordIndex = m_coordIndex;
    IndexedFaceSet33.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet33.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet33.normalIndex = m_normalIndex;
    IndexedFaceSet33.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet33.normalPerVertex = true;
    IndexedFaceSet33.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet33.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet33.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet33.set_colorIndex = NULL;
    IndexedFaceSet33.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet33.set_coordIndex = NULL;
    IndexedFaceSet33.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet33.set_normalIndex = NULL;
    IndexedFaceSet33.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet33.set_texCoordIndex = NULL;
    IndexedFaceSet33.set_texCoordIndex_length = 0;
    }
    Shape35.appearance = &Appearance35;
    Shape35.geometry = &IndexedFaceSet33;
    {
    static X3dNode* m_children[] = { &Background5, &Shape35 };
    Group15.children = m_children;
    Group15.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group15.bboxCenter = m_bboxCenter;
    Group15.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group15.bboxSize = m_bboxSize;
    Group15.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group15.addChildren = NULL;
    Group15.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group15.removeChildren = NULL;
    Group15.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform34.rotation = m_rotation;
    Transform34.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform34.translation = m_translation;
    Transform34.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform34.bboxCenter = m_bboxCenter;
    Transform34.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform34.bboxSize = m_bboxSize;
    Transform34.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform34.center = m_center;
    Transform34.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform34.scale = m_scale;
    Transform34.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform34.scaleOrientation = m_scaleOrientation;
    Transform34.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_7, &TIMERm_6, &Group15 };
    Transform34.children = m_children;
    Transform34.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform34.addChildren = NULL;
    Transform34.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform34.removeChildren = NULL;
    Transform34.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform34 };
    Group14.children = m_children;
    Group14.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group14.bboxCenter = m_bboxCenter;
    Group14.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group14.bboxSize = m_bboxSize;
    Group14.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group14.addChildren = NULL;
    Group14.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group14.removeChildren = NULL;
    Group14.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Group14 };
    Inline15.children = m_children;
    Inline15.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline15.bboxCenter = m_bboxCenter;
    Inline15.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline15.bboxSize = m_bboxSize;
    Inline15.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline15.addChildren = NULL;
    Inline15.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline15.removeChildren = NULL;
    Inline15.removeChildren_length = 0;
    }
    Inline16.m_parent = (X3dNode *)NULL;
    Group16.m_parent = (X3dNode *)&Inline16;
    Transform35.route_sources =  NULL;
    Transform35.num_route_source = 0;
    Transform35.m_parent = (X3dNode *)NULL;
    CORINTm_8.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_8.key = m_key;
    CORINTm_8.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.619297, 0.959445, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.779760, 0.966057, -0.506980, -1.645980, 0.964696, -0.929632, -2.257560, 1.170100, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890 };
    CORINTm_8.keyValue = m_keyValue;
    CORINTm_8.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_8.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_8.value_changed = NULL;
    CORINTm_8.value_changed_length = 0;
    }
    TIMERm_7.m_parent = (X3dNode *)NULL;
    TIMERm_7.cycleInterval = 2;
    TIMERm_7.enabled = true;
    TIMERm_7.loop = true;
    TIMERm_7.startTime = 0;
    TIMERm_7.stopTime = 0;
    TIMERm_7.cycleTime = 0;
    TIMERm_7.fraction_changed = 0;
    TIMERm_7.isActive = false;
    TIMERm_7.time = 0;
    Group17.m_parent = (X3dNode *)NULL;
    Background6.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background6.groundAngle = NULL;
    Background6.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background6.groundColor = m_groundColor;
    Background6.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background6.backUrl = NULL;
    Background6.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background6.rightUrl = NULL;
    Background6.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background6.frontUrl = NULL;
    Background6.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background6.leftUrl = NULL;
    Background6.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background6.topUrl = NULL;
    Background6.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background6.bottomUrl = NULL;
    Background6.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background6.skyAngle = NULL;
    Background6.skyAngle_length = 0;
    }
    Background6.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background6.skyColor = m_skyColor;
    Background6.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background6.set_bind = false;
    Background6.isBound = false;
    Shape36.m_parent = (X3dNode *)NULL;
    Appearance36.m_parent = (X3dNode *)NULL;
    Material32.m_parent = (X3dNode *)NULL;
    Material32.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material32.diffuseColor = m_diffuseColor;
    Material32.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material32.emissiveColor = m_emissiveColor;
    Material32.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material32.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material32.specularColor = m_specularColor;
    Material32.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material32.transparency = 6.382980e-2;
    Appearance36.material = &Material32;
    Appearance36.texture = NULL;
    Appearance36.textureTransform = NULL;
    IndexedFaceSet34.setGlName(33);
    IndexedFaceSet34.m_parent = (X3dNode *)NULL;
    COORD1m_11.m_parent = (X3dNode *)&IndexedFaceSet34;
    {
    static float m_point[] = { -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.780723, -0.294626, -1.000070, 0.734580, -0.981481, -1.700790, 0.307968, -0.506980, -1.580170, 0.280283, -1.021760, -2.071980, -0.277696, -0.976890 };
    COORD1m_11.point = m_point;
    COORD1m_11.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal52.m_parent = (X3dNode *)&IndexedFaceSet34;
    {
    static float m_vector[] = { 0.358806, -0.927849, 0.101757, 0.358806, -0.927849, 0.101757, 0.359718, -0.927470, 0.101994, 0.281824, -0.955978, 0.081744, 0.281824, -0.955978, 0.081744, 0.240191, -0.969864, 0.040901, 0.495155, -0.855218, 0.153047, 0.496190, -0.854566, 0.153340, 0.496190, -0.854566, 0.153340, 0.631845, -0.753624, 0.181169, 0.631845, -0.753624, 0.181169, 0.632281, -0.753233, 0.181277, 0.673713, -0.712478, 0.196177, 0.673713, -0.712478, 0.196177, 0.742001, -0.637272, 0.208134 };
    Normal52.vector = m_vector;
    Normal52.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet34.color = NULL;
    IndexedFaceSet34.coord = &COORD1m_11;
    IndexedFaceSet34.normal = &Normal52;
    IndexedFaceSet34.texCoord = NULL;
    IndexedFaceSet34.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet34.colorIndex = m_colorIndex;
    IndexedFaceSet34.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet34.colorPerVertex = true;
    IndexedFaceSet34.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet34.coordIndex = m_coordIndex;
    IndexedFaceSet34.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet34.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet34.normalIndex = m_normalIndex;
    IndexedFaceSet34.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet34.normalPerVertex = true;
    IndexedFaceSet34.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet34.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet34.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet34.set_colorIndex = NULL;
    IndexedFaceSet34.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet34.set_coordIndex = NULL;
    IndexedFaceSet34.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet34.set_normalIndex = NULL;
    IndexedFaceSet34.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet34.set_texCoordIndex = NULL;
    IndexedFaceSet34.set_texCoordIndex_length = 0;
    }
    Shape36.appearance = &Appearance36;
    Shape36.geometry = &IndexedFaceSet34;
    {
    static X3dNode* m_children[] = { &Background6, &Shape36 };
    Group17.children = m_children;
    Group17.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group17.bboxCenter = m_bboxCenter;
    Group17.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group17.bboxSize = m_bboxSize;
    Group17.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group17.addChildren = NULL;
    Group17.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group17.removeChildren = NULL;
    Group17.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform35.rotation = m_rotation;
    Transform35.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform35.translation = m_translation;
    Transform35.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform35.bboxCenter = m_bboxCenter;
    Transform35.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform35.bboxSize = m_bboxSize;
    Transform35.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform35.center = m_center;
    Transform35.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform35.scale = m_scale;
    Transform35.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform35.scaleOrientation = m_scaleOrientation;
    Transform35.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_8, &TIMERm_7, &Group17 };
    Transform35.children = m_children;
    Transform35.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform35.addChildren = NULL;
    Transform35.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform35.removeChildren = NULL;
    Transform35.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform35 };
    Group16.children = m_children;
    Group16.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group16.bboxCenter = m_bboxCenter;
    Group16.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group16.bboxSize = m_bboxSize;
    Group16.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group16.addChildren = NULL;
    Group16.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group16.removeChildren = NULL;
    Group16.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Group16 };
    Inline16.children = m_children;
    Inline16.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline16.bboxCenter = m_bboxCenter;
    Inline16.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline16.bboxSize = m_bboxSize;
    Inline16.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline16.addChildren = NULL;
    Inline16.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline16.removeChildren = NULL;
    Inline16.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { NULL, &Inline15, &Inline16 };
    Inline17.children = m_children;
    Inline17.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline17.bboxCenter = m_bboxCenter;
    Inline17.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline17.bboxSize = m_bboxSize;
    Inline17.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline17.addChildren = NULL;
    Inline17.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline17.removeChildren = NULL;
    Inline17.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform22.rotation = m_rotation;
    Transform22.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform22.translation = m_translation;
    Transform22.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform22.bboxCenter = m_bboxCenter;
    Transform22.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform22.bboxSize = m_bboxSize;
    Transform22.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform22.center = m_center;
    Transform22.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform22.scale = m_scale;
    Transform22.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform22.scaleOrientation = m_scaleOrientation;
    Transform22.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline14, &Transform27, &Inline17 };
    Transform22.children = m_children;
    Transform22.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform22.addChildren = NULL;
    Transform22.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform22.removeChildren = NULL;
    Transform22.removeChildren_length = 0;
    }
    WorldInfo10.m_parent = (X3dNode *)&Inline18;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "p_fly is a modification of the program tux_aqfh by Steve Baker", "tux_aqfh is GPL" };
    WorldInfo10.info = m_info;
    WorldInfo10.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo10.title = "Tux warbird 3D Game";
    {
    static X3dNode* m_children[] = { &Transform22, &WorldInfo10 };
    Inline18.children = m_children;
    Inline18.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline18.bboxCenter = m_bboxCenter;
    Inline18.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline18.bboxSize = m_bboxSize;
    Inline18.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline18.addChildren = NULL;
    Inline18.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline18.removeChildren = NULL;
    Inline18.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.140000 };
    Transform21.rotation = m_rotation;
    Transform21.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform21.translation = m_translation;
    Transform21.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform21.bboxCenter = m_bboxCenter;
    Transform21.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform21.bboxSize = m_bboxSize;
    Transform21.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform21.center = m_center;
    Transform21.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform21.scale = m_scale;
    Transform21.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform21.scaleOrientation = m_scaleOrientation;
    Transform21.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline18 };
    Transform21.children = m_children;
    Transform21.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform21.addChildren = NULL;
    Transform21.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform21.removeChildren = NULL;
    Transform21.removeChildren_length = 0;
    }
    Transform36.route_sources =  NULL;
    Transform36.num_route_source = 0;
    Transform36.m_parent = (X3dNode *)NULL;
    Shape37.m_parent = (X3dNode *)NULL;
    Text2.m_parent = (X3dNode *)NULL;
    Text2.fontStyle = NULL;
    {
    static float m_length[1];
    Text2.length = NULL;
    Text2.length_length = 0;
    }
    Text2.maxExtent = 0;
    {
    static const char* m_string[] = { "game over" };
    Text2.string = m_string;
    Text2.string_length = sizeof(m_string) / sizeof(const char*);
    }
    Shape37.appearance = NULL;
    Shape37.geometry = &Text2;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform36.rotation = m_rotation;
    Transform36.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 3 };
    Transform36.translation = m_translation;
    Transform36.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform36.bboxCenter = m_bboxCenter;
    Transform36.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform36.bboxSize = m_bboxSize;
    Transform36.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform36.center = m_center;
    Transform36.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform36.scale = m_scale;
    Transform36.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform36.scaleOrientation = m_scaleOrientation;
    Transform36.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape37 };
    Transform36.children = m_children;
    Transform36.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform36.addChildren = NULL;
    Transform36.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform36.removeChildren = NULL;
    Transform36.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    SCHRIFTm_6.rotation = m_rotation;
    SCHRIFTm_6.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    SCHRIFTm_6.translation = m_translation;
    SCHRIFTm_6.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    SCHRIFTm_6.bboxCenter = m_bboxCenter;
    SCHRIFTm_6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    SCHRIFTm_6.bboxSize = m_bboxSize;
    SCHRIFTm_6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    SCHRIFTm_6.center = m_center;
    SCHRIFTm_6.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    SCHRIFTm_6.scale = m_scale;
    SCHRIFTm_6.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    SCHRIFTm_6.scaleOrientation = m_scaleOrientation;
    SCHRIFTm_6.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform21, &Transform36 };
    SCHRIFTm_6.children = m_children;
    SCHRIFTm_6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    SCHRIFTm_6.addChildren = NULL;
    SCHRIFTm_6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    SCHRIFTm_6.removeChildren = NULL;
    SCHRIFTm_6.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &SCHRIFTm_6 };
    Anchor2.children = m_children;
    Anchor2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Anchor2.bboxCenter = m_bboxCenter;
    Anchor2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Anchor2.bboxSize = m_bboxSize;
    Anchor2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    Anchor2.description = "restart game";
    {
    static const char* m_parameter[1];
    Anchor2.parameter = NULL;
    Anchor2.parameter_length = 0;
    }
    {
    static const char* m_url[] = { "warbird.wrl" };
    Anchor2.url = m_url;
    Anchor2.url_length = sizeof(m_url) / sizeof(const char*);
    }
    {
    static X3dNode* m_addChildren[1];
    Anchor2.addChildren = NULL;
    Anchor2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Anchor2.removeChildren = NULL;
    Anchor2.removeChildren_length = 0;
    }
    ROTATORm_6.m_parent = (X3dNode *)&Inline19;
    {
    static float m_key[] = { 0, 0.500000, 1 };
    ROTATORm_6.key = m_key;
    ROTATORm_6.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0, 1, 0, 0, 0, 1, 0, 3.140000, 0, 1, 0, 6.280000 };
    ROTATORm_6.keyValue = m_keyValue;
    ROTATORm_6.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    ROTATORm_6.set_fraction = 0;
    {
    static float m_value_changed[] = { 0, 0, 1, 0 };
    ROTATORm_6.value_changed = m_value_changed;
    ROTATORm_6.value_changed_length = sizeof(m_value_changed) / sizeof(float);
    }
    TIMERm_8.m_parent = (X3dNode *)&Inline19;
    TIMERm_8.cycleInterval = 10;
    TIMERm_8.enabled = true;
    TIMERm_8.loop = true;
    TIMERm_8.startTime = 0;
    TIMERm_8.stopTime = 0;
    TIMERm_8.cycleTime = 0;
    TIMERm_8.fraction_changed = 0;
    TIMERm_8.isActive = false;
    TIMERm_8.time = 0;
    {
    static X3dNode* m_children[] = { &Background4, &Anchor1, &Anchor2, &ROTATORm_6, &TIMERm_8 };
    Inline19.children = m_children;
    Inline19.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline19.bboxCenter = m_bboxCenter;
    Inline19.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline19.bboxSize = m_bboxSize;
    Inline19.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline19.addChildren = NULL;
    Inline19.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline19.removeChildren = NULL;
    Inline19.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform20.rotation = m_rotation;
    Transform20.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -10 };
    Transform20.translation = m_translation;
    Transform20.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform20.bboxCenter = m_bboxCenter;
    Transform20.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform20.bboxSize = m_bboxSize;
    Transform20.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform20.center = m_center;
    Transform20.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform20.scale = m_scale;
    Transform20.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform20.scaleOrientation = m_scaleOrientation;
    Transform20.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline19 };
    Transform20.children = m_children;
    Transform20.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform20.addChildren = NULL;
    Transform20.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform20.removeChildren = NULL;
    Transform20.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group1, &Transform20 };
    Switch2.children = m_children;
    Switch2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    Switch2.whichChoice = 0;
    LOADm_NEWm_WORLD.m_parent = (X3dNode *)&root;
    {
    static const char* m_url[] = { "javascript:  \
  \
 // field MFInt32 box_in //  \
  \
 // field SFInt32 deleted //  \
  \
 // field SFInt32 waitticks //  \
  \
 // field SFBool victory //  \
  \
 // eventOut SFTime starttime_out //  \
  \
 // eventOut SFInt32 plane_choice //  \
  \
 // eventOut SFInt32 choice_out //  \
  \
 // eventOut SFInt32 victory_out //  \
  \
   function initialize()  \
      {  \
      // insert your url at game end here   \
      box_in[1]=0;  \
      box_in[2]=0;  \
      box_in[3]=0;  \
      box_in[4]=0;  \
      box_in[5]=0;  \
      starttime_out=0;  \
      plane_choice=0;  \
      deleted=waitticks;  \
      }  \
   function starttime_in(val)  \
      {  \
      if (!victory)        \
         if (deleted>0)  \
            {  \
            if (deleted==waitticks)  \
               {  \
               starttime_out=val;  \
               choice_out=0;  \
               }  \
            deleted--;  \
            }           \
         else {  \
            choice_out=-1;  \
        }  \
      }  \
   function test_end()  \
      {  \
      deleted=waitticks;  \
      if ((box_in[1]+box_in[2]+box_in[3]+box_in[4]+box_in[5])==5)  \
         {  \
         victory=TRUE;  \
         victory_out=1;   \
         }  \
      }  \
   function test_plane(val)  \
      {  \
      if (plane_choice==0)  \
         plane_choice=1;  \
      else  \
         plane_choice=0;  \
      }  \
   function box1_in(val)  \
      {  \
      test_plane(val);  \
      box_in[1]=val;  \
      test_end();  \
      }  \
   function box2_in(val)  \
      {  \
      test_plane(val);  \
      box_in[2]=val;  \
      test_end();  \
      }  \
   function box3_in(val)  \
      {  \
      test_plane(val);  \
      box_in[3]=val;  \
      test_end();  \
      }  \
   function box4_in(val)  \
      {  \
      test_plane(val);  \
      box_in[4]=val;  \
      test_end();  \
      }  \
   function box5_in(val)  \
      {  \
      test_plane(val);  \
      box_in[5]=val;  \
      test_end();  \
      }     \
", "c++:  \
      X3dScript_LOAD_NEW_WORLD *self = (X3dScript_LOAD_NEW_WORLD *)node;  \
      static bool LOAD_NEW_WORLD_init = false;  \
      if (!LOAD_NEW_WORLD_init) {  \
          self->box_in[0] = 0;  \
          self->box_in[1] = 0;  \
          self->box_in[2] = 0;  \
          self->box_in[3] = 0;  \
          self->box_in[4] = 0;  \
          self->starttime_out = 0;  \
          self->plane_choice = 0;  \
          self->deleted = self->waitticks;  \
          LOAD_NEW_WORLD_init = true;  \
      }           \
      if (!self->victory)        \
         if (self->deleted > 0)  \
            {  \
            if (self->deleted == self->waitticks)  \
               {  \
//               starttime_out=val;  \
               self->choice_out = 0;  \
               }  \
            self->deleted--;  \
            }           \
         else  \
            self->choice_out = -1;  \
  \
      if ((self->box1_in == 1) && (self->box_in[0] == 0))  \
          {  \
          self->deleted = self->waitticks;            \
          self->box_in[0] = self->box1_in;  \
          if (self->plane_choice == 0)  \
              self->plane_choice = 1;  \
          else  \
              self->plane_choice = 0;  \
          }  \
      if ((self->box2_in == 1) && (self->box_in[1] == 0))  \
          {  \
          self->deleted = self->waitticks;            \
          self->box_in[1] = self->box2_in;  \
          if (self->plane_choice == 0)  \
              self->plane_choice = 1;  \
          else  \
              self->plane_choice = 0;  \
          }  \
      if ((self->box3_in == 1) && (self->box_in[2] == 0))  \
          {  \
          self->deleted = self->waitticks;            \
          self->box_in[2] = self->box3_in;  \
          if (self->plane_choice == 0)  \
              self->plane_choice = 1;  \
          else  \
              self->plane_choice = 0;  \
          }  \
      if ((self->box4_in == 1) && (self->box_in[3] == 0))  \
          {  \
          self->deleted = self->waitticks;            \
          self->box_in[3] = self->box4_in;  \
          if (self->plane_choice == 0)  \
              self->plane_choice = 1;  \
          else  \
              self->plane_choice = 0;  \
          }  \
      if ((self->box5_in == 1) && (self->box_in[4] == 0))  \
          {  \
          self->deleted = self->waitticks;            \
          self->box_in[4] = self->box5_in;  \
          if (self->plane_choice == 0)  \
              self->plane_choice = 1;  \
          else  \
              self->plane_choice = 0;  \
          }  \
      if ((self->box_in[0] +   \
           self->box_in[1] +   \
           self->box_in[2] +   \
           self->box_in[3] +  \
           self->box_in[4]) == 5)  \
          {  \
          self->victory = true;  \
          self->victory_out = 1;  \
          }  \
" };
    LOADm_NEWm_WORLD.url = m_url;
    LOADm_NEWm_WORLD.url_length = sizeof(m_url) / sizeof(const char*);
    }
    LOADm_NEWm_WORLD.directOutput = false;
    LOADm_NEWm_WORLD.mustEvaluate = false;
    {
    static int m_box_in[] = { 0, 0, 0, 0, 0 };
    LOADm_NEWm_WORLD.box_in = m_box_in;
    LOADm_NEWm_WORLD.box_in_length = sizeof(m_box_in) / sizeof(int);
    }
    {
    static const char* m_your_url[] = { "" };
    LOADm_NEWm_WORLD.your_url = m_your_url;
    LOADm_NEWm_WORLD.your_url_length = sizeof(m_your_url) / sizeof(const char*);
    }
    {
    static const char* m_param[] = { "" };
    LOADm_NEWm_WORLD.param = m_param;
    LOADm_NEWm_WORLD.param_length = sizeof(m_param) / sizeof(const char*);
    }
    LOADm_NEWm_WORLD.deleted = 0;
    LOADm_NEWm_WORLD.waitticks = 300;
    LOADm_NEWm_WORLD.victory = false;
    LOADm_NEWm_WORLD.starttime_in = 0;
    LOADm_NEWm_WORLD.box1_in = 0;
    LOADm_NEWm_WORLD.box2_in = 0;
    LOADm_NEWm_WORLD.box3_in = 0;
    LOADm_NEWm_WORLD.box4_in = 0;
    LOADm_NEWm_WORLD.box5_in = 0;
    LOADm_NEWm_WORLD.starttime_out = 0;
    LOADm_NEWm_WORLD.plane_choice = 0;
    LOADm_NEWm_WORLD.choice_out = 0;
    LOADm_NEWm_WORLD.victory_out = 0;
    {
    static X3dNode* m_children[] = { &Switch2, &LOADm_NEWm_WORLD };
    root.children = m_children;
    root.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    root.bboxCenter = m_bboxCenter;
    root.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    root.bboxSize = m_bboxSize;
    root.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    root.addChildren = NULL;
    root.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    root.removeChildren = NULL;
    root.removeChildren_length = 0;
    }
    X3dScript_BOX1_HIT_CHECK::processEventCallback = X3dScript_BOX1_HIT_CHECKSendEventsCallback;
    X3dScript_BOX1_MOVE_ON::processEventCallback = X3dScript_BOX1_MOVE_ONSendEventsCallback;
    X3dScript_BOX1_HIDE_SCRIPT::processEventCallback = X3dScript_BOX1_HIDE_SCRIPTSendEventsCallback;
    X3dScript_STEER::processEventCallback = X3dScript_STEERSendEventsCallback;
    X3dScript_ROT_TRANS1::processEventCallback = X3dScript_ROT_TRANS1SendEventsCallback;
    X3dScript_ROT_TRANS2::processEventCallback = X3dScript_ROT_TRANS2SendEventsCallback;
    X3dScript_WALL_FIRE::processEventCallback = X3dScript_WALL_FIRESendEventsCallback;
    X3dScript_ROLL_TURN::processEventCallback = X3dScript_ROLL_TURNSendEventsCallback;
    X3dScript_UP_DOWN_TURN::processEventCallback = X3dScript_UP_DOWN_TURNSendEventsCallback;
    X3dScript_FLY_FORWARD::processEventCallback = X3dScript_FLY_FORWARDSendEventsCallback;
    X3dScript_FLY_ON::processEventCallback = X3dScript_FLY_ONSendEventsCallback;
    X3dScript_THING_SCRIPT::processEventCallback = X3dScript_THING_SCRIPTSendEventsCallback;
    X3dScript_BOX2_HIT_CHECK::processEventCallback = X3dScript_BOX2_HIT_CHECKSendEventsCallback;
    X3dScript_BOX2_MOVE_ON::processEventCallback = X3dScript_BOX2_MOVE_ONSendEventsCallback;
    X3dScript_BOX2_HIDE_SCRIPT::processEventCallback = X3dScript_BOX2_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX3_HIT_CHECK::processEventCallback = X3dScript_BOX3_HIT_CHECKSendEventsCallback;
    X3dScript_BOX3_MOVE_ON::processEventCallback = X3dScript_BOX3_MOVE_ONSendEventsCallback;
    X3dScript_BOX3_HIDE_SCRIPT::processEventCallback = X3dScript_BOX3_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX4_HIT_CHECK::processEventCallback = X3dScript_BOX4_HIT_CHECKSendEventsCallback;
    X3dScript_BOX4_MOVE_ON::processEventCallback = X3dScript_BOX4_MOVE_ONSendEventsCallback;
    X3dScript_BOX4_HIDE_SCRIPT::processEventCallback = X3dScript_BOX4_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX5_HIT_CHECK::processEventCallback = X3dScript_BOX5_HIT_CHECKSendEventsCallback;
    X3dScript_BOX5_MOVE_ON::processEventCallback = X3dScript_BOX5_MOVE_ONSendEventsCallback;
    X3dScript_BOX5_HIDE_SCRIPT::processEventCallback = X3dScript_BOX5_HIDE_SCRIPTSendEventsCallback;
    X3dScript_LOAD_NEW_WORLD::processEventCallback = X3dScript_LOAD_NEW_WORLDSendEventsCallback;
}
X3dNode *X3dSceneGraph::getNodeFromGlName(int glName) {
    switch (glName) {
       case 0:
         return &IndexedFaceSet1;
       case 1:
         return &IndexedFaceSet2;
       case 2:
         return &IndexedFaceSet3;
       case 3:
         return &IndexedFaceSet4;
       case 4:
         return &IndexedFaceSet5;
       case 5:
         return &IndexedFaceSet6;
       case 6:
         return &IndexedFaceSet7;
       case 7:
         return &IndexedFaceSet8;
       case 8:
         return &IndexedFaceSet9;
       case 9:
         return &IndexedFaceSet10;
       case 10:
         return &IndexedFaceSet11;
       case 11:
         return &IndexedFaceSet12;
       case 12:
         return &IndexedFaceSet13;
       case 13:
         return &IndexedFaceSet14;
       case 14:
         return &IndexedFaceSet15;
       case 15:
         return &IndexedFaceSet16;
       case 16:
         return &IndexedFaceSet17;
       case 17:
         return &IndexedFaceSet18;
       case 18:
         return &IndexedFaceSet19;
       case 19:
         return &IndexedFaceSet20;
       case 20:
         return &IndexedFaceSet21;
       case 21:
         return &IndexedFaceSet22;
       case 22:
         return &IndexedFaceSet23;
       case 23:
         return &IndexedFaceSet24;
       case 24:
         return &IndexedFaceSet25;
       case 25:
         return &IndexedFaceSet26;
       case 26:
         return &IndexedFaceSet27;
       case 27:
         return &IndexedFaceSet28;
       case 28:
         return &IndexedFaceSet29;
       case 29:
         return &IndexedFaceSet30;
       case 30:
         return &IndexedFaceSet31;
       case 31:
         return &IndexedFaceSet32;
       case 32:
         return &IndexedFaceSet33;
       case 33:
         return &IndexedFaceSet34;
    }
    return NULL;
}
void X3dMFBoolSendEvent(bool* target, bool* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFColorSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFColorRGBASendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFDoubleSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFFloatSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFInt32SendEvent(int* target, int* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFMatrix3fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFMatrix4fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFNodeSendEvent(X3dNode** target, X3dNode** source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFRotationSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFStringSendEvent(const char** target, const char** source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFTimeSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec2dSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec2fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec3dSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec3fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec4fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dSFBoolSendEvent(bool* target, bool source) {
    *target = source;
}
void X3dSFColorSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dSFColorRGBASendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 4; i++)
        target[i] = source[i];
}
void X3dSFDoubleSendEvent(double* target, double source) {
    *target = source;
}
void X3dSFFloatSendEvent(float* target, float source) {
    *target = source;
}
void X3dSFImageSendEvent(int* target, int* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dSFInt32SendEvent(int* target, int source) {
    *target = source;
}
void X3dSFMatrix3fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 9; i++)
        target[i] = source[i];
}
void X3dSFMatrix4fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 9; i++)
        target[i] = source[i];
}
void X3dSFNodeSendEvent(X3dNode** target, X3dNode* source) {
    *target = source;
}
void X3dSFRotationSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 4; i++)
        target[i] = source[i];
}
void X3dSFStringSendEvent(const char** target, const char* source) {
    *target = source;
}
void X3dSFTimeSendEvent(double* target, double source) {
    *target = source;
}
void X3dSFVec2dSendEvent(double* target, double* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 2; i++)
        target[i] = source[i];
}
void X3dSFVec2fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 2; i++)
        target[i] = source[i];
}
void X3dSFVec3dSendEvent(double* target, double* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dSFVec3fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dProcessEvents(X3dSceneGraph *self, void *data) {
    int nextEvent = 0;
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX1m_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->BOX1m_HIDEm_SCRIPT.isover_in), self->BOX1m_SENSOR.isOver);
            if (X3dScript_BOX1_HIDE_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_BOX1_HIDE_SCRIPT::processEventCallback(&self->BOX1m_HIDEm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->BOX1m_HIDE.translation, self->BOX1m_HIDEm_SCRIPT.hide_out);
                }
            }
        }
    }
    if (X3dPlaneSensor::processEventCallback != NULL) {
        nextEvent = X3dPlaneSensor::processEventCallback(&self->WALLm_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFVec3fSendEvent(self->STEER.wall_xy_in, self->WALLm_SENSOR.translation_changed);
            if (X3dScript_STEER::processEventCallback != NULL) {
                nextEvent = X3dScript_STEER::processEventCallback(&self->STEER, "wall_xy_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFFloatSendEvent(&(self->ROLLm_TURN.xvalue_in), self->STEER.xvalue);
                    if (X3dScript_ROLL_TURN::processEventCallback != NULL) {
                        nextEvent = X3dScript_ROLL_TURN::processEventCallback(&self->ROLLm_TURN, "xvalue_in", data);
                        if (nextEvent) {
                        }
                        {
                            X3dSFRotationSendEvent(self->ROLLm_AXIS.rotation, self->ROLLm_TURN.rotation_out);
                        }
                        {
                            X3dSFRotationSendEvent(self->ROTm_TRANS2.rotation_in, self->ROLLm_TURN.rotation_out);
                            if (X3dScript_ROT_TRANS2::processEventCallback != NULL) {
                                nextEvent = X3dScript_ROT_TRANS2::processEventCallback(&self->ROTm_TRANS2, "rotation_in", data);
                                if (nextEvent) {
                                }
                                {
                                    X3dSFVec3fSendEvent(self->FLYm_ON.translation_in, self->ROTm_TRANS2.translation_out);
                                    if (X3dScript_FLY_ON::processEventCallback != NULL) {
                                        nextEvent = X3dScript_FLY_ON::processEventCallback(&self->FLYm_ON, "translation_in", data);
                                        if (nextEvent) {
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->THING.translation, self->FLYm_ON.translation_out);
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX1m_HITm_CHECK.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX1_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX1_HIT_CHECK::processEventCallback(&self->BOX1m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->LOADm_NEWm_WORLD.box1_in), self->BOX1m_HITm_CHECK.deleted_out);
                                                    if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "box1_in", data);
                                                        if (nextEvent) {
                                                        }
                                                        {
                                                            X3dSFTimeSendEvent(&(self->SOUNDCLIP.startTime), self->LOADm_NEWm_WORLD.starttime_out);
                                                        }
                                                        {
                                                            X3dSFInt32SendEvent(&(self->ROLLm_TURN.rock_and_roll_flag_in), self->LOADm_NEWm_WORLD.plane_choice);
                                                            if (X3dScript_ROLL_TURN::processEventCallback != NULL) {
                                                                nextEvent = X3dScript_ROLL_TURN::processEventCallback(&self->ROLLm_TURN, "rock_and_roll_flag_in", data);
                                                                if (nextEvent) {
                                                                }
                                                            }
                                                        }
                                                        {
                                                            X3dSFInt32SendEvent(&(self->PLANEm_SWITCH.whichChoice), self->LOADm_NEWm_WORLD.plane_choice);
                                                        }
                                                        {
                                                            X3dSFInt32SendEvent(&(self->Switch1.whichChoice), self->LOADm_NEWm_WORLD.choice_out);
                                                        }
                                                        {
                                                            X3dSFInt32SendEvent(&(self->Switch2.whichChoice), self->LOADm_NEWm_WORLD.victory_out);
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->BOX1m_SWITCH.whichChoice), self->BOX1m_HITm_CHECK.choice_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX1m_MOVEm_ON.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX1_MOVE_ON::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX1_MOVE_ON::processEventCallback(&self->BOX1m_MOVEm_ON, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX1m_HITm_CHECK.box_position, self->BOX1m_MOVEm_ON.translation_out);
                                                    if (X3dScript_BOX1_HIT_CHECK::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_BOX1_HIT_CHECK::processEventCallback(&self->BOX1m_HITm_CHECK, "box_position", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX1m_MOVE.translation, self->BOX1m_MOVEm_ON.translation_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX2m_HITm_CHECK.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX2_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX2_HIT_CHECK::processEventCallback(&self->BOX2m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->LOADm_NEWm_WORLD.box2_in), self->BOX2m_HITm_CHECK.deleted_out);
                                                    if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "box2_in", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->BOX2m_SWITCH.whichChoice), self->BOX2m_HITm_CHECK.choice_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX2m_MOVEm_ON.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX2_MOVE_ON::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX2_MOVE_ON::processEventCallback(&self->BOX2m_MOVEm_ON, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX2m_HITm_CHECK.box_position, self->BOX2m_MOVEm_ON.translation_out);
                                                    if (X3dScript_BOX2_HIT_CHECK::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_BOX2_HIT_CHECK::processEventCallback(&self->BOX2m_HITm_CHECK, "box_position", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX2m_MOVE.translation, self->BOX2m_MOVEm_ON.translation_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX3m_HITm_CHECK.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX3_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX3_HIT_CHECK::processEventCallback(&self->BOX3m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->LOADm_NEWm_WORLD.box3_in), self->BOX3m_HITm_CHECK.deleted_out);
                                                    if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "box3_in", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->BOX3m_SWITCH.whichChoice), self->BOX3m_HITm_CHECK.choice_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX3m_MOVEm_ON.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX3_MOVE_ON::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX3_MOVE_ON::processEventCallback(&self->BOX3m_MOVEm_ON, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX3m_HITm_CHECK.box_position, self->BOX3m_MOVEm_ON.translation_out);
                                                    if (X3dScript_BOX3_HIT_CHECK::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_BOX3_HIT_CHECK::processEventCallback(&self->BOX3m_HITm_CHECK, "box_position", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX3m_MOVE.translation, self->BOX3m_MOVEm_ON.translation_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX4m_HITm_CHECK.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX4_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX4_HIT_CHECK::processEventCallback(&self->BOX4m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->LOADm_NEWm_WORLD.box4_in), self->BOX4m_HITm_CHECK.deleted_out);
                                                    if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "box4_in", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->BOX4m_SWITCH.whichChoice), self->BOX4m_HITm_CHECK.choice_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX4m_MOVEm_ON.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX4_MOVE_ON::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX4_MOVE_ON::processEventCallback(&self->BOX4m_MOVEm_ON, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX4m_HITm_CHECK.box_position, self->BOX4m_MOVEm_ON.translation_out);
                                                    if (X3dScript_BOX4_HIT_CHECK::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_BOX4_HIT_CHECK::processEventCallback(&self->BOX4m_HITm_CHECK, "box_position", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX4m_MOVE.translation, self->BOX4m_MOVEm_ON.translation_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX5m_HITm_CHECK.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX5_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX5_HIT_CHECK::processEventCallback(&self->BOX5m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->LOADm_NEWm_WORLD.box5_in), self->BOX5m_HITm_CHECK.deleted_out);
                                                    if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "box5_in", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFInt32SendEvent(&(self->BOX5m_SWITCH.whichChoice), self->BOX5m_HITm_CHECK.choice_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX5m_MOVEm_ON.translation_in, self->FLYm_ON.top_translation_out);
                                            if (X3dScript_BOX5_MOVE_ON::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX5_MOVE_ON::processEventCallback(&self->BOX5m_MOVEm_ON, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX5m_HITm_CHECK.box_position, self->BOX5m_MOVEm_ON.translation_out);
                                                    if (X3dScript_BOX5_HIT_CHECK::processEventCallback != NULL) {
                                                        nextEvent = X3dScript_BOX5_HIT_CHECK::processEventCallback(&self->BOX5m_HITm_CHECK, "box_position", data);
                                                        if (nextEvent) {
                                                        }
                                                    }
                                                }
                                                {
                                                    X3dSFVec3fSendEvent(self->BOX5m_MOVE.translation, self->BOX5m_MOVEm_ON.translation_out);
                                                }
                                            }
                                        }
                                        {
                                            X3dSFInt32SendEvent(&(self->BULLETm_SWITCH.whichChoice), self->FLYm_ON.bullet_choice_out);
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX1m_HITm_CHECK.translation_in, self->FLYm_ON.bullet_translation_out);
                                            if (X3dScript_BOX1_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX1_HIT_CHECK::processEventCallback(&self->BOX1m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BULLETm_MOVE.translation, self->FLYm_ON.bullet_translation_out);
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX2m_HITm_CHECK.translation_in, self->FLYm_ON.bullet_translation_out);
                                            if (X3dScript_BOX2_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX2_HIT_CHECK::processEventCallback(&self->BOX2m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX3m_HITm_CHECK.translation_in, self->FLYm_ON.bullet_translation_out);
                                            if (X3dScript_BOX3_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX3_HIT_CHECK::processEventCallback(&self->BOX3m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX4m_HITm_CHECK.translation_in, self->FLYm_ON.bullet_translation_out);
                                            if (X3dScript_BOX4_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX4_HIT_CHECK::processEventCallback(&self->BOX4m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                            }
                                        }
                                        {
                                            X3dSFVec3fSendEvent(self->BOX5m_HITm_CHECK.translation_in, self->FLYm_ON.bullet_translation_out);
                                            if (X3dScript_BOX5_HIT_CHECK::processEventCallback != NULL) {
                                                nextEvent = X3dScript_BOX5_HIT_CHECK::processEventCallback(&self->BOX5m_HITm_CHECK, "translation_in", data);
                                                if (nextEvent) {
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                {
                    X3dSFFloatSendEvent(&(self->UPm_DOWNm_TURN.yvalue_in), self->STEER.yvalue);
                    if (X3dScript_UP_DOWN_TURN::processEventCallback != NULL) {
                        nextEvent = X3dScript_UP_DOWN_TURN::processEventCallback(&self->UPm_DOWNm_TURN, "yvalue_in", data);
                        if (nextEvent) {
                        }
                        {
                            X3dSFRotationSendEvent(self->UPm_DOWNm_AXIS.rotation, self->UPm_DOWNm_TURN.rotation_out);
                        }
                        {
                            X3dSFRotationSendEvent(self->ROTm_TRANS1.rotation_in, self->UPm_DOWNm_TURN.rotation_out);
                            if (X3dScript_ROT_TRANS1::processEventCallback != NULL) {
                                nextEvent = X3dScript_ROT_TRANS1::processEventCallback(&self->ROTm_TRANS1, "rotation_in", data);
                                if (nextEvent) {
                                }
                                {
                                    X3dSFVec3fSendEvent(self->ROTm_TRANS2.translation_in, self->ROTm_TRANS1.translation_out);
                                    if (X3dScript_ROT_TRANS2::processEventCallback != NULL) {
                                        nextEvent = X3dScript_ROT_TRANS2::processEventCallback(&self->ROTm_TRANS2, "translation_in", data);
                                        if (nextEvent) {
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->WALLm_SENSOR2, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->WALLm_FIRE.isactive_in), self->WALLm_SENSOR2.isActive);
            if (X3dScript_WALL_FIRE::processEventCallback != NULL) {
                nextEvent = X3dScript_WALL_FIRE::processEventCallback(&self->WALLm_FIRE, "isactive_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFBoolSendEvent(&(self->FLYm_ON.fire_in), self->WALLm_FIRE.fire);
                    if (X3dScript_FLY_ON::processEventCallback != NULL) {
                        nextEvent = X3dScript_FLY_ON::processEventCallback(&self->FLYm_ON, "fire_in", data);
                        if (nextEvent) {
                        }
                    }
                }
                {
                    X3dSFBoolSendEvent(&(self->THINGm_SCRIPT.fire_in), self->WALLm_FIRE.fire);
                    if (X3dScript_THING_SCRIPT::processEventCallback != NULL) {
                        nextEvent = X3dScript_THING_SCRIPT::processEventCallback(&self->THINGm_SCRIPT, "fire_in", data);
                        if (nextEvent) {
                        }
                        {
                            X3dSFVec3fSendEvent(self->HIDEm_THING.scale, self->THINGm_SCRIPT.hide_out);
                        }
                    }
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->THINGm_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->THINGm_SCRIPT.isover_in), self->THINGm_SENSOR.isOver);
            if (X3dScript_THING_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_THING_SCRIPT::processEventCallback(&self->THINGm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
            }
        }
    }
    if (X3dTimeSensor::processEventCallback != NULL) {
        nextEvent = X3dTimeSensor::processEventCallback(&self->TIMER, "", data);
        if (nextEvent) {
        }
        {
            X3dSFTimeSendEvent(&(self->FLYm_FORWARD.time_in), self->TIMER.time);
            if (X3dScript_FLY_FORWARD::processEventCallback != NULL) {
                nextEvent = X3dScript_FLY_FORWARD::processEventCallback(&self->FLYm_FORWARD, "time_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->ROTm_TRANS1.translation_in, self->FLYm_FORWARD.translation_out);
                    if (X3dScript_ROT_TRANS1::processEventCallback != NULL) {
                        nextEvent = X3dScript_ROT_TRANS1::processEventCallback(&self->ROTm_TRANS1, "translation_in", data);
                        if (nextEvent) {
                        }
                    }
                }
                {
                    X3dSFTimeSendEvent(&(self->FLYm_ON.time_in), self->FLYm_FORWARD.time_out);
                    if (X3dScript_FLY_ON::processEventCallback != NULL) {
                        nextEvent = X3dScript_FLY_ON::processEventCallback(&self->FLYm_ON, "time_in", data);
                        if (nextEvent) {
                        }
                    }
                }
            }
        }
        {
            X3dSFTimeSendEvent(&(self->LOADm_NEWm_WORLD.starttime_in), self->TIMER.time);
            if (X3dScript_LOAD_NEW_WORLD::processEventCallback != NULL) {
                nextEvent = X3dScript_LOAD_NEW_WORLD::processEventCallback(&self->LOADm_NEWm_WORLD, "starttime_in", data);
                if (nextEvent) {
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX2m_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->BOX2m_HIDEm_SCRIPT.isover_in), self->BOX2m_SENSOR.isOver);
            if (X3dScript_BOX2_HIDE_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_BOX2_HIDE_SCRIPT::processEventCallback(&self->BOX2m_HIDEm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->BOX2m_HIDE.translation, self->BOX2m_HIDEm_SCRIPT.hide_out);
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX3m_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->BOX3m_HIDEm_SCRIPT.isover_in), self->BOX3m_SENSOR.isOver);
            if (X3dScript_BOX3_HIDE_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_BOX3_HIDE_SCRIPT::processEventCallback(&self->BOX3m_HIDEm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->BOX3m_HIDE.translation, self->BOX3m_HIDEm_SCRIPT.hide_out);
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX4m_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->BOX4m_HIDEm_SCRIPT.isover_in), self->BOX4m_SENSOR.isOver);
            if (X3dScript_BOX4_HIDE_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_BOX4_HIDE_SCRIPT::processEventCallback(&self->BOX4m_HIDEm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->BOX4m_HIDE.translation, self->BOX4m_HIDEm_SCRIPT.hide_out);
                }
            }
        }
    }
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX5m_SENSOR, "", data);
        if (nextEvent) {
        }
        {
            X3dSFBoolSendEvent(&(self->BOX5m_HIDEm_SCRIPT.isover_in), self->BOX5m_SENSOR.isOver);
            if (X3dScript_BOX5_HIDE_SCRIPT::processEventCallback != NULL) {
                nextEvent = X3dScript_BOX5_HIDE_SCRIPT::processEventCallback(&self->BOX5m_HIDEm_SCRIPT, "isover_in", data);
                if (nextEvent) {
                }
                {
                    X3dSFVec3fSendEvent(self->BOX5m_HIDE.translation, self->BOX5m_HIDEm_SCRIPT.hide_out);
                }
            }
        }
    }
    reInitSensor(&(self->BOX1m_SENSOR));
    reInitSensor(&(self->WALLm_SENSOR));
    reInitSensor(&(self->WALLm_SENSOR2));
    reInitSensor(&(self->THINGm_SENSOR));
    reInitSensor(&(self->TIMER));
    reInitSensor(&(self->BOX2m_SENSOR));
    reInitSensor(&(self->BOX3m_SENSOR));
    reInitSensor(&(self->BOX4m_SENSOR));
    reInitSensor(&(self->BOX5m_SENSOR));
}
